/*	$Id: cpm.c,v 1.13 2010/01/12 08:05:41 tsubai Exp $	*/

/*-
 * Copyright (c) 2001,2002 Tsubai Masanari.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/param.h>
#include <sys/device.h>
#include <sys/systm.h>
#include <machine/autoconf.h>

struct cpm_softc {
	struct device sc_dev;
};

int cpm_match(struct device *, struct cfdata *, void *);
void cpm_attach(struct device *, struct device *, void *);
int cpm_search(struct device *, struct cfdata *, const int *, void *);
int cpm_print(void *, const char *);

CFATTACH_DECL0(cpm, sizeof(struct cpm_softc), cpm_match, cpm_attach);

int
cpm_match(parent, cf, aux)
	struct device *parent;
	struct cfdata *cf;
	void *aux;
{
	struct confargs *ca = aux;

	if (strcmp(ca->ca_name, "cpm") == 0)
		return 1;

	return 0;
}

void
cpm_attach(parent, self, aux)
	struct device *parent, *self;
	void *aux;
{
	struct confargs ca;

	bzero(&ca, sizeof ca);
	printf("\n");

	config_search_ia(cpm_search, self, "cpm", &ca);
}

int
cpm_search(parent, cf, ld, aux)
	struct device *parent;
	struct cfdata *cf;
	const int *ld;
	void *aux;
{
	struct confargs *ca = aux;

	ca->ca_name = cf->cf_name;
	ca->ca_vec[0] = cf->cf_vec;
	if (config_match(parent, cf, ca) > 0)
		config_attach(parent, cf, ca, cpm_print);

	return 0;
}

int
cpm_print(aux, bus)
	void *aux;
	const char *bus;
{
	struct confargs *ca = aux;

	if (bus) {
		/* return QUIET; */
		printf("%s at %s", ca->ca_name, bus);
	}
	if (ca->ca_vec[0] > 0)
		printf(" vec %d", ca->ca_vec[0]);
	return UNCONF;
}
