/*	$Id: dpram.c,v 1.2 2012/02/29 08:15:33 tsubai Exp $	*/

/*-
 * Copyright (c) 2010 Tsubai Masanari.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/param.h>
#include <sys/extent.h>
#include <sys/malloc.h>
#include <sys/systm.h>

void dpram_init(void);
int dpram_alloc(u_int, u_int);
int dpram_reserve(u_int, u_int);
void dpram_free(u_int, u_int);

static struct extent *dpram_ex;

void
dpram_init()
{
	dpram_ex = extent_create("dpram", 0, 0xfff,
	    NULL, 0, EX_NOWAIT | EX_NOCOALESCE);
}

int
dpram_alloc(size, align)
	u_int size, align;
{
	u_long result;
	int error;

	if (dpram_ex == NULL)
		dpram_init();
	error = extent_alloc(dpram_ex, size, align, 0, EX_NOWAIT, &result);
	if (error)
		return -1;

	return result;
}

int
dpram_reserve(start, size)
	u_int start, size;
{
	if (dpram_ex == NULL)
		dpram_init();
	return extent_alloc_region(dpram_ex, start, size, EX_NOWAIT);
}

void
dpram_free(start, size)
	u_int start, size;
{
	extent_free(dpram_ex, start, size, 0);
}
