/*	$Id: rtc.c,v 1.1 2010/01/11 07:21:44 tsubai Exp $	*/

/*-
 * Copyright (c) 2009 Tsubai Masanari.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/param.h>
#include <dev/clock_subr.h>
#include <machine/mpc860dev.h>

static int rtc_gettime(struct todr_chip_handle *, struct timeval *);
static int rtc_settime(struct todr_chip_handle *, struct timeval *);

void
rtc_attach()
{
	static struct todr_chip_handle rtc;

	rtc.todr_gettime = rtc_gettime;
	rtc.todr_settime = rtc_settime;
	todr_attach(&rtc);
}

int
rtc_gettime(ch, t)
	struct todr_chip_handle *ch;
	struct timeval *t;
{
	volatile struct mpc860dev *immr = get_immr();
	time_t delta;

	t->tv_sec = immr->siu_rtc;
	immr->siu_rtck = 0;		/* lock */

	if (ch->base_time) {
		delta = t->tv_sec - ch->base_time;
		if (delta < 0)
			delta = -delta;
		if (delta > SECYR*5)
			return EINVAL;
	}
	return 0;
}

int
rtc_settime(ch, t)
	struct todr_chip_handle *ch;
	struct timeval *t;
{
	volatile struct mpc860dev *immr = get_immr();

	immr->siu_rtck = 0x55ccaa33;	/* unlock */
	immr->siu_rtc = t->tv_sec;
	immr->siu_rtck = 0;		/* lock again */
	return 0;
}
