/*	$Id: bus.h,v 1.6 2012/06/07 11:41:00 tsubai Exp $	*/

/*-
 * Copyright (c) 2002 Tsubai Masanari.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _MACHINE_BUS_H_
#define _MACHINE_BUS_H_

#define __BUS_SPACE_HAS_STREAM_METHODS

typedef u_long bus_addr_t;
typedef u_long bus_size_t;

typedef u_int bus_space_tag_t;
typedef u_long bus_space_handle_t;

/* bus_space_map flags */
#define BUS_SPACE_MAP_CACHEABLE		0x01
#define BUS_SPACE_MAP_LINEAR		0x02
#define BUS_SPACE_MAP_PREFETCHABLE	0x04

/* bus_space_barrier flags */
#define BUS_SPACE_BARRIER_READ	0x01		/* force read barrier */
#define BUS_SPACE_BARRIER_WRITE	0x02		/* force write barrier */

#define BUS_SPACE_ALIGNED_POINTER(p, t) ALIGNED_POINTER(p, t)

#define bus_space_read_stream_1(t, h, off) \
	bus_space_read_1(t, h, off)

#define bus_space_read_multi_stream_1(t, h, off, addr, count) \
	bus_space_read_multi_1(t, h, off, addr, count)

#define bus_space_read_region_stream_1(t, h, off, addr, count) \
	bus_space_read_region_1(t, h, off, addr, count)

#define bus_space_write_stream_1(t, h, off, val) \
	bus_space_write_1(t, h, off, val)

#define bus_space_write_multi_stream_1(t, h, off, addr, count) \
	bus_space_write_multi_1(t, h, off, addr, count)

#define bus_space_write_region_stream_1(t, h, off, addr, count) \
	bus_space_write_region_1(t, h, off, addr, count)

#define bus_space_subregion(t, bsh, off, size, bshp) \
    ((*(bshp) = (bsh) + (off)), 0)
#define bus_space_barrier(t, h, off, len, flags) \
    ((void)((t) + (h) + (off) + (len) + (flags)))

int bus_space_map(bus_space_tag_t, bus_addr_t, bus_size_t, int,
    bus_space_handle_t *);
void bus_space_unmap(bus_space_tag_t, bus_space_handle_t, bus_size_t);
paddr_t bus_space_mmap(bus_space_tag_t, bus_addr_t, off_t, int, int);
u_int8_t bus_space_read_1(bus_space_tag_t, bus_space_handle_t, bus_size_t);
u_int16_t bus_space_read_2(bus_space_tag_t, bus_space_handle_t, bus_size_t);
u_int32_t bus_space_read_4(bus_space_tag_t, bus_space_handle_t, bus_size_t);
void bus_space_write_1(bus_space_tag_t, bus_space_handle_t, bus_size_t,
    u_int8_t);
void bus_space_write_2(bus_space_tag_t, bus_space_handle_t, bus_size_t,
    u_int16_t);
void bus_space_write_4(bus_space_tag_t, bus_space_handle_t, bus_size_t,
    u_int32_t);
void bus_space_read_region_1(bus_space_tag_t, bus_space_handle_t, bus_size_t,
    u_int8_t *, bus_size_t);
void bus_space_read_region_2(bus_space_tag_t, bus_space_handle_t, bus_size_t,
    u_int16_t *, bus_size_t);
void bus_space_read_region_4(bus_space_tag_t, bus_space_handle_t, bus_size_t,
    u_int32_t *, bus_size_t);
void bus_space_write_region_1(bus_space_tag_t, bus_space_handle_t, bus_size_t,
    const u_int8_t *, bus_size_t);
void bus_space_write_region_2(bus_space_tag_t, bus_space_handle_t, bus_size_t,
    const u_int16_t *, bus_size_t);
void bus_space_write_region_4(bus_space_tag_t, bus_space_handle_t, bus_size_t,
    const u_int32_t *, bus_size_t);
void bus_space_set_region_1(bus_space_tag_t, bus_space_handle_t, bus_size_t,
    u_int8_t, bus_size_t);
void bus_space_set_region_2(bus_space_tag_t, bus_space_handle_t, bus_size_t,
    u_int16_t, bus_size_t);
void bus_space_set_region_4(bus_space_tag_t, bus_space_handle_t, bus_size_t,
    u_int32_t, bus_size_t);
void bus_space_read_multi_1(bus_space_tag_t, bus_space_handle_t, bus_size_t,
    u_int8_t *, bus_size_t);
void bus_space_read_multi_2(bus_space_tag_t, bus_space_handle_t, bus_size_t,
    u_int16_t *, bus_size_t);
void bus_space_read_multi_4(bus_space_tag_t, bus_space_handle_t, bus_size_t,
    u_int32_t *, bus_size_t);
void bus_space_write_multi_1(bus_space_tag_t, bus_space_handle_t, bus_size_t,
    const u_int8_t *, bus_size_t);
void bus_space_write_multi_2(bus_space_tag_t, bus_space_handle_t, bus_size_t,
    const u_int16_t *, bus_size_t);
void bus_space_write_multi_4(bus_space_tag_t, bus_space_handle_t, bus_size_t,
    const u_int32_t *, bus_size_t);
u_int8_t bus_space_read_stream_1(bus_space_tag_t, bus_space_handle_t,
    bus_size_t);
u_int16_t bus_space_read_stream_2(bus_space_tag_t, bus_space_handle_t,
    bus_size_t);
u_int32_t bus_space_read_stream_4(bus_space_tag_t, bus_space_handle_t,
    bus_size_t);
void bus_space_read_multi_stream_1(bus_space_tag_t, bus_space_handle_t,
    bus_size_t, u_int8_t *, bus_size_t);
void bus_space_read_multi_stream_2(bus_space_tag_t, bus_space_handle_t,
    bus_size_t, u_int16_t *, bus_size_t);
void bus_space_read_multi_stream_4(bus_space_tag_t, bus_space_handle_t,
    bus_size_t, u_int32_t *, bus_size_t);
void bus_space_read_region_stream_1(bus_space_tag_t, bus_space_handle_t,
    bus_size_t, u_int8_t *, bus_size_t);
void bus_space_read_region_stream_2(bus_space_tag_t, bus_space_handle_t,
    bus_size_t, u_int16_t *, bus_size_t);
void bus_space_read_region_stream_4(bus_space_tag_t, bus_space_handle_t,
    bus_size_t, u_int32_t *, bus_size_t);
void bus_space_write_stream_1(bus_space_tag_t, bus_space_handle_t, bus_size_t,
    u_int8_t);
void bus_space_write_stream_2(bus_space_tag_t, bus_space_handle_t, bus_size_t,
    u_int16_t);
void bus_space_write_stream_4(bus_space_tag_t, bus_space_handle_t, bus_size_t,
    u_int32_t);
void bus_space_write_multi_stream_1(bus_space_tag_t, bus_space_handle_t,
    bus_size_t, const u_int8_t *, bus_size_t);
void bus_space_write_multi_stream_2(bus_space_tag_t, bus_space_handle_t,
    bus_size_t, const u_int16_t *, bus_size_t);
void bus_space_write_multi_stream_4(bus_space_tag_t, bus_space_handle_t,
    bus_size_t, const u_int32_t *, bus_size_t);
void bus_space_write_region_stream_1(bus_space_tag_t, bus_space_handle_t,
    bus_size_t, const u_int8_t *, bus_size_t);
void bus_space_write_region_stream_2(bus_space_tag_t, bus_space_handle_t,
    bus_size_t, const u_int16_t *, bus_size_t);
void bus_space_write_region_stream_4(bus_space_tag_t, bus_space_handle_t,
    bus_size_t, const u_int32_t *, bus_size_t);

/*
 * Bus DMA methods.
 */

/*
 * Flags used in various bus DMA methods.
 */
#define BUS_DMA_WAITOK		0x000	/* safe to sleep (pseudo-flag) */
#define BUS_DMA_NOWAIT		0x001	/* not safe to sleep */
#define BUS_DMA_ALLOCNOW	0x002	/* perform resource allocation now */
#define BUS_DMA_COHERENT	0x004	/* hint: map memory DMA coherent */
#define BUS_DMA_STREAMING	0x008	/* hint: sequential, unidirectional */
#define BUS_DMA_READ		0x100	/* mapping is device -> memory only */
#define BUS_DMA_WRITE		0x200	/* mapping is memory -> device only */
#define BUS_DMA_NOCACHE		0x400	/* hint: map non-cached memory */

/* Forwards needed by prototypes below. */
struct mbuf;
struct uio;

/*
 * Operations performed by bus_dmamap_sync().
 */
#define BUS_DMASYNC_PREREAD	0x01	/* pre-read synchronization */
#define BUS_DMASYNC_POSTREAD	0x02	/* post-read synchronization */
#define BUS_DMASYNC_PREWRITE	0x04	/* pre-write synchronization */
#define BUS_DMASYNC_POSTWRITE	0x08	/* post-write synchronization */

typedef int bus_dma_tag_t;

#define bus_dmatag_subregion(t, mna, mxa, nt, f) EOPNOTSUPP
#define bus_dmatag_destroy(t)
#define BUS_DMA_TAG_VALID(t) (t)

/*
 *	bus_dma_segment_t
 *
 *	Describes a single contiguous DMA transaction.  Values
 *	are suitable for programming into DMA registers.
 */
typedef struct {
	bus_addr_t ds_addr;		/* DMA address */
	bus_size_t ds_len;		/* length of transfer */
} bus_dma_segment_t;

/*
 *	bus_dmamap_t
 *
 *	Describes a DMA mapping.
 */
typedef struct {
	/*
	 * PRIVATE MEMBERS: not for use by machine-independent code.
	 */
	bus_size_t	_dm_size;	/* largest DMA transfer mappable */
	int		_dm_segcnt;	/* number of segs this map can map */
	bus_size_t	_dm_boundary;	/* don't cross this */
	int		_dm_flags;	/* misc. flags */

	/*
	 * PUBLIC MEMBERS: these are used by machine-independent code.
	 */
	bus_size_t	dm_maxsegsz;	/* largest possible segment */
	bus_size_t	dm_mapsize;	/* size of the mapping */
	int		dm_nsegs;	/* # valid segments in mapping */
	bus_dma_segment_t dm_segs[1];	/* segments; variable length */
} *bus_dmamap_t;

int bus_dmamap_create(bus_dma_tag_t, bus_size_t, int, bus_size_t, bus_size_t,
	int, bus_dmamap_t *);
void bus_dmamap_destroy(bus_dma_tag_t, bus_dmamap_t);
int bus_dmamap_load(bus_dma_tag_t, bus_dmamap_t, void *, bus_size_t,
	struct proc *, int);
int bus_dmamap_load_mbuf(bus_dma_tag_t, bus_dmamap_t, struct mbuf *, int);
int bus_dmamap_load_uio(bus_dma_tag_t, bus_dmamap_t, struct uio *, int);
int bus_dmamap_load_raw(bus_dma_tag_t, bus_dmamap_t, bus_dma_segment_t *, int,
	bus_size_t, int);
void bus_dmamap_unload(bus_dma_tag_t, bus_dmamap_t);
void bus_dmamap_sync(bus_dma_tag_t, bus_dmamap_t, bus_addr_t, bus_size_t, int);

int bus_dmamem_alloc(bus_dma_tag_t, bus_size_t, bus_size_t, bus_size_t,
	bus_dma_segment_t *, int, int *, int);
void bus_dmamem_free(bus_dma_tag_t, bus_dma_segment_t *, int);
int bus_dmamem_map(bus_dma_tag_t, bus_dma_segment_t *, int, size_t, void **,
	int);
void bus_dmamem_unmap(bus_dma_tag_t, void *, size_t);
paddr_t bus_dmamem_mmap(bus_dma_tag_t, bus_dma_segment_t *, int, off_t, int,
	int);

#endif /* _MACHINE_BUS_H_ */
