/*	$NetBSD: intr.h,v 1.10 2001/01/14 02:00:40 thorpej Exp $	*/

/*-
 * Copyright (c) 1998 The NetBSD Foundation, Inc.
 * All rights reserved.
 *
 * This code is derived from software contributed to The NetBSD Foundation
 * by Charles M. Hannum.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _MPC860_INTR_H_
#define _MPC860_INTR_H_

/* Interrupt priority `levels'. */
#define IPL_NONE	0	/* nothing */
#define IPL_SOFTCLOCK	1	/* timeouts */
#define IPL_SOFTBIO	2
#define IPL_SOFTNET	3	/* protocol stacks */
#define IPL_SOFTSERIAL	4	/* serial */
#define IPL_VM		5	/* memory allocation */
#define IPL_SCHED	6
#define IPL_HIGH	7	/* everything */
#define NIPL		8

/* Interrupt sharing types. */
#define IST_NONE	0	/* none */
#define IST_PULSE	1	/* pulsed */
#define IST_EDGE	2	/* edge-triggered */
#define IST_LEVEL	3	/* level-triggered */

#ifndef _LOCORE

int splraise(int);
int spllower(int);
void splx(int);
void softintr(int);
void *softintr_establish(int, void (*)(void *), void *);
void softintr_disestablish(void *);
void softintr_schedule(void *);
void intr_calculatemasks(void);

extern int imask[];

#define ICU_LEN		16

/* Soft interrupt masks. */
#define SIR_CLOCK	1
#define SIR_BIO		2
#define SIR_NET		3
#define SIR_SERIAL	4
#define SPL_CLOCK	0

/*
 * Software interrupt masks
 */
#define splsoftclock()	splraise(imask[IPL_SOFTCLOCK])
#define splsoftbio()	splraise(imask[IPL_SOFTBIO])
#define splsoftnet()	splraise(imask[IPL_SOFTNET])
#define splsoftserial()	splraise(imask[IPL_SOFTSERIAL])

/*
 * Miscellaneous
 */
#define splvm()		splraise(imask[IPL_VM])
#define splhigh()	splraise(imask[IPL_HIGH])
#define splsched()	splhigh()
#define spl0()		spllower(imask[IPL_NONE])
#define splraiseipl(x)	splraise(imask[x])

#define setsoftclock()	softintr(SIR_CLOCK)
#define setsoftbio()	softintr(SIR_BIO)
#define setsoftnet()	softintr(SIR_NET)
#define setsoftserial()	softintr(SIR_SERIAL)

#define makeiplcookie(ipl) (ipl)
typedef int ipl_cookie_t;

extern long intrcnt[];

#define CNT_IRQ0	0
#define CNT_CLOCK	16

#endif /* !_LOCORE */

#endif /* !_MPC860_INTR_H_ */
