/*	$Id: mpc860dev.h,v 1.18 2015/06/25 12:38:05 tsubai Exp $	*/

/*-
 * Copyright (c) 2001,2002,2004 Tsubai Masanari.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* MPC860 internal memory mapped devices */

struct mpc860dev {

	/* 0x0000 General System Interface Unit */
	u_int32_t siu_siumcr;	/* SIU module configuration register */
	u_int32_t siu_sypcr;	/* System protection control register */
	u_int8_t  siu_res00[6];
	u_int16_t siu_swsr;	/* Software service register */
	u_int32_t siu_sipend;	/* SIU interrupt pending register */
	u_int32_t siu_simask;	/* SIU interrupt mask register */
	u_int32_t siu_siel;	/* SIU interrupt edge/level register */
	u_int32_t siu_sivec;	/* SIU interrupt vector register */
	u_int32_t siu_tesr;	/* Transfer error status register */
	u_int8_t  siu_res01[12];
	u_int32_t siu_sdcr;	/* SDMA configuration register */
	u_int8_t  siu_res02[76];

	/* 0x0080 PCMCIA */
	u_int32_t siu_pbr0;	/* PCMCIA base register 0 */
	u_int32_t siu_por0;	/* PCMCIA option register 0 */
	u_int32_t siu_pbr1;	/* PCMCIA base register 1 */
	u_int32_t siu_por1;	/* PCMCIA option register 1 */
	u_int32_t siu_pbr2;	/* PCMCIA base register 2 */
	u_int32_t siu_por2;	/* PCMCIA option register 2 */
	u_int32_t siu_pbr3;	/* PCMCIA base register 3 */
	u_int32_t siu_por3;	/* PCMCIA option register 3 */
	u_int32_t siu_pbr4;	/* PCMCIA base register 4 */
	u_int32_t siu_por4;	/* PCMCIA option register 4 */
	u_int32_t siu_pbr5;	/* PCMCIA base register 5 */
	u_int32_t siu_por5;	/* PCMCIA option register 5 */
	u_int32_t siu_pbr6;	/* PCMCIA base register 6 */
	u_int32_t siu_por6;	/* PCMCIA option register 6 */
	u_int32_t siu_pbr7;	/* PCMCIA base register 7 */
	u_int32_t siu_por7;	/* PCMCIA option register 7 */
	u_int8_t  siu_res03[32];
	u_int32_t siu_pgcra;	/* PCMCIA general control register A */
	u_int32_t siu_pgcrb;	/* PCMCIA general control register B */
	u_int32_t siu_pscr;	/* PCMCIA status changed register */
	u_int8_t  siu_res04[4];
	u_int32_t siu_pipr;	/* PCMCIA input pins register */
	u_int8_t  siu_res05[4];
	u_int32_t siu_per;	/* PCMCIA enable register */
	u_int8_t  siu_res06[4];

	/* 0x0100 Memory Controller */
	u_int32_t siu_br0;	/* Base register bank 0 */
	u_int32_t siu_or0;	/* Option register bank 0 */
	u_int32_t siu_br1;	/* Base register bank 1 */
	u_int32_t siu_or1;	/* Option register bank 1 */
	u_int32_t siu_br2;	/* Base register bank 2 */
	u_int32_t siu_or2;	/* Option register bank 2 */
	u_int32_t siu_br3;	/* Base register bank 3 */
	u_int32_t siu_or3;	/* Option register bank 3 */
	u_int32_t siu_br4;	/* Base register bank 4 */
	u_int32_t siu_or4;	/* Option register bank 4 */
	u_int32_t siu_br5;	/* Base register bank 5 */
	u_int32_t siu_or5;	/* Option register bank 5 */
	u_int32_t siu_br6;	/* Base register bank 6 */
	u_int32_t siu_or6;	/* Option register bank 6 */
	u_int32_t siu_br7;	/* Base register bank 7 */
	u_int32_t siu_or7;	/* Option register bank 7 */
	u_int8_t  siu_res07[36];
	u_int32_t siu_mar;	/* Memory address register */
	u_int32_t siu_mcr;	/* Memory command register */
	u_int8_t  siu_res08[4];
	u_int32_t siu_mamr;	/* Machine A mode register */
	u_int32_t siu_mbmr;	/* Machine B mode register */
	u_int16_t siu_mstat;	/* Memory status register */
	u_int16_t siu_mptpr;	/* Memory periodic timer prescaler */
	u_int32_t siu_mdr;	/* Memory data register */
	u_int8_t  siu_res09[128];

	/* 0x0200 System Integration Timers */
	u_int16_t siu_tbscr;	/* Timebase status and control register */
	u_int8_t  siu_res10[2];
	u_int32_t siu_tbrefa;	/* Timebase reference register A */
	u_int32_t siu_tbrefl;	/* Timebase reference register B */
	u_int8_t  siu_res11[20];
	u_int16_t siu_rtcsc;	/* Real-time clock csr */
	u_int8_t  siu_res12[2];
	u_int32_t siu_rtc;	/* Real-time clock register */
	u_int32_t siu_rtsec;	/* Real-time alarm seconds */
	u_int32_t siu_rtcal;	/* Real-time alarm register */
	u_int8_t  siu_res13[16];
	u_int16_t siu_piscr;	/* Periodic interrupt csr */
	u_int8_t  siu_res14[2];
	u_int32_t siu_pitc;	/* Periodic interrupt count register */
	u_int32_t siu_pitr;	/* Periodic interrupt timer register */
	u_int8_t  siu_res15[52];

	/* 0x0280 Clocks and Reset */
	u_int32_t siu_sccr;	/* System clock control register */
	u_int32_t siu_plprcr;	/* PLL, low-power, and reset control register */
	u_int32_t siu_rsr;	/* Reset status register */
	u_int8_t  siu_res16[116];

	/* 0x0300 System Integration Timers Key */
	u_int32_t siu_tbscrk;	/* Timebase status and control register key */
	u_int32_t siu_tbrefak;	/* Timebase reference register A key */
	u_int32_t siu_tbrefbk;	/* Timebase reference register B key */
	u_int32_t siu_tbk;	/* Timebase/decrementer register key */
	u_int8_t  siu_res17[16];
	u_int32_t siu_rtcsck;	/* Real-time clock csr key */
	u_int32_t siu_rtck;	/* Real-time clock register key */
	u_int32_t siu_rtseck;	/* Real-time alarm seconds key */
	u_int32_t siu_rtcalk;	/* Real-time alarm register key */
	u_int8_t  siu_res18[16];
	u_int32_t siu_piscrk;	/* Periodic interrupt csr key */
	u_int32_t siu_pitck;	/* Periodic interrupt count register key */
	u_int8_t  siu_res19[56];

	/* 0x0380 Clocks and Reset Keys */
	u_int32_t siu_sccrk;	/* System clock control key */
	u_int32_t siu_plprcrk;	/* PLL, low power and reset control reg key */
	u_int32_t siu_rsrk;	/* Reset status register key */
	u_int8_t  siu_res20[1236];

	/* 0x0860 I2C Controller */
	u_int8_t  cpm_i2mod;	/* I2C mode register */
	u_int8_t  cpm_res21[3];
	u_int8_t  cpm_i2add;	/* I2C address register */
	u_int8_t  cpm_res22[3];
	u_int8_t  cpm_i2brg;	/* I2C BRG register */
	u_int8_t  cpm_res23[3];
	u_int8_t  cpm_i2com;	/* I2C command register */
	u_int8_t  cpm_res24[3];
	u_int8_t  cpm_i2cer;	/* I2C event register */
	u_int8_t  cpm_res25[3];
	u_int8_t  cpm_i2cmr;	/* I2C mask register */
	u_int8_t  cpm_res26[139];

	/* 0x0900 DMA */
	u_int8_t  cpm_res27[4];
	u_int32_t cpm_sdar;	/* SDMA address register */
	u_int8_t  cpm_sdsr;	/* SDMA status register */
	u_int8_t  cpm_res28[3];
	u_int8_t  cpm_sdmr;	/* SDMA mask register */
	u_int8_t  cpm_res29[3];
	u_int8_t  cpm_idsr1;	/* IDMA1 status register */
	u_int8_t  cpm_res30[3];
	u_int8_t  cpm_idmr1;	/* IDMA1 mask register */
	u_int8_t  cpm_res31[3];
	u_int8_t  cpm_idsr2;	/* IDMA2 status register */
	u_int8_t  cpm_res32[3];
	u_int8_t  cpm_idmr2;	/* IDMA2 mask register */
	u_int8_t  cpm_res33[19];

	/* 0x0930 CPM Interrupt Control */
	u_int16_t cpm_civr;	/* CPM interrupt vector register */
	u_int8_t  cpm_res34[14];
	u_int32_t cpm_cicr;	/* CPM interrupt configuration register */
	u_int32_t cpm_cipr;	/* CPM interrupt pending register */
	u_int32_t cpm_cimr;	/* CPM interrupt mask register */
	u_int32_t cpm_cisr;	/* CPM in-service register */

	/* 0x0950 Parallel I/Output Port */
	u_int16_t cpm_padir;	/* Port A data direction register */
	u_int16_t cpm_papar;	/* Port A pin assignment register */
	u_int16_t cpm_paodr;	/* Port A open drain register */
	u_int16_t cpm_padat;	/* Port A data register */
	u_int8_t  cpm_res35[8];
	u_int16_t cpm_pcdir;	/* Port C data direction register */
	u_int16_t cpm_pcpar;	/* Port C pin assignment register */
	u_int16_t cpm_pcso;	/* Port C special options register */
	u_int16_t cpm_pcdat;	/* Port C data register */
	u_int16_t cpm_pcint;	/* Port C interrupt control register */
	u_int8_t  cpm_res36[6];
	u_int16_t cpm_pddir;	/* Port D data direction register */
	u_int16_t cpm_pdpar;	/* Port D pin assignment register */
	u_int8_t  cpm_res37[2];
	u_int16_t cpm_pddat;	/* Port D data register */
	u_int8_t  cpm_res38[8];

	/* 0x0980 CPM General-Purpose Timers */
	u_int16_t cpm_tgcr;	/* Timer global configuration register */
	u_int8_t  cpm_res39[14];
	u_int16_t cpm_tmr1;	/* Timer 1 mode register */
	u_int16_t cpm_tmr2;	/* Timer 2 mode register */
	u_int16_t cpm_trr1;	/* Timer 1 reference register */
	u_int16_t cpm_trr2;	/* Timer 2 reference register */
	u_int16_t cpm_tcr1;	/* Timer 1 capture register */
	u_int16_t cpm_tcr2;	/* Timer 2 capture register */
	u_int16_t cpm_tcn1;	/* Timer 1 counter */
	u_int16_t cpm_tcn2;	/* Timer 2 counter */
	u_int16_t cpm_tmr3;	/* Timer 3 mode register */
	u_int16_t cpm_tmr4;	/* Timer 4 mode register */
	u_int16_t cpm_trr3;	/* Timer 3 reference register */
	u_int16_t cpm_trr4;	/* Timer 4 reference register */
	u_int16_t cpm_tcr3;	/* Timer 3 capture register */
	u_int16_t cpm_tcr4;	/* Timer 4 capture register */
	u_int16_t cpm_tcn3;	/* Timer 3 counter */
	u_int16_t cpm_tcn4;	/* Timer 4 counter */
	u_int16_t cpm_ter1;	/* Timer 1 event register */
	u_int16_t cpm_ter2;	/* Timer 2 event register */
	u_int16_t cpm_ter3;	/* Timer 3 event register */
	u_int16_t cpm_ter4;	/* Timer 4 event register */
	u_int8_t  cpm_res40[8];

	/* 0x09c0 Communications Processor */
	u_int16_t cpm_cpcr;	/* CP command register */
	u_int8_t  cpm_res41[2];
	u_int16_t cpm_rccr;	/* RISC controller configuration register */
	u_int8_t  cpm_res42[3];
	u_int16_t cpm_rctr1;	/* RISC controller trap register 1 */
	u_int16_t cpm_rctr2;	/* RISC controller trap register 2 */
	u_int16_t cpm_rctr3;	/* RISC controller trap register 3 */
	u_int16_t cpm_rctr4;	/* RISC controller trap register 4 */
	u_int8_t  cpm_res43[2];
	u_int16_t cpm_rter;	/* RISC timer event register */
	u_int8_t  cpm_res44[2];
	u_int16_t cpm_rtmr;	/* RISC timers mask register */
	u_int8_t  cpm_res45[20];

	/* 0x09f0 Baud Rate Generators */
	u_int32_t cpm_brgc1;	/* BRG1 configuration register */
	u_int32_t cpm_brgc2;	/* BRG2 configuration register */
	u_int32_t cpm_brgc3;	/* BRG3 configuration register */
	u_int32_t cpm_brgc4;	/* BRG4 configuration register */

	/* 0x0a00 SCC1 */
	u_int32_t cpm_gsmr_l1;	/* SCC1 general mode register */
	u_int32_t cpm_gsmr_h1;	/* SCC1 general mode register */
	u_int16_t cpm_psmr1;	/* SCC1 protocol specific mode register */
	u_int8_t  cpm_res47[2];
	u_int16_t cpm_todr1;	/* SCC1 transmit-on-demand register */
	u_int16_t cpm_dsr1;	/* SCC1 data synchronization register */
	u_int16_t cpm_scce1;	/* SCC1 event register */
	u_int8_t  cpm_res48[2];
	u_int16_t cpm_sccm1;	/* SCC1 mask register */
	u_int8_t  cpm_res49[1];
	u_int8_t  cpm_sccs1;	/* SCC1 status register */
	u_int8_t  cpm_res50[8];

	/* 0x0a20 SCC2 */
	u_int32_t cpm_gsmr_l2;	/* SCC2 general mode register */
	u_int32_t cpm_gsmr_h2;	/* SCC2 general mode register */
	u_int16_t cpm_psmr2;	/* SCC2 protocol-specific mode register */
	u_int8_t  cpm_res51[2];
	u_int16_t cpm_todr2;	/* SCC2 transmit on demand register */
	u_int16_t cpm_dsr2;	/* SCC2 data synchronization register */
	u_int16_t cpm_scce2;	/* SCC2 event register */
	u_int8_t  cpm_res52[2];
	u_int16_t cpm_sccm2;	/* SCC2 mask register */
	u_int8_t  cpm_res53[1];
	u_int8_t  cpm_sccs2;	/* SCC2 status register */
	u_int8_t  cpm_res54[8];

	/* 0x0a40 SCC3 */
	u_int32_t cpm_gsmr_l3;	/* SCC3 general mode register */
	u_int32_t cpm_gsmr_h3;	/* SCC3 general mode register */
	u_int16_t cpm_psmr3;	/* SCC3 protocol specific mode register */
	u_int8_t  cpm_res55[2];
	u_int16_t cpm_todr3;	/* SCC3 transmit on demand register */
	u_int16_t cpm_dsr3;	/* SCC3 data synchronization register */
	u_int16_t cpm_scce3;	/* SCC3 event register */
	u_int8_t  cpm_res56[2];
	u_int16_t cpm_sccm3;	/* SCC3 mask register */
	u_int8_t  cpm_res57[1];
	u_int8_t  cpm_sccs3;	/* SCC3 status register */
	u_int8_t  cpm_res58[8];

	/* 0x0a60 SCC4 */
	u_int32_t cpm_gsmr_l4;	/* SCC4 general mode register */
	u_int32_t cpm_gsmr_h4;	/* SCC4 general mode register */
	u_int16_t cpm_psmr4;	/* SCC4 protocol specific mode register */
	u_int8_t  cpm_res59[2];
	u_int16_t cpm_todr4;	/* SCC4 transmit on demand register */
	u_int16_t cpm_dsr4;	/* SCC4 data synchronization register */
	u_int16_t cpm_scce4;	/* SCC4 event register */
	u_int8_t  cpm_res60[2];
	u_int16_t cpm_sccm4;	/* SCC4 mask register */
	u_int8_t  cpm_res61[1];
	u_int8_t  cpm_sccs4;	/* SCC4 status register */
	u_int8_t  cpm_res62[10];

	/* 0x0a82 SMC1 */
	u_int16_t cpm_smcmr1;	/* SMC1 mode register */
	u_int8_t  cpm_res63[2];
	u_int8_t  cpm_smce1;	/* SMC1 event register */
	u_int8_t  cpm_res64[3];
	u_int8_t  cpm_smcm1;	/* SMC1 mask register */
	u_int8_t  cpm_res65[7];

	/* 0x0a92 SMC2 */
	u_int16_t cpm_smcmr2;	/* SMC2 mode register */
	u_int8_t  cpm_res66[2];
	u_int8_t  cpm_smce2;	/* SMC2 event register */
	u_int8_t  cpm_res67[3];
	u_int8_t  cpm_smcm2;	/* SMC2 mask register */
	u_int8_t  cpm_res68[5];

	/* 0x0aa0 Serial Peripheral Interface (SPI) */
	u_int16_t cpm_spmode;	/* SPI mode register */
	u_int8_t  cpm_res69[4];
	u_int8_t  cpm_spie;	/* SPI event register */
	u_int8_t  cpm_res70[3];
	u_int8_t  cpm_spim;	/* SPI mask register */
	u_int8_t  cpm_res71[2];
	u_int8_t  cpm_spcom;	/* SPI command register */
	u_int8_t  cpm_res72[4];

	/* 0x0ab2 Parallel Interface Port (PIP) */
	u_int16_t cpm_pipc;	/* PIP configuration register */
	u_int8_t  cpm_res73[2];
	u_int16_t cpm_ptpr;	/* PIP timing parameters register */
	u_int32_t cpm_pbdir;	/* Port B data direction register */
	u_int32_t cpm_pbpar;	/* Port B pin assignment register */
	u_int16_t cpm_pbodr;	/* Port B open drain register */
	u_int32_t cpm_pbdat;	/* Port B data register */
	u_int8_t  cpm_res74[24];

	/* 0x0ae0 Serial Interface */
	u_int32_t cpm_simode;	/* SI mode register */
	u_int8_t  cpm_sigmr;	/* SI global mode register */
	u_int8_t  cpm_res75[1];
	u_int8_t  cpm_sistr;	/* SI status register */
	u_int8_t  cpm_sicmr;	/* SI command register */
	u_int8_t  cpm_res76[4];
	u_int32_t cpm_sicr;	/* SI clock route register */
	u_int32_t cpm_sirp;	/* Serial interface RAM pointer register */
	u_int8_t  cpm_res77[268];

	/* 0x0c00 Specialized RAM */
	u_int8_t  cpm_siram[512];	/* SI routing RAM */

	/* 0x0e00 FEC (860T) */
	u_int32_t fec_addr_high;	/* Upper 32 bits of hardware address */
	u_int32_t fec_addr_low;		/* Lower 16 bits of hardware address */
	u_int32_t fec_hash_table_high;	/* Upper 32 bits of hash table */
	u_int32_t fec_hash_table_low;	/* Lower 32 bits of hash table */
	void *    fec_r_des_start;	/* Beginning of RxBD ring */
	void *    fec_x_des_start;	/* Beginning of TxBD ring */
	u_int32_t fec_r_buff_size;	/* Receive buffer size */
	u_int8_t  fec_res0[36];
	u_int32_t fec_ecntrl;		/* Ethernet control */
	u_int32_t fec_ievent;		/* Interrupt event */
	u_int32_t fec_imask;		/* Interrupt mask */
	u_int32_t fec_ivec;		/* Interrupt level and vector status */
	u_int32_t fec_r_des_active;	/* Receive ring updated flag */
	u_int32_t fec_x_des_active;	/* Transmit ring updated flag */
	u_int8_t  fec_res1[40];
	u_int32_t fec_mii_data;		/* MII data */
	u_int32_t fec_mii_speed;	/* MII speed */
	u_int8_t  fec_res2[68];
	u_int32_t fec_r_bound;		/* End of FIFO RAM (RO) */
	u_int32_t fec_r_fstart;		/* Receive FIFO start address */
	u_int8_t  fec_res3[16];
	u_int32_t fec_x_wmrk;		/* Transmit watermark */
	u_int8_t  fec_res4[4];
	u_int32_t fec_x_fstart;		/* transmit FIFO start address */
	u_int8_t  fec_res5[68];
	u_int32_t fec_fun_code;		/* Function code to SDMA */
	u_int8_t  fec_res6[12];
	u_int32_t fec_r_cntrl;		/* Receive control */
	u_int32_t fec_r_hash;		/* Receive hash register */
	u_int8_t  fec_res7[56];
	u_int32_t fec_x_cntrl;		/* Transmit control */
	u_int8_t  fec_res78[4216];

	/* 0x2000 Dual-Ported RAM */
	u_int8_t  cpm_dpram[4096];	/* Dual-ported RAM */
	u_int8_t  cpm_res79[3072];
	u_int8_t  cpm_pram1[256];	/* Parameter RAM 0x3c00 */
	u_int8_t  cpm_pram2[256];	/* Parameter RAM 0x3d00 */
	u_int8_t  cpm_pram3[256];	/* Parameter RAM 0x3e00 */
	u_int8_t  cpm_pram4[256];	/* Parameter RAM 0x3f00 */
};

#define SCCR_COM     0x60000000		/* Clock output module */
#define SCCR_TBS     0x02000000		/* Timebase source */
#define SCCR_RTDIV   0x01000000		/* Real-time clock divide */
#define SCCR_RTSEL   0x00800000		/* Real-time clock select */
#define SCCR_CRQEN   0x00400000		/* CPM request enable */
#define SCCR_PRQEN   0x00200000		/* Power management request enable */
#define SCCR_EBDF    0x00060000		/* External bus division factor */
#define SCCR_DFSYNC  0x00006000		/* Division factor for SYNCCLK */
#define SCCR_DFBRG   0x00001800		/* Division factor for BRGCLK */
#define SCCR_DFNL    0x00000700		/* Division factor low frequency */
#define SCCR_DFNH    0x000000e0		/* Division factor high frequency */

#define PLPRCR_MF    0xfff00000		/* Multiplication factor */
#define PLPRCR_SPLSS 0x00008000		/* System PLL lock status */
#define PLPRCR_TEXPS 0x00004000		/* Timer expire status */
#define PLPRCR_TMIST 0x00001000		/* Timer interrupt status */
#define PLPRCR_CSRC  0x00000400		/* Clock source */
#define PLPRCR_LPM   0x00000300		/* Low power mode */
#define PLPRCR_LPM_NORM  0x00000000
#define PLPRCR_LPM_DOZE  0x00000100
#define PLPRCR_LPM_SLEEP 0x00000200
#define PLPRCR_LPM_DSLP  0x00000300
#define PLPRCR_CSR   0x00000080		/* Checkstop reset */
#define PLPRCR_LOLRE 0x00000040		/* Loss-of-lock reset enable */
#define PLPRCR_FIOPD 0x00000020		/* Force I/O pull down */


/* CPM definitions */
#define CPMIRQ 4	/* XXX (usually 4) */

#define CIVR_VN(x) ((x) >> 11)		/* 0-4: vector numver */
#define CIVR_IACK  0x0001		/* 15: interrupt acknowledge */

#define CICR_SCCPRI_1234 0x001b0000	/* 8-15: SCC[dcba] = SCC[1234] */
#define CICR_SCCPRI_1243 0x001e0000	/*       SCC[dcba] = SCC[1243] */
#define CICR_SCCPRI_1324 0x00270000	/*       ... */
#define CICR_SCCPRI_1342 0x002d0000
#define CICR_SCCPRI_1423 0x00360000
#define CICR_SCCPRI_1432 0x00390000
#define CICR_SCCPRI_2134 0x004b0000
#define CICR_SCCPRI_2143 0x004e0000
#define CICR_SCCPRI_2314 0x00630000
#define CICR_SCCPRI_2341 0x006c0000
#define CICR_SCCPRI_2413 0x00720000
#define CICR_SCCPRI_2431 0x00780000
#define CICR_SCCPRI_3124 0x00870000
#define CICR_SCCPRI_3142 0x008d0000
#define CICR_SCCPRI_3214 0x00930000
#define CICR_SCCPRI_3241 0x009c0000
#define CICR_SCCPRI_3412 0x00b10000
#define CICR_SCCPRI_3421 0x00b40000
#define CICR_SCCPRI_4123 0x00c60000
#define CICR_SCCPRI_4132 0x00c90000
#define CICR_SCCPRI_4213 0x00d20000
#define CICR_SCCPRI_4231 0x00d80000
#define CICR_SCCPRI_4312 0x00e10000
#define CICR_SCCPRI_4321 0x00e40000
#define CICR_IRL(level)  ((level)<<13)	/* 16-18: interrupt request level */
#define CICR_HP(intr)    ((intr) << 8)	/* 19-23: highest priority */
#define CICR_IEN	 0x00000080	/* 24: interrupt enable */
#define CICR_SPS	 0x00000001	/* 31: spread priority scheme */

#define CPM_IVEC_PC15	0x1f
#define CPM_IVEC_SCC1	0x1e
#define CPM_IVEC_SCC2	0x1d
#define CPM_IVEC_SCC3	0x1c
#define CPM_IVEC_SCC4	0x1b
#define CPM_IVEC_PC14	0x1a
#define CPM_IVEC_TIMER1	0x19
#define CPM_IVEC_PC13	0x18
#define CPM_IVEC_PC12	0x17
#define CPM_IVEC_SDMA	0x16
#define CPM_IVEC_IDMA1	0x15
#define CPM_IVEC_IDMA2	0x14
#define CPM_IVEC_TIMER2	0x12
#define CPM_IVEC_RTT	0x11
#define CPM_IVEC_I2C	0x10
#define CPM_IVEC_PC11	0x0f
#define CPM_IVEC_PC10	0x0e
#define CPM_IVEC_TIMER3	0x0c
#define CPM_IVEC_PC9	0x0b
#define CPM_IVEC_PC8	0x0a
#define CPM_IVEC_PC7	0x09
#define CPM_IVEC_TIMER4	0x07
#define CPM_IVEC_PC6	0x06
#define CPM_IVEC_SPI	0x05
#define CPM_IVEC_SMC1	0x04
#define CPM_IVEC_SMC2	0x03
#define CPM_IVEC_PIP	CPM_IVEC_SMC2
#define CPM_IVEC_PC5	0x02
#define CPM_IVEC_PC4	0x01


/* CPCR CH numbers. */
#define CPCR_SCC1  0x0000
#define CPCR_I2C   0x0010
#define CPCR_IDMA1 CPCR_I2C
#define CPCR_SCC2  0x0040
#define CPCR_SPI   0x0050
#define CPCR_IDMA2 CPCR_SPI
#define CPCR_TIMER CPCR_SPI
#define CPCR_SCC3  0x0080
#define CPCR_SMC1  0x0090
#define CPCR_DSP1  CPCR_SMC1
#define CPCR_SCC4  0x00c0
#define CPCR_SMC2  0x00d0
#define CPCR_DSP2  CPCR_SMC2
#define CPCR_PIP   CPCR_SMC2


#define GSMR_H_IRP  0x00040000	/* Infrared Rx polarity */
#define GSMR_H_GDE  0x00010000	/* Glitch detect enable */
#define GSMR_H_TCRC 0x0000c000	/* Transparent CRC (mask) */
#define GSMR_H_REVD 0x00002000	/* Reverse data */
#define GSMR_H_TRX  0x00001000	/* Transparent receiver */
#define GSMR_H_TTX  0x00000800	/* Transparent transmitter */
#define GSMR_H_CDP  0x00000400	/* CD pulse */
#define GSMR_H_CTSP 0x00000200	/* CTS pulse */
#define GSMR_H_CDS  0x00000100	/* CD sampling */
#define GSMR_H_CTSS 0x00000080	/* CTS sampling */
#define GSMR_H_TFL  0x00000040	/* Tx FIFO length */
#define GSMR_H_RFW  0x00000020	/* Rx FIFO width */
#define GSMR_H_TXSY 0x00000010	/* Transmitter synchronized to the receiver */
#define GSMR_H_SYNL 0x0000000c	/* Sync length (mask) */
#define GSMR_H_RTSM 0x00000002	/* RTS mode */
#define GSMR_H_RSYN 0x00000001	/* Receive synchronization timing */

#define GSMR_L_SIR  0x80000000	/* Serial infrared encoding (for SCC2) */
#define GSMR_L_EDGE 0x60000000	/* Clock edge (mask) */
#define GSMR_L_TCI  0x10000000	/* Transmit clock invert */
#define GSMR_L_TSNC 0x0c000000	/* Transmit sense (mask) */
#define GSMR_L_RINV 0x02000000	/* DPLL Rx input invert data */
#define GSMR_L_TINV 0x01000000	/* DPLL Tx input invert data */
#define GSMR_L_TPL  0x00e00000	/* Tx preamble length (mask) */
#define GSMR_L_TPP  0x00180000	/* Tx preamble pattern (mask) */
#define GSMR_L_TEND 0x00040000	/* Transmitter frame ending */
#define GSMR_L_TDCR 0x00030000	/* Transmitter DPLL clock rate (mask) */
#define GSMR_L_TDCR_1  0x00000000	/* 1x  */
#define GSMR_L_TDCR_8  0x00010000	/* 8x  */
#define GSMR_L_TDCR_16 0x00020000	/* 16x */
#define GSMR_L_TDCR_32 0x00030000	/* 32x */
#define GSMR_L_RDCR 0x0000c000	/* Receiver DPLL clock rate (mask) */
#define GSMR_L_RDCR_1  0x00000000	/* 1x  */
#define GSMR_L_RDCR_8  0x00004000	/* 8x  */
#define GSMR_L_RDCR_16 0x00008000	/* 16x */
#define GSMR_L_RDCR_32 0x0000c000	/* 32x */
#define GSMR_L_RENC 0x00003800	/* Receiver decoding method (mask) */
#define GSMR_L_TENC 0x00000700	/* Transmitter encoding method (mask) */
#define GSMR_L_DIAG 0x000000c0	/* Diagnostic mode  */
#define GSMR_L_ENR  0x00000020	/* Enable Rx */
#define GSMR_L_ENT  0x00000010	/* Enable Tx */
#define GSMR_L_MODE 0x0000000f	/* Channel protocol mode (mask) */
#define GSMR_L_MODE_HDLC   0x00000000	/* HDLC */
#define GSMR_L_MODE_ATALK  0x00000002	/* AppleTalk/LocalTalk */
#define GSMR_L_MODE_UART   0x00000004	/* UART */
#define GSMR_L_MODE_AHDLC  0x00000006	/* Async HDLC or IrDA */
#define GSMR_L_MODE_BISYNC 0x00000008	/* BISYNC */
#define GSMR_L_MODE_ETHER  0x0000000c	/* Ethernet */


#define PSMR_FLC	0x8000	/* Flow control */
#define PSMR_SL		0x4000	/* Stopbit length */
#define PSMR_CL5	0x0000	/* Character length */
#define PSMR_CL6	0x1000
#define PSMR_CL7	0x2000
#define PSMR_CL8	0x3000
#define PSMR_UM		0x0c00	/* UART mode */
#define PSMR_FRZ	0x0200	/* Freeze transmission */
#define PSMR_RZS	0x0100	/* Receive zero stopbits */
#define PSMR_SYN	0x0080	/* Synchronous mode */
#define PSMR_DRT	0x0040	/* Disable receiver on transmit */
#define PSMR_PEN	0x0010	/* Parity enable */
#define PSMR_RPM_O	0x0000	/* Receiver parity mode (odd) */
#define PSMR_RPM_S	0x0004	/*                      (space) */
#define PSMR_RPM_E	0x0008	/*                      (even) */
#define PSMR_RPM_M	0x000c	/*                      (mark) */
#define PSMR_TPM_O	0x0000	/* Transmitter parity mode (odd) */
#define PSMR_TPM_S	0x0001	/*                         (space) */
#define PSMR_TPM_E	0x0002	/*                         (even) */
#define PSMR_TPM_M	0x0003	/*                         (mark) */


/* NOTE: bd_addr field for RxBD must be even on the SCC. */
struct cpm_bdesc {
	u_int16_t bd_flags;
	u_int16_t bd_len;
	u_char *bd_addr;
};


/* SMC */
#define smc_bdesc cpm_bdesc

struct smc_uart {
	u_int16_t rbase;	/* RxBD base address */
	u_int16_t tbase;	/* TxBD base address */
	u_int8_t  rfcr;		/* Rx function code */
	u_int8_t  tfcr;		/* Tx function code */
	u_int16_t mrblr;	/* Maximum receive buffer length */
	u_int32_t rstate;	/* Rx internal state */
	u_int32_t riptr;	/* Rx internal data pointer */
	u_int16_t rbptr;	/* RxBD pointer */
	u_int16_t ricount;	/* Rx internal byte count */
	u_int32_t rtemp;	/* Rx temp */
	u_int32_t tstate;	/* Tx internal state */
	u_int32_t tiptr;	/* Tx internal data pointer */
	u_int16_t tbptr;	/* TxBD pointer */
	u_int16_t ticount;	/* Tx internal byte count */
	u_int32_t ttemp;	/* Tx temp */

	u_int16_t max_idl;	/* Maximum idle characters */
	u_int16_t idlc;		/* Temporary idle conter */
	u_int16_t brkln;	/* Last received break length */
	u_int16_t brkec;	/* Receive break condition counter */
	u_int16_t brkcr;	/* Break count register (transmit) */
	u_int16_t r_mask;	/* Temporary bit mask */
};


/* SCC */

struct scc_uart {
	/* SCC common part */
	u_int16_t rbase;	/* RxBD base address */
	u_int16_t tbase;	/* TxBD base address */
	u_int8_t  rfcr;		/* Rx function code */
	u_int8_t  tfcr;		/* Tx function code */
	u_int16_t mrblr;	/* Maximum receive buffer length */
	u_int32_t rstate;	/* Rx internal state */
	u_int32_t riptr;	/* Rx internal buffer pointer */
	u_int16_t rbptr;	/* Current RxBD pointer */
	u_int16_t ricount;	/* Rx internal byte count */
	u_int32_t rtemp;	/* Rx temp */
	u_int32_t tstate;	/* Tx internal state */
	u_int32_t tiptr;	/* Tx internal buffer pointer */
	u_int16_t tbptr;	/* Current TxBD pointer */
	u_int16_t ticount;	/* Tx internal byte count */
	u_int32_t ttemp;	/* Tx temp */
	u_int32_t rcrc;		/* Temp receive CRC */
	u_int32_t tcrc;		/* Temp transmit CRC */

	/* SCC UART specific part */
	u_int32_t res;
	u_int16_t max_idl;	/* Maximum idle characters */
	u_int16_t idlc;		/* Temporary idle conter */
	u_int16_t brkcr;	/* Break count register (transmit) */
	u_int16_t parec;	/* Parity errors */
	u_int16_t frmec;	/* Framing error */
	u_int16_t nosec;	/* Noise errors */
	u_int16_t brkec;	/* Break conditions */
	u_int16_t brkln;	/* Last received break length */
	u_int16_t uaddr1;	/* UART address character1 (multidrop) */
	u_int16_t uaddr2;	/* UART address character2 (multidrop) */
	u_int16_t temp;		/* Temporary storage */
	u_int16_t toseq;	/* Transmit out-of-sequence character */
	u_int16_t character[8];	/* Control character */
	u_int16_t rccm;		/* Receive control character mask */
	u_int16_t rccr;		/* Receive control character register */
	u_int16_t rlbc;		/* Receive last break character */
};

struct scc_enet {
	/* SCC common part */
	u_int16_t rbase;	/* RxBD base address */
	u_int16_t tbase;	/* TxBD base address */
	u_int8_t  rfcr;		/* Rx function code */
	u_int8_t  tfcr;		/* Tx function code */
	u_int16_t mrblr;	/* Maximum receive buffer length */
	u_int32_t rstate;	/* Rx internal state */
	u_int32_t riptr;	/* Rx internal buffer pointer */
	u_int16_t rbptr;	/* Current RxBD pointer */
	u_int16_t ricount;	/* Rx internal byte count */
	u_int32_t rtemp;	/* Rx temp */
	u_int32_t tstate;	/* Tx internal state */
	u_int32_t tiptr;	/* Tx internal buffer pointer */
	u_int16_t tbptr;	/* Current TxBD pointer */
	u_int16_t ticount;	/* Tx internal byte count */
	u_int32_t ttemp;	/* Tx temp */
	u_int32_t rcrc;		/* Temp receive CRC */
	u_int32_t tcrc;		/* Temp transmit CRC */

	/* SCC Ethernet specific part */
	u_int32_t c_pres;	/* Preset CRC */
	u_int32_t c_mask;	/* Constant mask for CRC */
	u_int32_t crcec;	/* CRC error */
	u_int32_t alec;		/* Alignment error */
	u_int32_t disfc;	/* Discard frame count */
	u_int16_t pads;		/* Short frame pad character */
	u_int16_t ret_lim;	/* Retry limit */
	u_int16_t ret_cnt;	/* Retry limit counter */
	u_int16_t mflr;		/* Maximum frame length */
	u_int16_t minflr;	/* Minimum frame length */
	u_int16_t maxd1;	/* Max DMA1 length */
	u_int16_t maxd2;	/* Max DMA2 length */
	u_int16_t maxd;		/* Rx max DMA */
	u_int16_t dma_cnt;	/* Rx DMA counter */
	u_int16_t max_b;	/* Maximum BD byte count */
	u_int16_t gaddr1;	/* Group address filter 1 */
	u_int16_t gaddr2;	/* Group address filter 2 */
	u_int16_t gaddr3;	/* Group address filter 3 */
	u_int16_t gaddr4;	/* Group address filter 4 */
	u_int32_t tbuf0_data0;	/* (CPM internal use) */
	u_int32_t tbuf0_data1;	/* (CPM internal use) */
	u_int32_t tbuf0_rba0;	/* (CPM internal use) */
	u_int32_t tbuf0_crc;	/* (CPM internal use) */
	u_int16_t tbuf0_bcnt;	/* (CPM internal use) */
	u_int16_t paddr1_h;	/* Station address (H) */
	u_int16_t paddr1_m;	/* Station address (M) */
	u_int16_t paddr1_l;	/* Station address (L) */
	u_int16_t p_per;	/* Persistence */
	u_int16_t rfbd_ptr;	/* Rx first BD pointer */
	u_int16_t tfbd_ptr;	/* Tx first BD pointer */
	u_int16_t tlbd_ptr;	/* Tx last BD pointer */
	u_int32_t tbuf1_data0;	/* (CPM internal use) */
	u_int32_t tbuf1_data1;	/* (CPM internal use) */
	u_int32_t tbuf1_rba0;	/* (CPM internal use) */
	u_int32_t tbuf1_crc;	/* (CPM internal use) */
	u_int16_t tbuf1_bcnt;	/* (CPM internal use) */
	u_int16_t tx_len;	/* Tx frame length counter */
	u_int16_t iaddr1;	/* Individual address filter 1 */
	u_int16_t iaddr2;	/* Individual address filter 2 */
	u_int16_t iaddr3;	/* Individual address filter 3 */
	u_int16_t iaddr4;	/* Individual address filter 4 */
	u_int16_t boff_cnt;	/* Backoff counter */
	u_int16_t taddr_h;	/* Temporary address (H) */
	u_int16_t taddr_m;	/* Temporary address (M) */
	u_int16_t taddr_l;	/* Temporary address (L) */
};


/* I2C */

struct i2c_pram {
	u_int16_t rbase;	/* RxBD table base address */
	u_int16_t tbase;	/* TxBD table base address */
	u_int8_t  rfcr;		/* Rx function code */
	u_int8_t  tfcr;		/* Tx function code */
	u_int16_t mrblr;	/* Maximum receive buffer length */
	u_int32_t rstate;	/* Rx internal state */
	u_int32_t rptr;		/* Rx internal data pointer */
	u_int16_t rbptr;	/* RxBD pointer */
	u_int16_t rcount;	/* Rx internal byte count */
	u_int32_t rtemp;	/* Rx temp */
	u_int32_t tstate;	/* Tx internal state */
	u_int32_t tptr;		/* Tx internal data pointer */
	u_int16_t tbptr;	/* TxBD pointer */
	u_int16_t tcount;	/* Tx internal byte count */
	u_int16_t ttemp;	/* TX temp */
};

#define I2MOD_REVD	0x20	/* Reverse data (LSB first) */
#define I2MOD_GCD	0x10	/* General call disable */
#define I2MOD_FLT	0x08	/* Clock filter */
#define I2MOD_PDIV	0x06	/* BRGCLK Prediveder */
#define I2MOD_PDIV_32	0x00	/* BRGCLK / 32 */
#define I2MOD_PDIV_16	0x02	/* BRGCLK / 16 */
#define I2MOD_PDIV_8	0x04	/* BRGCLK / 8 */
#define I2MOD_PDIV_4	0x06	/* BRGCLK / 4 */
#define I2MOD_EN	0x01	/* Enable I2C */

#define I2COM_MASTER	0x01
#define I2COM_SLAVE	0x00
#define I2COM_STR	0x80


/* SPI */

struct spi_pram {
	u_int16_t rbase;	/* RxBD base address */
	u_int16_t tbase;	/* TxBD base address */
	u_int8_t  rfcr;		/* Rx function code */
	u_int8_t  tfcr;		/* Tx function code */
	u_int16_t mrblr;	/* Maximum receive buffer length */
	u_int32_t rstate;	/* Rx internal state */
	u_int32_t riptr;	/* Rx internal data pointer */
	u_int16_t rbptr;	/* RxBD pointer */
	u_int16_t ricount;	/* Rx internal byte count */
	u_int32_t rtemp;	/* Rx temp */
	u_int32_t tstate;	/* Tx internal state */
	u_int32_t tiptr;	/* Tx internal data pointer */
	u_int16_t tbptr;	/* TxBD pointer */
	u_int16_t ticount;	/* Tx internal byte count */
	u_int32_t ttemp;	/* Tx temp */
};

#define SPMODE_LOOP	0x4000	/* Loop mode */
#define SPMODE_CI	0x2000	/* Clock invert */
#define SPMODE_CP	0x1000	/* Clock phase */
#define SPMODE_DIV16	0x0800	/* Divide by 16 */
#define SPMODE_REV	0x0400	/* Reverse data (1:MSB first) */
#define SPMODE_MS	0x0200	/* Master/slave */
#define SPMODE_EN	0x0100	/* Enable */
#define SPMODE_LEN	0x00f0	/* Character length */
#define SPMODE_PM	0x000f	/* Prescale modulus */

#define SPIE_MME	0x20	/* Multimaster error */
#define SPIE_TXE	0x10	/* Tx error */
#define SPIE_BSY	0x04	/* Busy */
#define SPIE_TXB	0x02	/* Tx buffer */
#define SPIE_RXB	0x01	/* Rx buffer */

#define SPCOM_STR	0x80	/* Start transmit */


/* Parameter RAM offset */
#define DPRAM_SCC1 0x1c00
#define DPRAM_I2C  0x1c80
#define DPRAM_SPI  0x1d80
#define DPRAM_SMC1 0x1e80


static __inline void *
get_immr()
{
	u_int immr;

	asm ("mfimmr  %0" : "=r"(immr));
	return (void *)(immr & 0xffff0000);
}
