/*	$Id: pmap.h,v 1.34 2010/01/05 06:54:50 tsubai Exp $	*/

/*
 * Copyright (c) 1997 Charles D. Cranor and Washington University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by Charles D. Cranor and
 *      Washington University.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * pmap.h: see pmap.c for the history of this pmap module.
 */

#ifndef _MPC860_PMAP_H_
#define _MPC860_PMAP_H_

#include <machine/pte.h>

#ifdef _KERNEL

struct pmap;

/*
 * the pmap structure
 *
 * note that the pm_obj contains the simple_lock, the reference count,
 * page list, and number of PTPs within the pmap.
 */
struct pmap {
	int pm_refs;
	u_int *pm_pdir;
	paddr_t pm_pdirpa;
	int pm_asid;
	struct pmap_statistics pm_stats;
};

extern struct pmap kernel_pmap_store;	/* kernel pmap */

#undef pmap_kernel
#define pmap_kernel()			(&kernel_pmap_store)
#define pmap_resident_count(pmap)	((pmap)->pm_stats.resident_count)
#define pmap_wired_count(pmap)		((pmap)->pm_stats.wired_count)
#define pmap_update(pmap)		/* nothing (yet) */

#define pmap_clear_modify(pg)		pmap_change_attrs(pg, 0, PG_CHG)
#define pmap_clear_reference(pg)	pmap_change_attrs(pg, 0, PG_REF)
#define pmap_copy(DP,SP,D,L,S)
#define pmap_is_modified(pg)		pmap_test_attrs(pg, PG_CHG)
#define pmap_is_referenced(pg)		pmap_test_attrs(pg, PG_REF)
#define pmap_phys_address(x)		(x)

void pmap_bootstrap(void);
int pmap_change_attrs(struct vm_page *, int, int);
void pmap_page_remove(struct vm_page *);
void pmap_protect(struct pmap *, vaddr_t, vaddr_t, vm_prot_t);
void pmap_real_memory(paddr_t *, psize_t *);
void pmap_remove(struct pmap *, vaddr_t, vaddr_t);
void pmap_remove_all(struct pmap *);
int pmap_test_attrs(struct vm_page *, int);
void pmap_write_protect(struct pmap *, vaddr_t, vaddr_t, vm_prot_t);
int pmap_emul_modify(struct pmap *, vaddr_t);

#define PMAP_NEED_PROCWR
void pmap_procwr(struct proc *, vaddr_t, size_t);
paddr_t vtophys(vaddr_t);

/*
 * pmap_protect: change the protection of pages in a pmap
 *
 * => this function is a frontend for pmap_remove/pmap_write_protect
 * => we only have to worry about making the page more protected.
 *	unprotecting a page is done on-demand at fault time.
 */
extern __inline void
pmap_protect(pmap, sva, eva, prot)
	struct pmap *pmap;
	vaddr_t sva, eva;
	vm_prot_t prot;
{
	if ((prot & VM_PROT_WRITE) == 0) {
		if (prot & (VM_PROT_READ|VM_PROT_EXECUTE))
			pmap_write_protect(pmap, sva, eva, prot);
		else
			pmap_remove(pmap, sva, eva);
	}
}

extern __inline void
pmap_remove_all(pmap)
	struct pmap *pmap;
{
}

/*
 * Alternate mapping hooks for pool pages.  Avoids thrashing the TLB.
 */
#define PMAP_MAP_POOLPAGE(pa)	(pa)
#define PMAP_UNMAP_POOLPAGE(va)	(va)

/*
 * Other hooks for the pool allocator.
 */
#define POOL_VTOPHYS(va)	PMAP_UNMAP_POOLPAGE((vaddr_t)(va))

#define PMAP_CI 0x80000000

#endif /* _KERNEL */
#endif /* _MPC860_PMAP_H_ */
