/*	$Id: cpu.c,v 1.13 2010/01/11 07:53:39 tsubai Exp $	*/

/*-
 * Copyright (c) 2009 Tsubai Masanari.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/param.h>
#include <sys/systm.h>
#include <sys/device.h>

#include <machine/autoconf.h>

int cpumatch(struct device *, struct cfdata *, void *);
void cpuattach(struct device *, struct device *, void *);
void identifycpu(char *);

CFATTACH_DECL0(cpu, sizeof(struct device), cpumatch, cpuattach);

int
cpumatch(parent, cf, aux)
	struct device *parent;
	struct cfdata *cf;
	void *aux;
{
	struct confargs *ca = aux;

	if (strcmp(ca->ca_name, "cpu") == 0)
		return 1;
	return 0;
}

void
cpuattach(parent, self, aux)
	struct device *parent, *self;
	void *aux;
{
	char model[80];

	identifycpu(model);
	printf(": %s\n", model);
}

struct cputab {
	int version;
	char *name;
};
static struct cputab models[] = {
	{ 0x50,	    "860" },
	{ 0,	    NULL }
};

void
identifycpu(cpu_model)
	char *cpu_model;
{
	int pvr, vers, rev;
	struct cputab *cp = models;

	asm ("mfpvr %0" : "=r"(pvr));
	vers = pvr >> 16;
	rev = pvr & 0xffff;

	while (cp->name) {
		if (cp->version == vers)
			break;
		cp++;
	}
	if (cp->name)
		strcpy(cpu_model, cp->name);
	else
		sprintf(cpu_model, "Version %x", vers);
	sprintf(cpu_model + strlen(cpu_model), " (Revision %x)", rev);
}

/*
 Rev  |  Mask  |IMMR[16:31]
 D.4  | 3K20A  | 0x0502
 D.3  | 2K20A  | 0x0501
 C.1  | 2H96G  | 0x0031
 C.1  | 3H96G  | 0x0031
 C.0  | 0H96G  | 0x0030
 C.0  | 1H96G  | 0x0030
 B.1  | 1J24A  | 0x0020
 B.1  | 0H86G  | 0x0020
 B.1  | 2J24A  | 0x0020
 B.1  | 9J24A  | 0x0020
 B.2  | 0J21M  | 0x0020
 B.3  | 1J21M  | 0x0020
 B.5  | 3J21M  | 0x0020
 A.3  | 4F84C  | 0x0013
 A.2  | 2F84C  | 0x0012
 A.1  | 1F84C  | 0x0010
 0.3  | 3E64C  | 0x0002
 0.2  | 2E64C  | 0x0002
*/
