/*	$Id: exception.S,v 1.43 2012/03/01 09:09:03 tsubai Exp $	*/

/*
 * Copyright (C) 1995, 1996 Wolfgang Solfrank.
 * Copyright (C) 1995, 1996 TooLs GmbH.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by TooLs GmbH.
 * 4. The name of TooLs GmbH may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TOOLS GMBH ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL TOOLS GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "opt_ddb.h"
#include "opt_ipkdb.h"
#include "assym.h"

#include <sys/syscall.h>

#include <machine/param.h>
#include <machine/asm.h>
#include <machine/psl.h>
#include <machine/spr.h>
#include <machine/trap.h>

/*
 * This code gets copied to all the trap vectors
 * (except ISI/DSI, ALI, the interrupts, and possibly the debugging
 * traps when using IPKDB).
 */
ENTRY_NOPROFILE(trapcode)
	mtsprg	1,1			/* save SP */
	GET_CPUINFO(1)
	stmw	28,CI_TEMPSAVE(1)	/* free r28-r31 */
	mflr	28			/* save LR */
	mfcr	29			/* save CR */
/* Test whether we already had PR set */
	mfsrr1	31
	andi.	31,31,PSL_PR
	beq	1f			/* branch if supervisor */
	lwz	1,CI_CURPCB(1)
	addi	1,1,USPACE		/* stack is top of user struct */
	bla	s_trap
1:
	mfsprg	1,1			/* restore SP */
	bla	s_trap

/*
 * For ALI: has to save DSISR and DAR
 */
ENTRY_NOPROFILE(alitrap)
	mtsprg	1,1			/* save SP */
	GET_CPUINFO(1)
	stmw	28,CI_TEMPSAVE(1)	/* free r28-r31 */
	mfdar	30
	mfdsisr	31
	stmw	30,CI_TEMPSAVE+16(1)
	mflr	28			/* save LR */
	mfcr	29			/* save CR */
/* Test whether we already had PR set */
	mfsrr1	31
	andi.	31,31,PSL_PR
	beq	1f			/* branch if supervisor */
	lwz	1,CI_CURPCB(1)
	addi	1,1,USPACE		/* stack is top of user struct */
	bla	s_trap
1:
	mfsprg	1,1			/* restore SP */
	bla	s_trap

/*
 * Similar to the above for DSI
 */
ENTRY_NOPROFILE(dsitrap)
	mtsprg	1,1			/* save SP */
	GET_CPUINFO(1)
	stmw	28,CI_TEMPSAVE(1)	/* free r28-r31 */
	mflr	28			/* save LR */
	mfcr	29			/* save CR */
	mfdar	30
	mfdsisr	31
	stmw	30,CI_TEMPSAVE+16(1)
	bla	disitrap

/*
 * Similar to the above for ISI
 */
ENTRY_NOPROFILE(isitrap)
	mtsprg	1,1			/* save SP */
	GET_CPUINFO(1)
	stmw	28,CI_TEMPSAVE(1)	/* free r28-r31 */
	mflr	28			/* save LR */
	mfcr	29			/* save CR */
	bla	disitrap

/*
 * This one for the external interrupt handler.
 */
ENTRY_NOPROFILE(extint)
	mtsprg	1,1			/* save SP */
	GET_CPUINFO(1)
	stmw	28,CI_TEMPSAVE(1)	/* free r28-r31 */
	mflr	28			/* save LR */
	mfcr	29			/* save CR */
	mfxer	30			/* save XER */
	lwz	31,CI_INTRDEPTH(1)	/* were we already running on intstk? */
	addic.	31,31,1
	stw	31,CI_INTRDEPTH(1)
	beq+	1f
	mfsprg	1,1			/* yes, get old SP */
	ba	extintr
1:
	lwz	1,CI_INTSTK(1)		/* get interrupt stack */
	ba	extintr

/*
 * And this one for the decrementer interrupt handler.
 */
ENTRY_NOPROFILE(decrint)
	mtsprg	1,1			/* save SP */
	GET_CPUINFO(1)
	stmw	28,CI_TEMPSAVE(1)	/* free r28-r31 */
	mflr	28			/* save LR */
	mfcr	29			/* save CR */
	mfxer	30			/* save XER */
	lwz	31,CI_INTRDEPTH(1)	/* were we already running on intstk? */
	addic.	31,31,1
	stw	31,CI_INTRDEPTH(1)
	beq+	1f
	mfsprg	1,1			/* yes, get old SP */
	ba	decrintr
1:
	lwz	1,CI_INTSTK(1)		/* get interrupt stack */
	ba	decrintr

#ifdef DDB
/*
 * In case of DDB we want a separate trap catcher for it
 */
ENTRY_NOPROFILE(ddblow)
	mtsprg	1,1			/* save SP */
	GET_CPUINFO(1)
	stmw	28,CI_DDBSAVE(1)	/* free r28-r31 */
	mflr	28			/* save LR */
	mfcr	29			/* save CR */
	lwz	1,CI_DDBSTK(1)		/* get new SP */
	bla	ddbtrap
#endif	/* DDB */

/*
 * FRAME_SETUP assumes:
 *	SPRG1		SP (1)
 *	savearea	r28-r31,DAR,DSISR	(DAR & DSISR only for DSI traps)
 *	28		LR
 *	29		CR
 *	1		kernel stack
 *	LR		trap type
 *	SRR0/1		as at start of trap
 */
#define FRAME_SETUP(savearea)						\
/* Have to enable translation to allow access of kernel stack: */	\
	GET_CPUINFO(30);						\
	mfsrr0	31;							\
	stw	31,savearea+24(30);					\
	mfsrr1	31;							\
	stw	31,savearea+28(30);					\
	mfsprg	30,1;							\
	mfmsr	31;							\
	ori	31,31,(PSL_DR|PSL_IR);					\
	mtmsr	31;							\
	isync;								\
	stwu	30,-FRAMELEN(1);					\
	stw	0,FRAME_0+8(1);						\
	stw	30,FRAME_1+8(1);					\
	stw	2,FRAME_2+8(1);						\
	stw	28,FRAME_LR+8(1);					\
	stw	29,FRAME_CR+8(1);					\
	GET_CPUINFO(2);							\
	lmw	28,savearea(2);						\
	stmw	3,FRAME_3+8(1);						\
	lmw	28,savearea+16(2);					\
	mfxer	3;							\
	mfctr	4;							\
	mflr	5;							\
	andi.	5,5,0xff00;						\
	stw	3,FRAME_XER+8(1);					\
	stw	4,FRAME_CTR+8(1);					\
	stw	5,FRAME_EXC+8(1);					\
	stw	28,FRAME_DAR+8(1);					\
	stw	29,FRAME_DSISR+8(1);					\
	stw	30,FRAME_SRR0+8(1);					\
	stw	31,FRAME_SRR1+8(1)

#define FRAME_LEAVE(savearea)						\
/* Now restore regs: */							\
	lwz	3,FRAME_SRR0+8(1);					\
	lwz	4,FRAME_SRR1+8(1);					\
	lwz	5,FRAME_CTR+8(1);					\
	lwz	6,FRAME_XER+8(1);					\
	lwz	7,FRAME_LR+8(1);					\
	GET_CPUINFO(2);							\
	stw	3,savearea+24(2);					\
	stw	4,savearea+28(2);					\
	mtctr	5;							\
	mtxer	6;							\
	mtlr	7;							\
	lmw	3,FRAME_3+8(1);						\
	stmw	28,savearea(2);		/* free r28-r31 */		\
	lwz	28,FRAME_2+8(1);	/* save r2 in r28 */		\
	lwz	29,FRAME_CR+8(1);	/* save cr in r29 */		\
	lwz	0,FRAME_0+8(1);						\
	lwz	1,FRAME_1+8(1);						\
/* Disable translation, machine check and recoverability: */		\
	mfmsr	31;							\
	andi.	31,31,~(PSL_DR|PSL_IR|PSL_ME|PSL_RI)@l;			\
	mtmsr	31;							\
	isync;								\
	mtsprg	2,28;			/* save r2 in sprg2 */		\
	mtcr	29;			/* restore cr */		\
	lwz	30,savearea+24(2);					\
	lwz	31,savearea+28(2);					\
	mtsrr0	30;							\
	mtsrr1	31;							\
	lmw	28,savearea(2);		/* restore r28-r31 */		\
	mfsprg	2,2			/* restore r2 */

disitrap:
realtrap:
/* Test whether we already had PR set */
	mfsrr1	1
	andi.	1,1,PSL_PR
	mfsprg	1,1			/* restore SP (might have been
					   overwritten) */
	beq	s_trap			/* branch if supervisor */
	GET_CPUINFO(1)
	lwz	1,CI_CURPCB(1)
	addi	1,1,USPACE		/* stack is top of user struct */

/*
 * Now the common trap catching code.
 */
s_trap:
	FRAME_SETUP(CI_TEMPSAVE)
/* Now we can recover interrupts again: */
trapagain:
	mfmsr	7
	ori	7,7,(PSL_EE|PSL_ME|PSL_RI)@l
	mtmsr	7
	isync
/* Call C trap code: */
	addi	3,1,8
	bl	_C_LABEL(trap)
trapexit:
/* Disable interrupts: */
	mfmsr	3
	andi.	3,3,~PSL_EE@l
	mtmsr	3
/* Test AST pending: */
	lwz	5,FRAME_SRR1+8(1)
	andi.	5,5,PSL_PR
	beq	1f			/* branch if supervisor */
	GET_CPUINFO(3)
	lwz	4,CI_ASTPENDING(3)
	andi.	4,4,1
	beq	1f
	li	6,EXC_AST
	stw	6,FRAME_EXC+8(1)
	b	trapagain
1:
	FRAME_LEAVE(CI_TEMPSAVE)
	rfi

/*
 * Child comes here at the end of a fork.
 * Return to userspace via the trap return path.
 * XXX r3 and r4 are passed from cpu_switchto() unchanged.
 */
ENTRY_NOPROFILE(fork_trampoline)
	bl	_C_LABEL(lwp_startup)
	mtlr	31
	mr	3,30
	blrl				/* jump indirect to r31 */
	b	trapexit

/*
 * External interrupt second level handler
 */
#define INTRENTER							\
/* Save volatile registers: */						\
	stwu	1,-96(1);		/* temporarily */		\
	stw	0,84(1);						\
	mfsprg	0,1;			/* get original SP */		\
	stw	0,0(1);			/* and store it */		\
	stw	3,80(1);						\
	stw	4,76(1);						\
	stw	5,72(1);						\
	stw	6,68(1);						\
	stw	7,64(1);						\
	stw	8,60(1);						\
	stw	9,56(1);						\
	stw	10,52(1);						\
	stw	11,48(1);						\
	stw	12,44(1);						\
	stw	28,40(1);		/* saved LR */			\
	stw	29,36(1);		/* saved CR */			\
	stw	30,32(1);		/* saved XER */			\
	GET_CPUINFO(5);							\
	lmw	28,CI_TEMPSAVE(5);	/* restore r28-r31 */		\
	mfctr	6;							\
	lwz	5,CI_INTRDEPTH(5);					\
	mfsrr0	4;							\
	mfsrr1	3;							\
	stw	6,28(1);						\
	stw	5,20(1);						\
	stw	4,12(1);						\
	stw	3,8(1);							\
/* interrupts are recoverable here, and enable translation */		\
	mfmsr	5;							\
	ori	5,5,(PSL_IR|PSL_DR|PSL_RI);				\
	mtmsr	5;							\
	isync

	.globl	_C_LABEL(extint_call)
extintr:
	INTRENTER
_C_LABEL(extint_call):
	bl	_C_LABEL(extint_call)	/* to be filled in later */

intr_exit:
/* Disable interrupts (should already be disabled) and MMU here: */
	mfmsr	3
	andi.	3,3,~(PSL_EE|PSL_ME|PSL_RI|PSL_DR|PSL_IR)@l
	mtmsr	3
	isync
/* restore possibly overwritten registers: */
	lwz	12,44(1)
	lwz	11,48(1)
	lwz	10,52(1)
	lwz	9,56(1)
	lwz	8,60(1)
	lwz	7,64(1)
	lwz	6,8(1)
	lwz	5,12(1)
	lwz	4,28(1)
	lwz	3,32(1)
	mtsrr1	6
	mtsrr0	5
	mtctr	4
	mtxer	3
/* Returning to user mode? */
	andi.	6,6,PSL_PR		/* saved SRR1 */
	beq	1f			/* branch if supervisor */
	GET_CPUINFO(4)
	lwz	6,CI_ASTPENDING(4)	/* Test AST pending */
	andi.	6,6,1
	beq	1f
/* Setup for entry to realtrap: */
	lwz	3,0(1)			/* get saved SP */
	mtsprg	1,3
	li	6,EXC_AST
	stmw	28,CI_TEMPSAVE(4)	/* establish tempsave again */
	mtlr	6
	lwz	28,40(1)		/* saved LR */
	lwz	29,36(1)		/* saved CR */
	lwz	6,68(1)
	lwz	5,72(1)
	lwz	4,76(1)
	lwz	3,80(1)
	lwz	0,84(1)
	GET_CPUINFO(30)
	lwz	31,CI_INTRDEPTH(30)	/* adjust reentrancy count */
	addi	31,31,-1
	stw	31,CI_INTRDEPTH(30)
	b	realtrap
1:
/* Here is the normal exit of extintr: */
	lwz	5,36(1)
	lwz	6,40(1)
	mtcr	5
	mtlr	6
	lwz	6,68(1)
	lwz	5,72(1)
	GET_CPUINFO(3)
	lwz	4,CI_INTRDEPTH(3)	/* adjust reentrancy count */
	addi	4,4,-1
	stw	4,CI_INTRDEPTH(3)
	lwz	4,76(1)
	lwz	3,80(1)
	lwz	0,84(1)
	lwz	1,0(1)
	rfi

/*
 * Decrementer interrupt second level handler
 */
decrintr:
	INTRENTER
	addi	3,1,8			/* intr frame */
	bl	_C_LABEL(decr_intr)
	b	intr_exit

#ifdef DDB
/*
 * Deliberate entry to ddbtrap
 */
ENTRY_NOPROFILE(ddb_trap)
	mtsprg	1,1
	mfmsr	3
	mtsrr1	3
	andi.	3,3,~(PSL_EE|PSL_ME)@l
	mtmsr	3			/* disable interrupts */
	isync
	GET_CPUINFO(3)
	stmw	28,CI_DDBSAVE(3)
	mflr	28
	li	29,EXC_BPT
	mtlr	29
	mfcr	29
	mtsrr0	28

/*
 * Now the ddb trap catching code.
 */
ddbtrap:
	FRAME_SETUP(CI_DDBSAVE)
/* Call C trap code: */
	addi	3,1,8
	bl	_C_LABEL(ddb_trap_glue)
	cmpwi	3,0
	bne	ddbleave
/* This wasn't for DDB, so switch to real trap: */
	GET_CPUINFO(4)
	lwz	3,FRAME_EXC+8(1)	/* save exception */
	stw	3,CI_DDBSAVE+16(4)
	FRAME_LEAVE(CI_DDBSAVE)
	mtsprg	1,1			/* prepare for entrance to realtrap */
	GET_CPUINFO(1)
	stmw	28,CI_TEMPSAVE(1)
	mflr	28
	mfcr	29
	lwz	31,CI_DDBSAVE+16(1)
	mtlr	31
	b	realtrap
ddbleave:
	FRAME_LEAVE(CI_DDBSAVE)
	rfi
#endif /* DDB */

ENTRY_NOPROFILE(syscalltrap)
	mtsprg	1,1			/* save SP */
	GET_CPUINFO(1)
	stmw	28,CI_TEMPSAVE(1)	/* free r28-r31 */
	mflr	28			/* save LR */
	mfcr	29			/* save CR */
	lwz	1,CI_CURPCB(1)
	addi	1,1,USPACE		/* stack is top of user struct */
	bla	1f
1:
	FRAME_SETUP(CI_TEMPSAVE)
/* Now we can recover interrupts again: */
	mfmsr	7
	ori	7,7,(PSL_EE|PSL_ME|PSL_RI)@l
	mtmsr	7
	isync
	GET_CPUINFO(3)
	lwz	3,CI_CURLWP(3)
	lwz	3,L_PROC(3)
	lwz	4,P_MD_SYSCALL(3)
	mtlr	4
	addi	3,1,8
	blrl
	b	trapexit

	.lcomm	tlbstk,4096,16

ENTRY_NOPROFILE(dtlbmiss)
	mtspr	M_TW,1		/* save r1 */
	mtsprg	3,28
	mfcr	28
	mfsrr1	1
	andi.	1,1,PSL_PR
	beq	1f		/* branch if supervisor */

	mfspr	1,M_TWB		/* L1 pte (ptp) address */
	lwz	1,0(1)
	rlwinm	1,1,0,31,19	/* l1pte &= 0xfffff001 */
	mtspr	MD_TWC,1
	mfspr	1,MD_TWC	/* L2 pte address */
	lwz	1,0(1)
	rlwinm	1,1,0,~0xe00	/* l2pte &= ~0xef0 */
	rlwinm	1,1,0,~0x0f0
	ori	1,1,0x840	/* hit only user, kernel/user R/W */
	mtspr	MD_RPN,1	/* write TLB entry */
	mfspr	1,M_TW		/* restore r1 */
	mtcr	28
	mfsprg	28,3
	rfi

1:	ba	s_dtlbmiss

s_dtlbmiss:
	lis	1,tlbstk+4096@ha
	addi	1,1,tlbstk+4096@l
	stwu	1,-80(1)
	stw	0,8(1)
	stw	2,12(1)
	stw	3,16(1)
	stw	4,20(1)
	stw	5,24(1)
	stw	6,28(1)
	stw	7,32(1)
	stw	8,36(1)
	stw	9,40(1)
	stw	10,44(1)
	stw	11,48(1)
	stw	12,52(1)
	stw	13,56(1)
	stw	29,60(1)
	stw	30,64(1)
	stw	31,68(1)
	mflr	29
	mfctr	30
	mfxer	31
	mfspr	3,MD_EPN	/* miss EA */
	bl	_C_LABEL(kdtlbload)
tlbload_return:
	mtlr	29
	mtctr	30
	mtxer	31
	lwz	0,8(1)
	lwz	2,12(1)
	lwz	3,16(1)
	lwz	4,20(1)
	lwz	5,24(1)
	lwz	6,28(1)
	lwz	7,32(1)
	lwz	8,36(1)
	lwz	9,40(1)
	lwz	10,44(1)
	lwz	11,48(1)
	lwz	12,52(1)
	lwz	13,56(1)
	lwz	29,60(1)
	lwz	30,64(1)
	lwz	31,68(1)
	mfspr	1,M_TW		/* restore r1 */
	mtcr	28
	mfsprg	28,3
	rfi

ENTRY_NOPROFILE(itlbmiss)
	mtspr	M_TW,1		/* save r1 */
	mtsprg	3,28
	mfcr	28
	mfsrr1	1
	andi.	1,1,PSL_PR
	beq	1f		/* branch if supervisor */

	mfspr	1,MI_EPN	/* miss EA */
	mtspr	MD_EPN,1	/* set miss EA */
	mfspr	1,M_TWB		/* L1 pte (ptp) address */
	lwz	1,0(1)
	rlwinm	1,1,0,31,19	/* l1pte &= 0xfffff001 */
	mtspr	MI_TWC,1
	mtspr	MD_TWC,1	/* use MD_TWC to calculate L2 pte address */
	mfspr	1,MD_TWC	/* L2 pte address */
	lwz	1,0(1)
	rlwinm	1,1,0,~0xe00	/* l2pte &= ~0xef0 */
	rlwinm	1,1,0,~0x0f0
	ori	1,1,0x840	/* hit only user, kernel/user R/W */
	mtspr	MI_RPN,1	/* write TLB entry */
	mfspr	1,M_TW		/* restore r1 */
	mtcr	28
	mfsprg	28,3
	rfi

1:	ba	s_itlbmiss

s_itlbmiss:
	lis	1,tlbstk+4096@ha
	addi	1,1,tlbstk+4096@l
	stwu	1,-80(1)
	stw	0,8(1)
	stw	2,12(1)
	stw	3,16(1)
	stw	4,20(1)
	stw	5,24(1)
	stw	6,28(1)
	stw	7,32(1)
	stw	8,36(1)
	stw	9,40(1)
	stw	10,44(1)
	stw	11,48(1)
	stw	12,52(1)
	stw	13,56(1)
	stw	29,60(1)
	stw	30,64(1)
	stw	31,68(1)
	mflr	29
	mfctr	30
	mfxer	31
	mfspr	3,MI_EPN	/* miss EA */
	bl	_C_LABEL(kitlbload)
	b	tlbload_return
