/*	$Id: locore.S,v 1.44 2010/01/07 07:48:35 tsubai Exp $	*/

/*
 * Copyright (C) 1995, 1996 Wolfgang Solfrank.
 * Copyright (C) 1995, 1996 TooLs GmbH.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by TooLs GmbH.
 * 4. The name of TooLs GmbH may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TOOLS GMBH ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL TOOLS GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "opt_ddb.h"
#include "opt_lockdebug.h"
#include "assym.h"

#include <sys/syscall.h>

#include <machine/param.h>
#include <machine/asm.h>

/*
 * Globals
 */
	.data
GLOBAL(esym)
	.long	0			/* end of symbol table */
GLOBAL(proc0paddr)
	.long	0			/* proc0 p_addr */

GLOBAL(intrnames)
	.asciz	"vec 0", "vec 1", "vec 2", "vec 3"
	.asciz	"vec 4", "vec 5", "vec 6", "vec 7"
	.asciz	"vec 8", "vec 9", "vec 10", "vec 11"
	.asciz	"vec 12", "vec 13", "vec 14", "vec 15"
	.asciz	"clock"
GLOBAL(eintrnames)
	.align	4
GLOBAL(intrcnt)
	.long	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
	.long	0,0,0,0
GLOBAL(eintrcnt)

/*
 * This symbol is here for the benefit of kvm_mkdb, and is supposed to
 * mark the start of kernel text.
 */
	.text
	.globl	_C_LABEL(kernel_text)
_C_LABEL(kernel_text):

	.type	_start,@function
	.globl	_start
_start:
	li	0,0
	mtmsr	0
	isync

	li	7,PGOFSET
	add	4,4,7
	andc	4,4,7			/* esym = roundup(esym, NBPG) */

	lis	7,_C_LABEL(proc0paddr)@ha
	stw	4,_C_LABEL(proc0paddr)@l(7)
	addi	4,4,USPACE

	addi	1,4,-FRAMELEN
	stw	0,0(1)

	bl	_C_LABEL(initppc)
	bl	_C_LABEL(main)
	b	.


/*
 * struct lwp *
 * cpu_switchto(struct lwp *current, struct lwp *new)
 * Switch to the indicated new LWP.
 * 	r3 - current LWP
 * 	r4 - LWP to switch to
 */
ENTRY(cpu_switchto)
	mflr	0			/* save lr */
	stw	0,4(1)
	stwu	1,-16(1)
	stw	31,12(1)
	stw	30,8(1)

	mr	30,3			/* r30 = curlwp */
	mr	31,4			/* r31 = newlwp */

	cmpwi	30,0			/* old lwp was exiting? */
	beq	switch_exited

	mfcr	11			/* save cr */
	mr	12,2			/* save r2 */
	stwu	1,-SFRAMELEN(1)		/* still running on old stack */
	stmw	11,12(1)
	lwz	5,L_ADDR(30)
	stw	1,PCB_SP(5)		/* save SP */

switch_exited:
	GET_CPUINFO(5)
	stw	31,CI_CURLWP(5)		/* record new process */
	lwz	6,L_ADDR(31)
	stwu	6,CI_CURPCB(5)		/* indicate new pcb */
	stwcx.	6,0,5			/* clear possible reservation */

	lwz	1,PCB_SP(6)		/* get new procs SP */
	lmw	11,12(1)		/* get other regs */
	lwz	1,0(1)			/* get saved SP */
	mr	2,12			/* get saved r2 */
	mtcr	11			/* get saved cr */

	lwz	31,12(1)
	lwz	30,8(1)
	addi	1,1,16
	lwz	0,4(1)
	mtlr	0
	blr

/*
 * int setfault()
 *
 * Similar to setjmp to setup for handling faults on accesses to user memory.
 * Any routine using this may only call bcopy, either the form below,
 * or the (currently used) C code optimized, so it doesn't use any non-volatile
 * registers.
 */
ENTRY(setfault)
	mflr	0
	mfcr	12
	GET_CPUINFO(4)
	lwz	4,CI_CURPCB(4)
	stw	3,PCB_FAULT(4)
	stw	0,0(3)
	stw	1,4(3)
	stw	2,8(3)
	stmw	12,12(3)
	sync
	isync
	li	3,0
	blr

/*
 * int _lock_cas(uintptr_t *ptr, uintptr_t old, uintptr_t new);
 * XXX Assume r3 != r1.
 */
ENTRY_NOPROFILE(_lock_cas)
1:	lwz	6,0(3)
	cmpw	6,4
	bne	3f

	lwarx	6,0,3
	cmpw	6,4
	bne	2f

	stwcx.	5,0,3
	bne-	1b

	li	3,1
	blr

2:	addi	3,1,-4
	stwcx.	3,0,3			/* Clear reservation. */
3:	li	3,0
	blr

#ifndef LOCKDEBUG
/*
 * void mutex_enter(kmutex_t *);
 */
ENTRY_NOPROFILE(mutex_enter)
	GET_CPUINFO(4)
	lwz	5,CI_CURLWP(4)
1:
	lwarx	0,0,3
	cmpwi	0,0
	bne	2f

	stwcx.	5,0,3
	bne-	1b
	isync
	blr
2:
	addi	4,1,-4
	stwcx.	4,0,4			/* Clear reservation. */
	b	_C_LABEL(mutex_vector_enter)

/*
 * void mutex_exit(kmutex_t *);
 */
ENTRY_NOPROFILE(mutex_exit)
	GET_CPUINFO(4)
	lwz	5,CI_CURLWP(4)

	lwz	0,0(3)
	cmpw	0,5
	bne	2f

	sync
	li	0,0
	stw	0,0(3)
	blr
2:
	b	_C_LABEL(mutex_vector_exit)
#endif /* !LOCKDEBUG */

/*
 * Signal trampoline; copied to top of user stack.
 */
	.globl	_C_LABEL(sigcode),_C_LABEL(esigcode)
_C_LABEL(sigcode):
	addi	1,1,-16			/* reserved space for callee */
	mr	31,5			/* save scp */
	blrl
	mr	3,31			/* restore scp */
	li	0,SYS_compat_16___sigreturn14
	sc				/* sigreturn(scp) */
	li	0,SYS_exit
	sc				/* exit(errno) */
_C_LABEL(esigcode):

	.align 4
ENTRY_NOPROFILE(setplprcr)
	mfimmr	4			/* 00 */
	rlwinm	4,4,0,0,15		/* 04 */
	nop				/* 08 */
	eieio				/* 0c */
	stw	3,644(4)		/* 10 */
	divw	3,4,4			/* 14 */
	divw	3,4,4			/* 18 */
	isync				/* 1c */
	blr

user_text = 0x01800000
