/*	$Id: pio.h,v 1.16 2003/07/07 14:55:17 tsubai Exp $	*/

/*
 * Copyright (c) 1997 Per Fogelstrom, Opsycon AB and RTMX Inc, USA.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed under OpenBSD by
 *	Per Fogelstrom Opsycon AB for RTMX Inc, North Carolina, USA.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef _POWERPC_PIO_H_
#define _POWERPC_PIO_H_

/*
 * I/O macros.
 */
static inline void __out8 __P((volatile u_int8_t *, u_int));
static inline void __out16 __P((volatile u_int16_t *, u_int));
static inline void __out32 __P((volatile u_int32_t *, u_int));
static inline void __out16rb __P((volatile u_int16_t *, u_int));
static inline void __out32rb __P((volatile u_int32_t *, u_int));

static inline u_int __in8 __P((volatile u_int8_t *));
static inline u_int __in16 __P((volatile u_int16_t *));
static inline u_int __in32 __P((volatile u_int32_t *));
static inline u_int __in16rb __P((volatile u_int16_t *));
static inline u_int __in32rb __P((volatile u_int32_t *));

static inline void __outs8 __P((volatile u_int8_t *, const u_int8_t *,
	u_int));
static inline void __outs16 __P((volatile u_int16_t *, const u_int16_t *,
	u_int));
static inline void __outs32 __P((volatile u_int32_t *, const u_int32_t *,
	u_int));
static inline void __outs16rb __P((volatile u_int16_t *, const u_int16_t *,
	u_int));
static inline void __outs32rb __P((volatile u_int32_t *, const u_int32_t *,
	u_int));
static inline void __ins8 __P((volatile u_int8_t *, u_int8_t *, u_int));
static inline void __ins16 __P((volatile u_int16_t *, u_int16_t *, u_int));
static inline void __ins32 __P((volatile u_int32_t *, u_int32_t *, u_int));
static inline void __ins16rb __P((volatile u_int16_t *, u_int16_t *, u_int));
static inline void __ins32rb __P((volatile u_int32_t *, u_int32_t *, u_int));

static inline void
__out8(a, v)
	volatile u_int8_t *a;
	u_int v;
{
	*a = v;
	asm volatile ("eieio; sync");
}

static inline void
__out16(a, v)
	volatile u_int16_t *a;
	u_int v;
{
	*a = v;
	asm volatile ("eieio; sync");
}

static inline void
__out32(a, v)
	volatile u_int32_t *a;
	u_int v;
{
	*a = v;
	asm volatile ("eieio; sync");
}

static inline void
__out16rb(a, v)
	volatile u_int16_t *a;
	u_int v;
{
	asm volatile ("sthbrx %0,0,%1" :: "r"(v), "r"(a));
	asm volatile ("eieio; sync");
}

static inline void
__out32rb(a, v)
	volatile u_int32_t *a;
	u_int v;
{
	asm volatile ("stwbrx %0,0,%1" :: "r"(v), "r"(a));
	asm volatile ("eieio; sync");
}

static inline u_int
__in8(a)
	volatile u_int8_t *a;
{
	u_int v;

	v = *a;
	asm volatile ("eieio; sync");
	return v;
}

static inline u_int
__in16(a)
	volatile u_int16_t *a;
{
	u_int v;

	v = *a;
	asm volatile ("eieio; sync");
	return v;
}

static inline u_int
__in32(a)
	volatile u_int32_t *a;
{
	u_int v;

	v = *a;
	asm volatile ("eieio; sync");
	return v;
}

static inline u_int
__in16rb(a)
	volatile u_int16_t *a;
{
	u_int v;

	asm volatile ("lhbrx %0,0,%1" : "=r"(v) : "r"(a));
	asm volatile ("eieio; sync");
	return v;
}

static inline u_int
__in32rb(a)
	volatile u_int32_t *a;
{
	u_int v;

	asm volatile ("lwbrx %0,0,%1" : "=r"(v) : "r"(a));
	asm volatile ("eieio; sync");
	return v;
}

static inline void
__outs8(a, s, c)
	volatile u_int8_t *a;
	const u_int8_t *s;
	u_int c;
{
	while (c--)
		*a = *s++;
	asm volatile ("eieio; sync");
}

static inline void
__outs16(a, s, c)
	volatile u_int16_t *a;
	const u_int16_t *s;
	u_int c;
{
	while (c--)
		*a = *s++;
	asm volatile ("eieio; sync");
}

static inline void
__outs32(a, s, c)
	volatile u_int32_t *a;
	const u_int32_t *s;
	u_int c;
{
	while (c--)
		*a = *s++;
	asm volatile ("eieio; sync");
}

static inline void
__outs16rb(a, s, c)
	volatile u_int16_t *a;
	const u_int16_t *s;
	u_int c;
{
	while (c--)
		asm volatile ("sthbrx %0,0,%1" :: "r"(*s++), "r"(a));
	asm volatile ("eieio; sync");
}

static inline void
__outs32rb(a, s, c)
	volatile u_int32_t *a;
	const u_int32_t *s;
	u_int c;
{
	while (c--)
		asm volatile ("stwbrx %0,0,%1" :: "r"(*s++), "r"(a));
	asm volatile ("eieio; sync");
}

static inline void
__ins8(a, d, c)
	volatile u_int8_t *a;
	u_int8_t *d;
	u_int c;
{
	while (c--)
		*d++ = *a;
	asm volatile ("eieio; sync");
}

static inline void
__ins16(a, d, c)
	volatile u_int16_t *a;
	u_int16_t *d;
	u_int c;
{
	while (c--)
		*d++ = *a;
	asm volatile ("eieio; sync");
}

static inline void
__ins32(a, d, c)
	volatile u_int32_t *a;
	u_int32_t *d;
	u_int c;
{
	while (c--)
		*d++ = *a;
	asm volatile ("eieio; sync");
}

static inline void
__ins16rb(a, d, c)
	volatile u_int16_t *a;
	u_int16_t *d;
	u_int c;
{
	while (c--)
		asm volatile ("lhbrx %0,0,%1" : "=r"(*d++) : "r"(a));
	asm volatile ("eieio; sync");
}

static inline void
__ins32rb(a, d, c)
	volatile u_int32_t *a;
	u_int32_t *d;
	u_int c;
{
	while (c--)
		asm volatile ("lwbrx %0,0,%1" : "=r"(*d++) : "r"(a));
	asm volatile ("eieio; sync");
}

#define in8(a)		__in8((volatile u_int8_t *)(a))
#define in16(a)		__in16((volatile u_int16_t *)(a))
#define in32(a)		__in32((volatile u_int32_t *)(a))
#define in16rb(a)	__in16rb((volatile u_int16_t *)(a))
#define in32rb(a)	__in32rb((volatile u_int32_t *)(a))

#define out8(a,v)	__out8((volatile u_int8_t *)(a), v)
#define out16(a,v)	__out16((volatile u_int16_t *)(a), v)
#define out32(a,v)	__out32((volatile u_int32_t *)(a), v)
#define out16rb(a,v)	__out16rb((volatile u_int16_t *)(a), v)
#define out32rb(a,v)	__out32rb((volatile u_int32_t *)(a), v)

#define ins8(a,d,c)	__ins8((volatile u_int8_t *)(a), d, c)
#define ins16(a,d,c)	__ins16((volatile u_int16_t *)(a), d, c)
#define ins32(a,d,c)	__ins32((volatile u_int32_t *)(a), d, c)
#define ins16rb(a,d,c)	__ins16rb((volatile u_int16_t *)(a), d, c)
#define ins32rb(a,d,c)	__ins32rb((volatile u_int32_t *)(a), d, c)

#define outs8(a,s,c)	__outs8((volatile u_int8_t *)(a), s, c)
#define outs16(a,s,c)	__outs16((volatile u_int16_t *)(a), s, c)
#define outs32(a,s,c)	__outs32((volatile u_int32_t *)(a), s, c)
#define outs16rb(a,s,c)	__outs16rb((volatile u_int16_t *)(a), s, c)
#define outs32rb(a,s,c)	__outs32rb((volatile u_int32_t *)(a), s, c)

#endif /*_POWERPC_PIO_H_*/
