/*	$Id: memcpy.S,v 1.9 2007/03/24 06:28:40 tsubai Exp $	*/

/*-
 * Copyright (c) 2005 Tsubai Masanari.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <machine/asm.h>

ENTRY_NOPROFILE(bcopy)		/* src = r3, dst = r4, len = r5 */
	mr	6,3
	mr	3,4
	mr	4,6
	/* FALLTHROUGH */

ENTRY_NOPROFILE(memmove)
ENTRY(memcpy)			/* dst = r3(r6), src = r4, len = r5 */
	cmpwi	5,0		/* Return if len == 0. */
	beqlr

	cmplwi	5,24		/* Special case if len <= 24. */
	bgt	1f
	mtxer	5
	lswx	7,0,4
	stswx	7,0,3
	blr

1:	cmplw	3,4
	beqlr			/* Return if dst == src. */
	bgt	backward

	/*
	 * Forward copy.
	 */
	mr	6,3
	andi.	7,6,3
	beq	2f		/* Skip if (dst & 3) == 0. */

	/* Copy leading unaligned bytes. */
	subfic	7,7,4
	lwz	0,0(4)		/* Load 1-3 bytes. */
	add	4,4,7
	mtxer	7
	stswx	0,0,6		/* Store them. */
	add	6,6,7
	sub	5,5,7

2:	/* dst is aligned here. */
	srwi.	7,5,2		/* r7 = len >> 2 */
	beq	4f
	mtctr	7
3:	lwz	0,0(4)		/* Word copy. */
	addi	4,4,4
	stw	0,0(6)
	addi	6,6,4
	bdnz	3b

4:	andi.	7,5,3
	beqlr

	/* Copy trailing bytes. */
	mtxer	7
	lswx	8,0,4
	stswx	8,0,6
	blr

	/*
	 * Backward copy.
	 */
backward:
	add	4,4,5		/* src += len */
	add	6,3,5		/* dst += len */

	andi.	7,6,3
	beq	5f		/* Skip if (dst & 3) == 0. */

	mtxer	7
	sub	4,4,7
	lswx	8,0,4
	sub	6,6,7
	stswx	8,0,6
	sub	5,5,7		/* len -= (dst & 3) */

5:	/* dst is aligned here. */
	srwi.	7,5,2		/* r7 = len >> 2 */
	beq	7f
	mtctr	7
6:	lwzu	0,-4(4)		/* Word copy. */
	stwu	0,-4(6)
	bdnz	6b

7:	andi.	7,5,3
	beqlr

	mtxer	7
	neg	7,7
	lwzx	0,7,4
	stswx	0,7,6
	blr
