/*	$Id: cpu_750fx.c,v 1.23 2010/11/27 12:14:53 tsubai Exp $	*/

/*-
 * Copyright (c) 2002-2005 Tsubai Masanari.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/param.h>
#include <sys/reboot.h>
#include <sys/systm.h>

#include <machine/bat.h>
#include <machine/cpufunc.h>
#include <machine/spr.h>

/*
 * 750FX specific part.
 */
#define HID0_EMCP    0x80000000	/* Enable MCP */
#define HID0_DBP     0x40000000	/* Disable 60x bus parity generation */
#define HID0_EBA     0x20000000	/* Enable 60x bus address parity checking */
#define HID0_EBD     0x10000000	/* Enable 60x bus data parity checking */
#define HID0_PAR     0x01000000	/* Disable precharge of ARTRY */
#define HID0_DOZE    0x00800000	/* Enable doze mode */
#define HID0_NAP     0x00400000	/* Enable nap mode */
#define HID0_SLEEP   0x00200000	/* Enable sleep mode */
#define HID0_DPM     0x00100000	/* Enable Dynamic power management */
#define HID0_RISEG   0x00080000	/* Read I-SEG */
#define HID0_MUM     0x00020000	/* Enable Miss Under Miss */
#define HID0_NHR     0x00010000	/* Not hard reset */
#define HID0_ICE     0x00008000	/* Enable i-cache */
#define HID0_DCE     0x00004000	/* Enable d-cache */
#define HID0_ILOCK   0x00002000	/* i-cache lock */
#define HID0_DLOCK   0x00001000	/* d-cache lock */
#define HID0_ICFI    0x00000800	/* i-cache flush invalidate */
#define HID0_DCFI    0x00000400	/* d-cache flush invalidate */
#define HID0_SPD     0x00000200	/* Disable speculative cache access */
#define HID0_IFEM    0x00000100	/* Enable M-bit for I-fetch */
#define HID0_SGE     0x00000080	/* Enable store gathering */
#define HID0_DCFA    0x00000040	/* Data cache flush assist */
#define HID0_BTIC    0x00000020	/* Enable BTIC */
#define HID0_ABE     0x00000008	/* Enable address broadcast */
#define HID0_BHT     0x00000004	/* Enable branch history table */
#define HID0_NOOPTI  0x00000001	/* No-op the dcbt(st) */

#define HID0_BITMASK "\020" \
    "\040EMCP\037DBP\036EBA\035EBD\031PAR" \
    "\030DOZE\027NAP\026SLEEP\025DPM\024RISEG\022MUM\021NHR" \
    "\020ICE\017DCE\016ILOCK\015DLOCK\014ICFI\013DCFI\012SPD\011IFEM" \
    "\010SGE\007DCFA\006BTIC\004ABE\003BHT\001NOOPTI"

#define HID1_PCE0    0x80000000	/* PLL external configuration bit 0 (RO) */
#define HID1_PCE1    0x40000000	/* PLL external configuration bit 1 (RO) */
#define HID1_PCE2    0x20000000	/* PLL external configuration bit 2 (RO) */
#define HID1_PCE3    0x10000000	/* PLL external configuration bit 3 (RO) */
#define HID1_PCE4    0x08000000	/* PLL external configuration bit 4 (RO) */
#define HID1_PRE0    0x04000000	/* PLL external range bit 0 (RO) */
#define HID1_PRE1    0x02000000	/* PLL external range bit 1 (RO) */
#define HID1_PSS     0x01000000	/* Processor switch status */
#define HID1_ECLK    0x00800000	/* Enable CLKOUT pin */
#define HID1_CLKSEL  0x00700000	/* CLKOUT selection */
#define HID1_PI0     0x00020000	/* PLL0 internal configuration select */
#define HID1_PS      0x00010000	/* PLL select */
#define HID1_PC00    0x00008000	/* PLL0 configuration bit 0 */
#define HID1_PC01    0x00004000	/* PLL0 configuration bit 1 */
#define HID1_PC02    0x00002000	/* PLL0 configuration bit 2 */
#define HID1_PC03    0x00001000	/* PLL0 configuration bit 3 */
#define HID1_PC04    0x00000800	/* PLL0 configuration bit 4 */
#define HID1_PR00    0x00000400	/* PLL0 range selection bit 0 */
#define HID1_PR01    0x00000200	/* PLL0 range selection bit 1 */
#define HID1_PC10    0x00000080	/* PLL1 configuration bit 0 */
#define HID1_PC11    0x00000040	/* PLL1 configuration bit 1 */
#define HID1_PC12    0x00000020	/* PLL1 configuration bit 2 */
#define HID1_PC13    0x00000010	/* PLL1 configuration bit 3 */
#define HID1_PC14    0x00000008	/* PLL1 configuration bit 4 */
#define HID1_PR10    0x00000004	/* PLL1 range selection bit 0 */
#define HID1_PR11    0x00000002	/* PLL1 range selection bit 1 */

#define HID1_BITMASK "\020" \
    "\040PCE0\037PCE1\036PCE2\035PCE3\034PCE4\033PRE0\032PRE1\031PSS" \
    "\030ECLK\022PI0\021PS" \
    "\020PC00\017PC01\016PC02\015PC03\014PC04\013PR00\012PR01" \
    "\010PC10\007PC11\006PC12\005PC13\004PC14\003PR10\002PR11"

#define L2CR_L2E     0x80000000	/* 0: L2 enable */
#define L2CR_L2CE    0x40000000	/* 1: L2 checkstop enable */
#define L2CR_L2DO    0x00400000	/* 9: L2 data only */
#define L2CR_L2I     0x00200000	/* 10: L2 global invalidate */
#define L2CR_L2WT    0x00080000	/* 12: write-through */
#define L2CR_L2TS    0x00040000	/* 13: test support */
#define L2CR_L2LOCK0 0x00000800	/* 20: L2 lock[0] */
#define L2CR_L2LOCK1 0x00000400	/* 21: L2 lock[1] */
#define L2CR_L2SHEE  0x00000200	/* 22: snoop hit in locked line error enable */
#define L2CR_L2SHERR 0x00000100	/* 23: snoop hit in locked line error */
#define L2CR_L2LOCK  0x000000f0	/* 24-27: cache lock control (GX) */
#define L2CR_L2IO    0x00000008	/* 28: L2 instruction only (GX) */
#define L2CR_L2IP    0x00000001	/* 31: L2 global invalidate in progress */

#define L2CR_BITMASK "\020" \
    "\040L2E\037L2CE\027L2DO\026L2I\024L2WT\023L2TS" \
    "\014L2LOCK0\013L2LOCK1\012L2SHEE\011L2SHERR\001L2IP"

asm ("IBAT4U = 560");
asm ("IBAT5U = 562");
asm ("IBAT6U = 564");
asm ("IBAT7U = 566");
asm ("DBAT4U = 568");
asm ("DBAT5U = 570");
asm ("DBAT6U = 572");
asm ("DBAT7U = 574");

void ppc750fx_enable_l2cache(void);

static int cpu_pvr;

/*
 * XXX If L2_TSTCLK is pulled low, the PVR may read 0x000802xx.
 */
int
ppc750fx_probe()
{
	u_int pvr, hid1, l2cr, x;

	if (cpu_pvr) {
		switch (cpu_pvr >> 16) {
		case PPC750FX:
		case PPC750GX:
			return cpu_pvr >> 16;
		}
		return 0;
	}

	asm ("mfpvr %0" : "=r"(pvr));
	cpu_pvr = pvr;

	switch (pvr >> 16) {
	case PPC750FX:
	case PPC750GX:
		return pvr >> 16;
	}

	if ((pvr & 0xffffff00) != 0x00080200)	/* Not 750 or 750[FG]X */
		return 0;

	asm volatile ("mfspr %0,HID1" : "=r"(hid1));

#define HID1_PC1 (HID1_PC10|HID1_PC11|HID1_PC12|HID1_PC13|HID1_PC14)

	if (hid1 & HID1_PS)
		goto fxgx;
	if (hid1 & HID1_PC1)
		goto fxgx;

	asm volatile ("mtspr HID1,%0" :: "r"(hid1 | HID1_PC1));
	delay(10);
	asm volatile ("mfspr %0,HID1" : "=r"(x));
	asm volatile ("mtspr HID1,%0" :: "r"(hid1));

	if (x & HID1_PC1)
		goto fxgx;

	return 0;

fxgx:
	asm volatile ("mfspr %0,L2CR" : "=r"(l2cr));
	asm volatile ("sync");
	asm volatile ("mtspr L2CR,%0" :: "r"(l2cr | L2CR_L2IO));
	asm volatile ("mfspr %0,L2CR" : "=r"(x));
	asm volatile ("mtspr L2CR,%0" :: "r"(l2cr));
	asm volatile ("sync; isync");

	pvr &= 0xffff;
	if (x & L2CR_L2IO)
		pvr |= PPC750GX << 16;
	else
		pvr |= PPC750FX << 16;

	cpu_pvr = pvr;
	return pvr >> 16;
}

/*
 * Invalidate bat0 - bat7.
 */
void
ppc750fx_clearbat()
{

	asm volatile ("mtibatu 0,%0" :: "r"(0));
	asm volatile ("mtibatu 1,%0" :: "r"(0));
	asm volatile ("mtibatu 2,%0" :: "r"(0));
	asm volatile ("mtibatu 3,%0" :: "r"(0));
#if 0 /* XXX Avoid assembler error. */
	asm volatile ("mtibatu 4,%0" :: "r"(0));
	asm volatile ("mtibatu 5,%0" :: "r"(0));
	asm volatile ("mtibatu 6,%0" :: "r"(0));
	asm volatile ("mtibatu 7,%0" :: "r"(0));
#else
	asm volatile ("mtspr IBAT4U,%0" :: "r"(0));
	asm volatile ("mtspr IBAT5U,%0" :: "r"(0));
	asm volatile ("mtspr IBAT6U,%0" :: "r"(0));
	asm volatile ("mtspr IBAT7U,%0" :: "r"(0));
#endif

	asm volatile ("mtdbatu 0,%0" :: "r"(0));
	asm volatile ("mtdbatu 1,%0" :: "r"(0));
	asm volatile ("mtdbatu 2,%0" :: "r"(0));
	asm volatile ("mtdbatu 3,%0" :: "r"(0));
#if 0 /* XXX Avoid assembler error. */
	asm volatile ("mtdbatu 4,%0" :: "r"(0));
	asm volatile ("mtdbatu 5,%0" :: "r"(0));
	asm volatile ("mtdbatu 6,%0" :: "r"(0));
	asm volatile ("mtdbatu 7,%0" :: "r"(0));
#else
	asm volatile ("mtspr DBAT4U,%0" :: "r"(0));
	asm volatile ("mtspr DBAT5U,%0" :: "r"(0));
	asm volatile ("mtspr DBAT6U,%0" :: "r"(0));
	asm volatile ("mtspr DBAT7U,%0" :: "r"(0));
#endif
}

#define PLL_RNG_DEFAULT 0  /* 600 MHz - 900 MHz */
#define PLL_RNG_FAST    1  /* 900 MHz - 1.0 GHz (GX) */
#define PLL_RNG_SLOW    2  /* 500 MHz - 600 MHz */

#ifdef PPC750FX_PLL
#define PLLVAL(x) ((x) < 10 ? (x) * 2 : (x) + 10)
static u_int pll_cfg = PLLVAL(PPC750FX_PLL);
#else
static u_int pll_cfg = 0;
#endif

void
ppc750fx_config_hid()
{
	u_int hid0, hid1, pll, pll_rng, old_cfg;

	asm ("mfspr %0,HID0" : "=r"(hid0));

	hid0 &= HID0_EBA | HID0_EBD | HID0_PAR | HID0_MUM | HID0_ICE | HID0_DCE;
	hid0 |= HID0_EMCP | HID0_DOZE | HID0_DPM;
	hid0 |= HID0_SGE | HID0_BTIC | HID0_ABE | HID0_BHT;

	asm volatile ("mtspr HID0,%0" :: "r"(hid0));

	if (pll_cfg == 0 || pll_cfg > 30)
		return;

	/*
	 * Configure PLL.
	 * XXX Assume bus clock is 100MHz.
	 */
	if (pll_cfg < 12)
		pll_rng = PLL_RNG_SLOW;
	else if (pll_cfg < 18)
		pll_rng = PLL_RNG_DEFAULT;
	else
		pll_rng = PLL_RNG_FAST;		/* XXX GX only */

	pll = pll_cfg << 3 | pll_rng << 1;
	asm ("mfspr %0,HID1" : "=r"(hid1));

	if (hid1 & HID1_PS) {
		/* Use PLL0 */
		hid1 &= ~0xff00;
		hid1 |= pll << 8;
		hid1 |= HID1_PI0;
		old_cfg = (hid1 & 0xff) >> 3;
	} else {
		/* Use PLL1 */
		hid1 &= ~0xff;
		hid1 |= pll;
		if (hid1 & HID1_PI0)
			old_cfg = (hid1 & 0xff00) >> 11;
		else
			old_cfg = (hid1 & 0xff000000) >> 27;
	}
	/* printf("Changing old PLL %d to new PLL %d\n", old_cfg, pll_cfg); */

	/* We can't switch when both are half-cycle mode. */
	if (old_cfg < 20 && pll_cfg < 20 && (old_cfg & pll_cfg & 1)) {
		printf("ppc750fx_config_hid: can't switch\n");
		return;
	}

	asm volatile ("sync; isync");
	asm volatile ("mtspr HID1, %0" :: "r"(hid1));

	/* Wait for PLL lock.  (max 100us) */
	delay(1000);

	hid1 ^= HID1_PS;
	asm volatile ("mtspr HID1, %0" :: "r"(hid1));
	asm volatile ("isync");
}

/*
 * Print HID[01] configuration.
 */
void
ppc750fx_print_hid(name)
	const char *name;
{
	u_int hid0, hid1;
	char buf[160];

	asm ("mfspr %0,HID0" : "=r"(hid0));
	asm ("mfspr %0,HID1" : "=r"(hid1));

	snprintb(buf, sizeof buf, HID0_BITMASK, hid0);
	printf("%s: HID0 %s\n", name, buf);
	snprintb(buf, sizeof buf, HID1_BITMASK, hid1);
	printf("%s: HID1 %s\n", name, buf);
}

u_int
ppc750fx_corefreq(busfreq)
	u_int busfreq;
{
	u_int hid1, cfg;

	static const char pll_mul2[] = {
		 0,  0,  2,  2,  4,  5,  6,  7,
		 8,  9, 10, 11, 12, 13, 14, 15,
		16, 17, 18, 19, 20, 22, 24, 26,
		28, 30, 32, 34, 36, 38, 40,  0
	};

	asm ("mfspr %0,HID1" : "=r"(hid1));

	switch (hid1 & (HID1_PS|HID1_PI0)) {
	case 0:
		cfg = (hid1 & 0xff000000) >> 27;	/* PLL0 external */
		break;
	case HID1_PI0:
		cfg = (hid1 & 0xff00) >> 11;		/* PLL0 */
		break;
	/* case HID1_PS: */
	/* case HID1_PS|HID1_PI0: */
	default:
		cfg = (hid1 & 0xff) >> 3;		/* PLL1 */
		break;
	}

	return busfreq * pll_mul2[cfg] / 2;
}

void
ppc750fx_config_l2cr(name)
	const char *name;
{
	u_int l2cr;

	asm ("mfspr %0,L2CR" : "=r"(l2cr));

	if ((l2cr & L2CR_L2E) == 0) {
		ppc750fx_enable_l2cache();
		asm volatile ("mfspr %0,L2CR" : "=r"(l2cr));
	}

	switch (cpu_pvr >> 16) {
	case PPC750GX:
		printf("%s: 1MB on-chip L2 cache\n", name);
		break;
	case PPC750FX:
		printf("%s: 512KB on-chip L2 cache\n", name);
		break;
	}
	l2cr &= L2CR_L2E|L2CR_L2CE;
	asm volatile ("mtspr L2CR,%0" :: "r"(l2cr));
	asm volatile ("sync");
}

void
ppc750fx_enable_l2cache()
{
	u_int msr, hid0, l2cr;

	/* Disable interrupts and DPM. */
	msr = mfmsr();
	mtmsr(msr & ~PSL_EE);

	asm ("mfspr %0,HID0" : "=r"(hid0));
	asm volatile ("mtspr HID0,%0" :: "r"(hid0 & ~HID0_DPM));

	/* Perform L2 global invalidate. */
	asm volatile ("sync; isync");

	asm volatile ("mtspr L2CR,%0" :: "r"(L2CR_L2I));

	do {
		asm volatile ("mfspr %0,L2CR" : "=r"(l2cr));
	} while (l2cr & L2CR_L2IP);

	asm volatile ("mtspr L2CR,%0" :: "r"(L2CR_L2CE));
	asm volatile ("isync");
	asm volatile ("mtspr L2CR,%0" :: "r"(L2CR_L2E | L2CR_L2CE));

	/* Restore DPM and PSL_EE. */
	asm volatile ("mtspr HID0,%0" :: "r"(hid0));
	mtmsr(msr);
}
