/*	$Id: exception.S,v 1.54 2012/03/01 08:48:34 tsubai Exp $	*/

/*
 * Copyright (C) 1995, 1996 Wolfgang Solfrank.
 * Copyright (C) 1995, 1996 TooLs GmbH.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by TooLs GmbH.
 * 4. The name of TooLs GmbH may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TOOLS GMBH ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL TOOLS GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "opt_ddb.h"
#include "opt_ipkdb.h"
#include "assym.h"

#include <sys/syscall.h>

#include <machine/param.h>
#include <machine/asm.h>
#include <machine/bat.h>
#include <machine/pmap.h>
#include <machine/psl.h>
#include <machine/spr.h>
#include <machine/trap.h>

/*
 * Some instructions gas doesn't understand (yet?)
 */
#define bdneq	bdnzf 2,

/*
 * This code gets copied to all the trap vectors
 * (except ISI/DSI, ALI, the interrupts, and possibly the debugging
 * traps when using IPKDB).
 */
ENTRY_NOPROFILE(trapcode)
	mtsprg	1,1			/* save SP */
	GET_CPUINFO(1)
	stmw	28,CI_TEMPSAVE(1)	/* free r28-r31 */
	mflr	28			/* save LR */
	mfcr	29			/* save CR */
/* Test whether we already had PR set */
	mfsrr1	31
	andi.	31,31,PSL_PR
	beql	1f			/* branch if supervisor */
	lwz	1,CI_CURPCB(1)
	addi	1,1,USPACE		/* stack is top of user struct */
	bl	2f
1:
	mfsprg	1,1			/* restore SP */
2:
	mflr	31
	rlwinm	31,31,0,0,23		/* r31 &= ~0xff */
	mtlr	31
	ba	s_trap

/*
 * For ALI: has to save DSISR and DAR
 */
ENTRY_NOPROFILE(alitrap)
	mtsprg	1,1			/* save SP */
	GET_CPUINFO(1)
	stmw	28,CI_TEMPSAVE(1)	/* free r28-r31 */
	mfdar	30
	mfdsisr	31
	stmw	30,CI_TEMPSAVE+16(1)
	mflr	28			/* save LR */
	mfcr	29			/* save CR */
/* Test whether we already had PR set */
	mfsrr1	31
	andi.	31,31,PSL_PR
	beq	1f			/* branch if supervisor */
	lwz	1,CI_CURPCB(1)
	addi	1,1,USPACE		/* stack is top of user struct */
	li	31,EXC_ALI
	mtlr	31
	ba	s_trap
1:
	mfsprg	1,1			/* restore SP */
	li	31,EXC_ALI
	mtlr	31
	ba	s_trap

/*
 * Similar to the above for DSI
 * Has to handle BAT spills
 * and standard pagetable spills
 */
ENTRY_NOPROFILE(dsitrap)
	mtsprg	1,1			/* save SP */
	GET_CPUINFO(1)
	stmw	28,CI_DISISAVE(1)	/* free r28-r31 */
	mfcr	29			/* save CR */
	mfsrr1	31			/* test kernel mode */
	andi.	31,31,PSL_PR
	bne	1f			/* branch if user mode */
	mfdsisr	31
	andis.	31,31,DSISR_DABR@h	/* DABR match */
	bne	1f
	mfdar	31			/* get fault address */
	rlwinm	31,31,7,25,28		/* get segment * 8 */

	/* get batu */
	addis	31,31,_C_LABEL(battable)@ha
	lwz	30,_C_LABEL(battable)@l(31)
	andi.	28,30,BATU_Vs
	beq	1f			/* branch if supervisor valid is
					   false */
	/* get batl */
	lwz	31,_C_LABEL(battable)+4@l(31)
/* We randomly use the highest two bat registers here */
	mftb	28
	andi.	28,28,1
	bne	2f
	mtdbatu	2,30
	mtdbatl	2,31
	b	3f
2:
	mtdbatu	3,30
	mtdbatl	3,31
3:
	mtcr	29			/* restore CR */
	lmw	28,CI_DISISAVE(1)	/* restore r28-r31 */
	mfsprg	1,1			/* restore SP */
	rfi				/* return to trapped code */
1:
	mflr	28			/* save LR */
	li	31,EXC_DSI
	mtlr	31
	ba	s_dsitrap

/*
 * Similar to the above for ISI
 */
ENTRY_NOPROFILE(isitrap)
	mtsprg	1,1			/* save SP */
	GET_CPUINFO(1)
	stmw	28,CI_DISISAVE(1)	/* free r28-r31 */
	mfcr	29			/* save CR */
	mfsrr1	31			/* test kernel mode */
	andi.	31,31,PSL_PR
	bne	1f			/* branch if user mode */
	mfsrr0	31			/* get fault address */
	rlwinm	31,31,7,25,28		/* get segment * 8 */

	/* get batu */
	addis	31,31,_C_LABEL(battable)@ha
	lwz	30,_C_LABEL(battable)@l(31)
	andi.	28,30,BATU_Vs
	beq	1f			/* branch if supervisor valid is
					   false */
	mtibatu	3,30

	/* get batl */
	lwz	30,_C_LABEL(battable)+4@l(31)
	mtibatl	3,30

	mtcr	29			/* restore CR */
	lmw	28,CI_DISISAVE(1)	/* restore r28-r31 */
	mfsprg	1,1			/* restore SP */
	rfi				/* return to trapped code */
1:
	mflr	28			/* save LR */
	li	31,EXC_ISI
	mtlr	31
	ba	s_isitrap

/*
 * This one for the external interrupt handler.
 */
ENTRY_NOPROFILE(extint)
	mtsprg	1,1			/* save SP */
	GET_CPUINFO(1)
	stmw	28,CI_TEMPSAVE(1)	/* free r28-r31 */
	mflr	28			/* save LR */
	mfcr	29			/* save CR */
	mfxer	30			/* save XER */
	lwz	31,CI_INTRDEPTH(1)	/* were we already running on intstk? */
	addic.	31,31,1
	stw	31,CI_INTRDEPTH(1)
	beq+	1f
	mfsprg	1,1			/* yes, get old SP */
	ba	extintr
1:
	lwz	1,CI_INTSTK(1)		/* get interrupt stack */
	ba	extintr

/*
 * And this one for the decrementer interrupt handler.
 */
ENTRY_NOPROFILE(decrint)
	mtsprg	1,1			/* save SP */
	GET_CPUINFO(1)
	stmw	28,CI_TEMPSAVE(1)	/* free r28-r31 */
	mflr	28			/* save LR */
	mfcr	29			/* save CR */
	mfxer	30			/* save XER */
	lwz	31,CI_INTRDEPTH(1)	/* were we already running on intstk? */
	addic.	31,31,1
	stw	31,CI_INTRDEPTH(1)
	beq+	1f
	mfsprg	1,1			/* yes, get old SP */
	ba	decrintr
1:
	lwz	1,CI_INTSTK(1)		/* get interrupt stack */
	ba	decrintr

/*
 * Performance Monitor Exception (0x0f00) and
 * AltiVec Unavailable Exception (0x0f20) handler.
 */
ENTRY_NOPROFILE(perftrap)
	mtsprg	1,1			/* save SP */
	GET_CPUINFO(1)
	stmw	28,CI_TEMPSAVE(1)	/* free r28-r31 */
	mflr	28			/* save LR */
	mfcr	29			/* save CR */
	li	31,EXC_PERF
	b	2f
	nop

	mtsprg	1,1			/* save SP */
	GET_CPUINFO(1)
	stmw	28,CI_TEMPSAVE(1)	/* free r28-r31 */
	mflr	28			/* save LR */
	mfcr	29			/* save CR */
	li	31,EXC_VEC
2:
	mtlr	31
/* Test whether we already had PR set */
	mfsrr1	31
	andi.	31,31,PSL_PR
	beq	1f			/* branch if supervisor */
	lwz	1,CI_CURPCB(1)
	addi	1,1,USPACE		/* stack is top of user struct */
	ba	s_trap
1:
	mfsprg	1,1			/* restore SP */
	ba	s_trap

/*
 * Now the tlb software load for 603 processors:
 * (Code essentially from the 603e User Manual, Chapter 5, but
 * corrected a lot.)
 */
ENTRY_NOPROFILE(tlbimiss)
	mfspr	2,HASH1			/* get first pointer */
	li	1,8
	mfctr	0			/* save counter */
	mfspr	3,ICMP			/* get first compare value */
	addi	2,2,-8			/* predec pointer */
1:
	mtctr	1			/* load counter */
2:
	lwzu	1,8(2)			/* get next pte */
	cmplw	1,3			/* see if found pte */
	bdneq	2b			/* loop if not eq */
	bne	3f			/* not found */
	lwz	1,4(2)			/* load tlb entry lower word */
	andi.	3,1,8			/* check G-bit */
	bne	4f			/* if guarded, take ISI */
	mtctr	0			/* restore counter */
	mfspr	0,IMISS			/* get the miss address for the tlbli */
	mfsrr1	3			/* get the saved cr0 bits */
	mtcrf	0x80,3			/* and restore */
	ori	1,1,0x100		/* set the reference bit */
	mtspr	RPA,1			/* set the pte */
	srwi	1,1,8			/* get byte 7 of pte */
	tlbli	0			/* load the itlb */
	stb	1,6(2)			/* update page table */
	rfi

3:	/* not found in pteg */
	andi.	1,3,0x40		/* have we already done second hash? */
	bne	5f
	mfspr	2,HASH2			/* get the second pointer */
	ori	3,3,0x40		/* change the compare value */
	li	1,8
	addi	2,2,-8			/* predec pointer */
	b	1b
4:	/* guarded */
	mfsrr1	3
	andi.	2,3,0xffff		/* clean upper srr1 */
	oris	2,2,0x8000000@h		/* set srr<4> to flag prot violation */
	b	6f
5:	/* not found anywhere */
	mfsrr1	3
	andi.	2,3,0xffff		/* clean upper srr1 */
	oris	2,2,0x40000000@h	/* set srr1<1> to flag pte not found */
6:
	mtctr	0			/* restore counter */
	mtsrr1	2
	mfmsr	0
	xoris	0,0,0x20000@h		/* flip the msr<tgpr> bit */
	mtcrf	0x80,3			/* restore cr0 */
	mtmsr	0			/* now with native gprs */
	isync
	ba	EXC_ISI

ENTRY_NOPROFILE(tlbdlmiss)
	mfspr	2,HASH1			/* get first pointer */
	li	1,8
	mfctr	0			/* save counter */
	mfspr	3,DCMP			/* get first compare value */
	addi	2,2,-8			/* predec pointer */
1:
	mtctr	1			/* load counter */
2:
	lwzu	1,8(2)			/* get next pte */
	cmplw	1,3			/* see if found pte */
	bdneq	2b			/* loop if not eq */
	bne	3f			/* not found */
	lwz	1,4(2)			/* load tlb entry lower word */
	mtctr	0			/* restore counter */
	mfspr	0,DMISS			/* get the miss address for the tlbld */
	mfsrr1	3			/* get the saved cr0 bits */
	mtcrf	0x80,3			/* and restore */
	ori	1,1,0x100		/* set the reference bit */
	mtspr	RPA,1			/* set the pte */
	srwi	1,1,8			/* get byte 7 of pte */
	tlbld	0			/* load the dtlb */
	stb	1,6(2)			/* update page table */
	rfi

3:	/* not found in pteg */
	andi.	1,3,0x40		/* have we already done second hash? */
	bne	5f
	mfspr	2,HASH2			/* get the second pointer */
	ori	3,3,0x40		/* change the compare value */
	li	1,8
	addi	2,2,-8			/* predec pointer */
	b	1b
5:	/* not found anywhere */
	mfsrr1	3
	lis	1,0x40000000@h		/* set dsisr<1> to flag pte not found */
	mtctr	0			/* restore counter */
	andi.	2,3,0xffff		/* clean upper srr1 */
	mtsrr1	2
	mtdsisr	1			/* load the dsisr */
	mfspr	1,DMISS			/* get the miss address */
	mtdar	1			/* put in dar */
	mfmsr	0
	xoris	0,0,0x20000@h		/* flip the msr<tgpr> bit */
	mtcrf	0x80,3			/* restore cr0 */
	mtmsr	0			/* now with native gprs */
	isync
	ba	EXC_DSI

ENTRY_NOPROFILE(tlbdsmiss)
	mfspr	2,HASH1			/* get first pointer */
	li	1,8
	mfctr	0			/* save counter */
	mfspr	3,DCMP			/* get first compare value */
	addi	2,2,-8			/* predec pointer */
1:
	mtctr	1			/* load counter */
2:
	lwzu	1,8(2)			/* get next pte */
	cmplw	1,3			/* see if found pte */
	bdneq	2b			/* loop if not eq */
	bne	3f			/* not found */
	lwz	1,4(2)			/* load tlb entry lower word */
	andi.	3,1,0x80		/* check the C-bit */
	beq	4f
5:
	mtctr	0			/* restore counter */
	mfspr	0,DMISS			/* get the miss address for the tlbld */
	mfsrr1	3			/* get the saved cr0 bits */
	mtcrf	0x80,3			/* and restore */
	mtspr	RPA,1			/* set the pte */
	tlbld	0			/* load the dtlb */
	rfi

3:	/* not found in pteg */
	andi.	1,3,0x40		/* have we already done second hash? */
	bne	5f
	mfspr	2,HASH2			/* get the second pointer */
	ori	3,3,0x40		/* change the compare value */
	li	1,8
	addi	2,2,-8			/* predec pointer */
	b	1b
4:	/* found, but C-bit = 0 */
	rlwinm.	3,1,30,0,1		/* test PP */
	bge-	7f
	andi.	3,1,1
	beq+	8f
9:	/* found, but protection violation (PP==00)*/
	mfsrr1	3
	lis	1,0xa000000@h		/* indicate protection violation
					   on store */
	b	1f
7:	/* found, PP=1x */
	mfspr	3,DMISS			/* get the miss address */
	mfsrin	1,3			/* get the segment register */
	mfsrr1	3
	rlwinm	3,3,18,31,31		/* get PR-bit */
	rlwnm.	1,1,3,1,1		/* get the key */
	bne-	9b			/* protection violation */
8:	/* found, set reference/change bits */
	lwz	1,4(2)			/* reload tlb entry */
	ori	1,1,0x180
	sth	1,6(2)
	b	5b
5:	/* not found anywhere */
	mfsrr1	3
	lis	1,0x42000000@h		/* set dsisr<1> to flag pte not found */
					/* dsisr<6> to flag store */
1:
	mtctr	0			/* restore counter */
	andi.	2,3,0xffff		/* clean upper srr1 */
	mtsrr1	2
	mtdsisr	1			/* load the dsisr */
	mfspr	1,DMISS			/* get the miss address */
	mtdar	1			/* put in dar */
	mfmsr	0
	xoris	0,0,0x20000@h		/* flip the msr<tgpr> bit */
	mtcrf	0x80,3			/* restore cr0 */
	mtmsr	0			/* now with native gprs */
	isync
	ba	EXC_DSI

#ifdef DDB
/*
 * In case of DDB we want a separate trap catcher for it
 */
ENTRY_NOPROFILE(ddblow)
	mtsprg	1,1			/* save SP */
	GET_CPUINFO(1)
	stmw	28,CI_DDBSAVE(1)	/* free r28-r31 */
	mflr	28			/* save LR */
	mfcr	29			/* save CR */
	lwz	1,CI_DDBSTK(1)		/* get new SP */
	li	31,0
	stwu	31,-16(1)		/* end of stack chain */
	bl	1f
1:
	mflr	31
	rlwinm	31,31,0,0,23		/* r31 &= ~0xff */
	mtlr	31
	ba	ddbtrap
#endif	/* DDB */

#ifdef IPKDB
#define CI_IPKDBSAVE CI_DDBSAVE
#define CI_IPKDBSTK  CI_DDBSTK

/*
 * In case of IPKDB we want a separate trap catcher for it
 */
ENTRY_NOPROFILE(ipkdblow)
	mtsprg	1,1			/* save SP */
	GET_CPUINFO(1)
	stmw	28,CI_IPKDBSAVE(1)	/* free r28-r31 */
	mflr	28			/* save LR */
	mfcr	29			/* save CR */
	lwz	1,CI_IPKDBSTK(1)	/* get new SP */
	li	31,0
	stwu	31,-16(1)		/* end of stack chain */
	bl	1f
1:
	mflr	31
	rlwinm	31,31,0,0,23		/* r31 &= ~0xff */
	mtlr	31
	ba	ipkdbtrap
#endif	/* IPKDB */

/*
 * FRAME_SETUP assumes:
 *	SPRG1		SP (1)
 *	savearea	r28-r31,DAR,DSISR	(DAR & DSISR only for DSI traps)
 *	28		LR
 *	29		CR
 *	1		kernel stack
 *	LR		trap type
 *	SRR0/1		as at start of trap
 */
#define FRAME_SETUP(savearea)						\
/* Have to enable translation to allow access of kernel stack: */	\
	GET_CPUINFO(30);						\
	mfsrr0	31;							\
	stw	31,savearea+24(30);					\
	mfsrr1	31;							\
	stw	31,savearea+28(30);					\
	mfsprg	30,1;							\
	mfmsr	31;							\
	ori	31,31,(PSL_DR|PSL_IR);					\
	mtmsr	31;							\
	isync;								\
	stwu	30,-FRAMELEN(1);					\
	stw	0,FRAME_0+8(1);						\
	stw	30,FRAME_1+8(1);					\
	stw	2,FRAME_2+8(1);						\
	stw	28,FRAME_LR+8(1);					\
	stw	29,FRAME_CR+8(1);					\
	GET_CPUINFO(2);							\
	lmw	28,savearea(2);						\
	stmw	3,FRAME_3+8(1);						\
	lmw	28,savearea+16(2);					\
	mfxer	3;							\
	mfctr	4;							\
	mflr	5;							\
	stw	3,FRAME_XER+8(1);					\
	stw	4,FRAME_CTR+8(1);					\
	stw	5,FRAME_EXC+8(1);					\
	stw	28,FRAME_DAR+8(1);					\
	stw	29,FRAME_DSISR+8(1);					\
	stw	30,FRAME_SRR0+8(1);					\
	stw	31,FRAME_SRR1+8(1)

#define FRAME_LEAVE(savearea)						\
/* Now restore regs: */							\
	lwz	3,FRAME_SRR0+8(1);					\
	lwz	4,FRAME_SRR1+8(1);					\
	lwz	5,FRAME_CTR+8(1);					\
	lwz	6,FRAME_XER+8(1);					\
	lwz	7,FRAME_LR+8(1);					\
	GET_CPUINFO(2);							\
	stw	3,savearea+24(2);					\
	stw	4,savearea+28(2);					\
	mtctr	5;							\
	mtxer	6;							\
	mtlr	7;							\
	lmw	3,FRAME_3+8(1);						\
	stmw	28,savearea(2);		/* free r28-r31 */		\
	lwz	28,FRAME_2+8(1);	/* save r2 in r28 */		\
	lwz	29,FRAME_CR+8(1);	/* save cr in r29 */		\
	lwz	0,FRAME_0+8(1);						\
	lwz	1,FRAME_1+8(1);						\
/* Disable translation, machine check and recoverability: */		\
	mfmsr	31;							\
	andi.	31,31,~(PSL_DR|PSL_IR|PSL_ME|PSL_RI)@l;			\
	mtmsr	31;							\
	isync;								\
	mtsprg	2,28;			/* save r2 in sprg2 */		\
	lwz	30,savearea+24(2);					\
	lwz	31,savearea+28(2);					\
	mtsrr0	30;							\
	mtsrr1	31;							\
/* Decide whether we return to user mode: */				\
	andi.	31,31,PSL_PR;						\
	beq	1f;			/* branch if supervisor */	\
/* Restore user & kernel access SR: */					\
	lwz	28,CI_CURPM(2);		/* get real address of pmap */	\
	lwz	30,PM_USRSR(28);					\
	lwz	31,PM_KERNELSR(28);					\
	mtsr	USER_SR,30;						\
	mtsr	KERNEL_SR,31;						\
1:	mtcr	29;							\
	lmw	28,savearea(2);		/* restore r28-r31 */		\
	mfsprg	2,2			/* restore r2 */

/*
 * Preamble code for DSI/ISI traps
 */
disitrap:
	lmw	30,CI_DISISAVE(1)
	stmw	30,CI_TEMPSAVE(1)
	lmw	30,CI_DISISAVE+8(1)
	stmw	30,CI_TEMPSAVE+8(1)
	mfdar	30
	mfdsisr	31
	stmw	30,CI_TEMPSAVE+16(1)
realtrap:
/* Test whether we already had PR set */
	mfsrr1	1
	andi.	1,1,PSL_PR
	mfsprg	1,1			/* restore SP (might have been
					   overwritten) */
	beq	s_trap			/* branch if supervisor */
	GET_CPUINFO(1)
	lwz	1,CI_CURPCB(1)
	addi	1,1,USPACE		/* stack is top of user struct */

/*
 * Now the common trap catching code.
 */
s_trap:
/* First have to enable KERNEL mapping */
	lis	31,KERNEL_SEGMENT@h
	ori	31,31,KERNEL_SEGMENT@l
	mtsr	KERNEL_SR,31
	FRAME_SETUP(CI_TEMPSAVE)
/* Now we can recover interrupts again: */
trapagain:
	mfmsr	7
	ori	7,7,(PSL_EE|PSL_ME|PSL_RI)@l
	mtmsr	7
	isync
/* Call C trap code: */
	addi	3,1,8
	bl	_C_LABEL(trap)
trapexit:
/* Disable interrupts: */
	mfmsr	3
	andi.	3,3,~PSL_EE@l
	mtmsr	3
/* Test AST pending: */
	lwz	5,FRAME_SRR1+8(1)
	andi.	5,5,PSL_PR
	beq	1f			/* branch if supervisor */
	GET_CPUINFO(3)
	lwz	4,CI_ASTPENDING(3)
	andi.	4,4,1
	beq	1f
	li	6,EXC_AST
	stw	6,FRAME_EXC+8(1)
	b	trapagain
1:
	FRAME_LEAVE(CI_TEMPSAVE)
	rfi

/*
 * Child comes here at the end of a fork.
 * Return to userspace via the trap return path.
 * XXX r3 and r4 are passed from cpu_switchto() unchanged.
 */
ENTRY_NOPROFILE(fork_trampoline)
	bl	_C_LABEL(lwp_startup)
	mtlr	31
	mr	3,30
	blrl				/* jump indirect to r31 */
	b	trapexit

/*
 * DSI second stage fault handler
 */
s_dsitrap:
	mfdsisr	31			/* test whether this may be a
					   spill fault */
	andis.	31,31,0x40000000@h
	beq	disitrap		/* branch if table miss is false */
	lwz	1,CI_SPILLSTK(1)	/* get spill stack */
	stwu	1,-52(1)
	stw	0,48(1)			/* save volatile registers */
	stw	3,44(1)
	stw	4,40(1)
	stw	5,36(1)
	stw	6,32(1)
	stw	7,28(1)
	stw	8,24(1)
	stw	9,20(1)
	stw	10,16(1)
	stw	11,12(1)
	stw	12,8(1)
	mfxer	30			/* save XER */
	mfctr	31			/* & CTR */
	mfdar	3
	bl	_C_LABEL(pte_spill)	/* try a spill */
	li	4,EXC_DSI
s_pte_spill:
	cmpwi	3,0
	mtlr	4			/* trap type */
	mtxer	30			/* restore XER */
	mtctr	31			/* restore CTR */
	lwz	12,8(1)			/* restore volatile registers */
	lwz	11,12(1)
	lwz	10,16(1)
	lwz	9,20(1)
	lwz	8,24(1)
	lwz	7,28(1)
	lwz	6,32(1)
	lwz	5,36(1)
	lwz	4,40(1)
	lwz	3,44(1)
	lwz	0,48(1)
	GET_CPUINFO(1)
	beq	disitrap
	mtcr	29			/* restore CR */
	mtlr	28			/* restore LR */
	lmw	28,CI_DISISAVE(1)	/* restore r28-r31 */
	mfsprg	1,1			/* restore SP */
	rfi				/* return to trapped code */

/*
 * ISI second stage fault handler
 */
s_isitrap:
	mfsrr1	31			/* test whether this may be a
					   spill fault */
	andis.	31,31,0x40000000@h
	beq	disitrap		/* branch if table miss is false */
	lwz	1,CI_SPILLSTK(1)	/* get spill stack */
	stwu	1,-52(1)
	stw	0,48(1)			/* save volatile registers */
	stw	3,44(1)
	stw	4,40(1)
	stw	5,36(1)
	stw	6,32(1)
	stw	7,28(1)
	stw	8,24(1)
	stw	9,20(1)
	stw	10,16(1)
	stw	11,12(1)
	stw	12,8(1)
	mfxer	30			/* save XER */
	mfctr	31			/* & ctr */
	mfsrr0	3
	bl	_C_LABEL(pte_spill)	/* try a spill */
	li	4,EXC_ISI
	b	s_pte_spill		/* above */

/*
 * External interrupt second level handler
 */
#define INTRENTER							\
/* Save volatile registers: */						\
	stwu	1,-96(1);		/* temporarily */		\
	stw	0,84(1);						\
	mfsprg	0,1;			/* get original SP */		\
	stw	0,0(1);			/* and store it */		\
	stw	3,80(1);						\
	stw	4,76(1);						\
	stw	5,72(1);						\
	stw	6,68(1);						\
	stw	7,64(1);						\
	stw	8,60(1);						\
	stw	9,56(1);						\
	stw	10,52(1);						\
	stw	11,48(1);						\
	stw	12,44(1);						\
	stw	28,40(1);		/* saved LR */			\
	stw	29,36(1);		/* saved CR */			\
	stw	30,32(1);		/* saved XER */			\
	GET_CPUINFO(5);							\
	lmw	28,CI_TEMPSAVE(5);	/* restore r28-r31 */		\
	mfctr	6;							\
	lwz	5,CI_INTRDEPTH(5);					\
	mfsrr0	4;							\
	mfsrr1	3;							\
	stw	6,28(1);						\
	stw	5,20(1);						\
	stw	4,12(1);						\
	stw	3,8(1);							\
/* interrupts are recoverable here, and enable translation */		\
	lis	3,(KERNEL_SEGMENT|SR_SUKEY|SR_PRKEY)@h;			\
	ori	3,3,(KERNEL_SEGMENT|SR_SUKEY|SR_PRKEY)@l;		\
	mtsr	KERNEL_SR,3;						\
	mfmsr	5;							\
	ori	5,5,(PSL_IR|PSL_DR|PSL_RI);				\
	mtmsr	5;							\
	isync

	.globl	_C_LABEL(extint_call)
extintr:
	INTRENTER
_C_LABEL(extint_call):
	bl	_C_LABEL(extint_call)	/* to be filled in later */

intr_exit:
/* Disable interrupts (should already be disabled) and MMU here: */
	mfmsr	3
	andi.	3,3,~(PSL_EE|PSL_ME|PSL_RI|PSL_DR|PSL_IR)@l
	mtmsr	3
	isync
/* restore possibly overwritten registers: */
	lwz	12,44(1)
	lwz	11,48(1)
	lwz	10,52(1)
	lwz	9,56(1)
	lwz	8,60(1)
	lwz	7,64(1)
	lwz	6,8(1)
	lwz	5,12(1)
	lwz	4,28(1)
	lwz	3,32(1)
	mtsrr1	6
	mtsrr0	5
	mtctr	4
	mtxer	3
/* Returning to user mode? */
	andi.	6,6,PSL_PR		/* saved SRR1 */
	beq	1f			/* branch if supervisor */
	GET_CPUINFO(4)
	lwz	3,CI_CURPM(4)		/* get current pmap real address */
	lwz	3,PM_KERNELSR(3)
	mtsr	KERNEL_SR,3		/* Restore kernel SR */
	lwz	6,CI_ASTPENDING(4)	/* Test AST pending */
	andi.	6,6,1
	beq	1f
/* Setup for entry to realtrap: */
	lwz	3,0(1)			/* get saved SP */
	mtsprg	1,3
	li	6,EXC_AST
	stmw	28,CI_TEMPSAVE(4)	/* establish tempsave again */
	mtlr	6
	lwz	28,40(1)		/* saved LR */
	lwz	29,36(1)		/* saved CR */
	lwz	6,68(1)
	lwz	5,72(1)
	lwz	4,76(1)
	lwz	3,80(1)
	lwz	0,84(1)
	GET_CPUINFO(30)
	lwz	31,CI_INTRDEPTH(30)	/* adjust reentrancy count */
	addi	31,31,-1
	stw	31,CI_INTRDEPTH(30)
	b	realtrap
1:
/* Here is the normal exit of extintr: */
	lwz	5,36(1)
	lwz	6,40(1)
	mtcr	5
	mtlr	6
	lwz	6,68(1)
	lwz	5,72(1)
	GET_CPUINFO(3)
	lwz	4,CI_INTRDEPTH(3)	/* adjust reentrancy count */
	addi	4,4,-1
	stw	4,CI_INTRDEPTH(3)
	lwz	4,76(1)
	lwz	3,80(1)
	lwz	0,84(1)
	lwz	1,0(1)
	rfi

/*
 * Decrementer interrupt second level handler
 */
decrintr:
	INTRENTER
	addi	3,1,8			/* intr frame */
	bl	_C_LABEL(decr_intr)
	b	intr_exit

#ifdef DDB
/*
 * Deliberate entry to ddbtrap
 */
ENTRY_NOPROFILE(ddb_trap)
	mtsprg	1,1
	mfmsr	3
	mtsrr1	3
	andi.	3,3,~(PSL_EE|PSL_ME)@l
	mtmsr	3			/* disable interrupts */
	isync
	GET_CPUINFO(3)
	stmw	28,CI_DDBSAVE(3)
	mflr	28
	li	29,EXC_BPT
	mtlr	29
	mfcr	29
	mtsrr0	28

/*
 * Now the ddb trap catching code.
 */
ddbtrap:
	FRAME_SETUP(CI_DDBSAVE)
/* Call C trap code: */
	addi	3,1,8
	bl	_C_LABEL(ddb_trap_glue)
	cmpwi	3,0
	bne	ddbleave
/* This wasn't for DDB, so switch to real trap: */
	GET_CPUINFO(4)
	lwz	3,FRAME_EXC+8(1)	/* save exception */
	stw	3,CI_DDBSAVE+16(4)
	FRAME_LEAVE(CI_DDBSAVE)
	mtsprg	1,1			/* prepare for entrance to realtrap */
	GET_CPUINFO(1)
	stmw	28,CI_TEMPSAVE(1)
	mflr	28
	mfcr	29
	lwz	31,CI_DDBSAVE+16(1)
	mtlr	31
	b	realtrap
ddbleave:
	FRAME_LEAVE(CI_DDBSAVE)
	rfi
#endif /* DDB */

#ifdef IPKDB
/*
 * Deliberate entry to ipkdbtrap
 */
ENTRY_NOPROFILE(ipkdb_trap)
	mtsprg	1,1
	mfmsr	3
	mtsrr1	3
	andi.	3,3,~(PSL_EE|PSL_ME)@l
	mtmsr	3			/* disable interrupts */
	isync
	GET_CPUINFO(3)
	stmw	28,CI_IPKDBSAVE(3)
	mflr	28
	li	29,EXC_BPT
	mtlr	29
	mfcr	29
	mtsrr0	28

/*
 * Now the ipkdb trap catching code.
 */
ipkdbtrap:
	FRAME_SETUP(CI_IPKDBSAVE)
/* Call C trap code: */
	addi	3,1,8
	bl	_C_LABEL(ipkdb_trap_glue)
	cmpwi	3,0
	bne	ipkdbleave
/* This wasn't for IPKDB, so switch to real trap: */
	GET_CPUINFO(4)
	lwz	3,FRAME_EXC+8(1)	/* save exception */
	stw	3,CI_IPKDBSAVE+16(4)
	FRAME_LEAVE(CI_IPKDBSAVE)
	mtsprg	1,1			/* prepare for entrance to realtrap */
	GET_CPUINFO(1)
	stmw	28,CI_TEMPSAVE(1)
	mflr	28
	mfcr	29
	lwz	31,CI_IPKDBSAVE+16(1)
	mtlr	31
	b	realtrap
ipkdbleave:
	FRAME_LEAVE(CI_IPKDBSAVE)
	rfi

ipkdbfault:
	ba	_ipkdbfault
_ipkdbfault:
	mfsrr0	3
	addi	3,3,4
	mtsrr0	3
	li	3,-1
	rfi

/*
 * int ipkdbfbyte(unsigned char *p)
 */
ENTRY_NOPROFILE(ipkdbfbyte)
	li	9,EXC_DSI		/* establish new fault routine */
	lwz	5,0(9)
	lis	6,ipkdbfault@ha
	lwz	6,ipkdbfault@l(6)
	stw	6,0(9)
#ifdef IPKDBUSERHACK
	lis	8,_C_LABEL(ipkdbsr)@ha
	lwz	8,_C_LABEL(ipkdbsr)@l(8)
	mtsr	USER_SR,8
	isync
#endif
	dcbst	0,9			/* flush data... */
	sync
	icbi	0,9			/* and instruction caches */
	lbz	3,0(3)			/* fetch data */
	stw	5,0(9)			/* restore previous fault handler */
	dcbst	0,9			/* and flush data... */
	sync
	icbi	0,9			/* and instruction caches */
	blr

/*
 * int ipkdbsbyte(unsigned char *p, int c)
 */
ENTRY_NOPROFILE(ipkdbsbyte)
	li	9,EXC_DSI		/* establish new fault routine */
	lwz	5,0(9)
	lis	6,ipkdbfault@ha
	lwz	6,ipkdbfault@l(6)
	stw	6,0(9)
#ifdef IPKDBUSERHACK
	lis	8,_C_LABEL(ipkdbsr)@ha
	lwz	8,_C_LABEL(ipkdbsr)@l(8)
	mtsr	USER_SR,8
	isync
#endif
	dcbst	0,9			/* flush data... */
	sync
	icbi	0,9			/* and instruction caches */
	mr	6,3
	li	3,0
	stb	4,0(6)
	dcbst	0,6			/* Now do appropriate flushes
					   to data... */
	sync
	icbi	0,6			/* and instruction caches */
	stw	5,0(9)			/* restore previous fault handler */
	dcbst	0,9			/* and flush data... */
	sync
	icbi	0,9			/* and instruction caches */
	blr
#endif	/* IPKDB */

ENTRY_NOPROFILE(syscalltrap)
	mtsprg	1,1			/* save SP */
	GET_CPUINFO(1)
	stmw	28,CI_TEMPSAVE(1)	/* free r28-r31 */
	mflr	28			/* save LR */
	mfcr	29			/* save CR */
	lwz	1,CI_CURPCB(1)
	addi	1,1,USPACE		/* stack is top of user struct */
	li	31,EXC_SC
	mtlr	31
	ba	1f
1:
/* First have to enable KERNEL mapping */
	lis	31,KERNEL_SEGMENT@h
	ori	31,31,KERNEL_SEGMENT@l
	mtsr	KERNEL_SR,31
	FRAME_SETUP(CI_TEMPSAVE)
/* Now we can recover interrupts again: */
	mfmsr	7
	ori	7,7,(PSL_EE|PSL_ME|PSL_RI)@l
	mtmsr	7
	isync
	GET_CPUINFO(3)
	lwz	3,CI_CURLWP(3)
	lwz	3,L_PROC(3)
	lwz	4,P_MD_SYSCALL(3)
	mtlr	4
	addi	3,1,8
	blrl
	b	trapexit
