/*	$Id: locore_subr.S,v 1.65 2009/12/22 10:44:50 tsubai Exp $	*/

/*
 * Copyright (C) 1995, 1996 Wolfgang Solfrank.
 * Copyright (C) 1995, 1996 TooLs GmbH.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by TooLs GmbH.
 * 4. The name of TooLs GmbH may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TOOLS GMBH ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL TOOLS GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "opt_lockdebug.h"
#include "assym.h"

#include <sys/syscall.h>

#include <machine/param.h>
#include <machine/asm.h>
#include <machine/psl.h>

GLOBAL(OF_buf)
	.long	_OF_buf
	.lcomm	_OF_buf,NBPG,4

	.text
/*
 * Enter power-saving mode if available.
 */
ENTRY(cpu_idle)
	GET_CPUINFO(3)
	lwz	4,CI_WANT_RESCHED(3)
	cmpwi	4,0
	bnelr

	.globl	_C_LABEL(idle_hook)
_C_LABEL(idle_hook):
	blr				/* overwritten later */

	dssall
					/* XXX cache flush? */
					/* XXX i-cache disable? */
	isync
					/* XXX TLB invalidate? */

	sync
	mfmsr	3
	oris	3,3,PSL_POW@h		/* enter power saving mode */
	mtmsr	3
	isync
	blr

/*
 * struct lwp *
 * cpu_switchto(struct lwp *current, struct lwp *new)
 * Switch to the indicated new LWP.
 * 	r3 - current LWP
 * 	r4 - LWP to switch to
 */
ENTRY(cpu_switchto)
	mflr	0			/* save lr */
	stw	0,4(1)
	stwu	1,-16(1)
	stw	31,12(1)
	stw	30,8(1)

	mr	30,3			/* r30 = curlwp */
	mr	31,4			/* r31 = newlwp */

	cmpwi	30,0			/* old lwp was exiting? */
	beq	switch_exited

	mfcr	11			/* save cr */
	mr	12,2			/* save r2 */
	stwu	1,-SFRAMELEN(1)		/* still running on old stack */
	stmw	11,12(1)
	lwz	5,L_ADDR(30)
	stw	1,PCB_SP(5)		/* save SP */

switch_exited:
	GET_CPUINFO(5)
	stw	31,CI_CURLWP(5)		/* record new process */
	lwz	6,L_ADDR(31)
	stwu	6,CI_CURPCB(5)		/* indicate new pcb */
	stwcx.	6,0,5			/* clear possible reservation */

	lwz	1,PCB_SP(6)		/* get new procs SP */
	lmw	11,12(1)		/* get other regs */
	lwz	1,0(1)			/* get saved SP */
	mr	2,12			/* get saved r2 */
	mtcr	11			/* get saved cr */

	lwz	31,12(1)
	lwz	30,8(1)
	addi	1,1,16
	lwz	0,4(1)
	mtlr	0
	blr

#ifdef __HAVE_FAST_SOFTINTS

	.globl softintr_ret
softintr_ret:
	GET_CPUINFO(4)
	lwz	5,CI_MTX_COUNT(4)
	addi	5,5,1
	stw	5,CI_MTX_COUNT(4)

	li	0,0
	stw	0,L_CTXSWTCH(3)

	addi	1,1,16
	lwz	0,4(1)
	mtlr	0
	blr

/*
 * r3 = softint lwp
 * r4 = spl level
 */
ENTRY(Xsoftintr)
	mflr	0
	stw	0,4(1)
	stwu	1,-16(1)

	lis	5,softintr_ret@ha
	addi	5,5,softintr_ret@l
	stw	5,4(1)
	stwu	1,-16(1)
	stw	31,12(1)
	stw	30,8(1)

	mfmsr	5
	andi.	5,5,~PSL_EE@l
	mtmsr	5
	isync

	GET_CPUINFO(9)
	lwz	30,CI_CURLWP(9)		/* r30 = curlwp (interrupted lwp) */

	mfcr	11			/* save cr */
	mr	12,2			/* save r2 */
	stwu	1,-SFRAMELEN(1)		/* still running on old stack */
	stmw	11,12(1)
	lwz	5,L_ADDR(30)
	stw	1,PCB_SP(5)		/* save SP */

	GET_CPUINFO(5)
	stw	3,CI_CURLWP(5)		/* record new process */
	lwz	6,L_ADDR(3)
	stw	6,CI_CURPCB(5)		/* indicate new pcb */

  mr 5,1
	addi	1,6,USPACE-16		/* new SP */
  stw 5,0(1)				/* maintain stack chain */

	mfmsr	5
	ori	5,5,PSL_EE@l
	mtmsr	5

	mr	3,30
	bl	_C_LABEL(softint_dispatch)

	mfmsr	5
	andi.	5,5,~PSL_EE@l
	mtmsr	5
	isync

	lwz	6,L_ADDR(30)
	lwz	1,PCB_SP(6)

	lwz	1,0(1)			/* get saved SP */

	GET_CPUINFO(5)
	stw	30,CI_CURLWP(5)		/* restore original curlwp */

	lwz	6,L_ADDR(30)
	stw	6,CI_CURPCB(5)

	mfmsr	5
	ori	5,5,PSL_EE@l
	mtmsr	5

	lwz	31,12(1)
	lwz	30,8(1)
	addi	1,1,32			/* unwind two 16-byte frames */
	lwz	0,4(1)
	mtlr	0
	blr

#endif /* __HAVE_FAST_SOFTINTS */

/*
 * int setfault()
 *
 * Similar to setjmp to setup for handling faults on accesses to user memory.
 * Any routine using this may only call bcopy, either the form below,
 * or the (currently used) C code optimized, so it doesn't use any non-volatile
 * registers.
 */
ENTRY(setfault)
	mflr	0
	mfcr	12
	GET_CPUINFO(4)
	lwz	4,CI_CURPCB(4)
	stw	3,PCB_FAULT(4)
	stw	0,0(3)
	stw	1,4(3)
	stw	2,8(3)
	stmw	12,12(3)
	sync
	isync
	li	3,0
	blr

/*
 * int _lock_cas(uintptr_t *ptr, uintptr_t old, uintptr_t new);
 * XXX Assume r3 != r1.
 */
ENTRY_NOPROFILE(_lock_cas)
1:	lwz	6,0(3)
	cmpw	6,4
	bne	3f

	lwarx	6,0,3
	cmpw	6,4
	bne	2f

	stwcx.	5,0,3
	bne-	1b

	li	3,1
	blr

2:	addi	3,1,-4
	stwcx.	3,0,3			/* Clear reservation. */
3:	li	3,0
	blr

#ifndef LOCKDEBUG
/*
 * void mutex_enter(kmutex_t *);
 */
ENTRY_NOPROFILE(mutex_enter)
	GET_CPUINFO(4)
	lwz	5,CI_CURLWP(4)
1:
	lwarx	0,0,3
	cmpwi	0,0
	bne	2f

	stwcx.	5,0,3
	bne-	1b
	isync
	blr
2:
	addi	4,1,-4
	stwcx.	4,0,4			/* Clear reservation. */
	b	_C_LABEL(mutex_vector_enter)

/*
 * void mutex_exit(kmutex_t *);
 */
ENTRY_NOPROFILE(mutex_exit)
	GET_CPUINFO(4)
	lwz	5,CI_CURLWP(4)

	lwz	0,0(3)
	cmpw	0,5
	bne	2f

	sync
	li	0,0
	stw	0,0(3)
	blr
2:
	b	_C_LABEL(mutex_vector_exit)
#endif /* !LOCKDEBUG */

/*
 * The following code gets copied to the top of the user stack on process
 * execution.  It does signal trampolining on signal delivery.
 *
 * On entry r1 points to a struct sigcontext at bottom of current stack.
 * All other registers are unchanged.
 */
	.globl	_C_LABEL(sigcode),_C_LABEL(esigcode)
_C_LABEL(sigcode):
	addi	1,1,-16			/* reserved space for callee */
	mr	31,5			/* save scp */
	blrl
	mr	3,31			/* restore scp */
	li	0,SYS_compat_16___sigreturn14
	sc				/* sigreturn(scp) */
	li	0,SYS_exit
	sc				/* exit(errno) */
_C_LABEL(esigcode):
