/*	$Id: com_obio.c,v 1.13 2013/08/10 07:12:58 tsubai Exp $	*/

/*-
 * Copyright (c) 2008 Tsubai Masanari.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/param.h>
#include <sys/device.h>
#include <sys/tty.h>

#include <machine/autoconf.h>
#include <machine/bus.h>

#include <ts3000/board.h>

#include <dev/ic/comvar.h>

int com_obio_match(struct device *, struct cfdata *, void *);
void com_obio_attach(struct device *, struct device *, void *);

CFATTACH_DECL_NEW(com_obio, sizeof(struct com_softc),
    com_obio_match, com_obio_attach, NULL, NULL);

int
com_obio_match(parent, cf, aux)
	struct device *parent;
	struct cfdata *cf;
	void *aux;
{
	struct confargs *ca = aux;

	if (strcmp(ca->ca_name, "com") == 0)
		return 1;
	return 0;
}

void
com_obio_attach(parent, self, aux)
	struct device *parent, *self;
	void *aux;
{
	struct com_softc *sc = device_private(self);
	struct confargs *ca = aux;
	bus_space_tag_t iot = 0;
	bus_space_handle_t ioh = ca->ca_addr;

	switch (board_type) {
	case BOARD_TS400:
	case BOARD_TS800:
	case BOARD_ACS16:
	case BOARD_ACS32:
	case BOARD_ACS48:
	case BOARD_CS4008:
		sc->sc_frequency = COM_FREQ * 4;
		break;
	default:
		sc->sc_frequency = COM_FREQ * 8;
	}
	sc->sc_dev = self;
	COM_INIT_REGS(sc->sc_regs, iot, ioh, ca->ca_addr);
	com_attach_subr(sc);
}
