/*	$Id: flash.c,v 1.13 2015/06/23 19:46:58 tsubai Exp $	*/

/*-
 * Copyright (c) 2009 Tsubai Masanari.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/param.h>
#include <sys/conf.h>
#include <sys/device.h>

#include <machine/autoconf.h>
#include <machine/pio.h>
#include <machine/pte.h>

#include <ts3000/board.h>

struct flashtab {
	u_char vendor;
	u_char product;
	u_short size;
	char *name;
};
static struct flashtab rom[] = {
	{ 0x1f, 0xc0, 2048, "AT49BV160" },
	{ 0xbf, 0xd6,  256, "SST39VF020" },
	{ 0 }
};

struct flash_softc {
	struct device sc_dev;
	paddr_t sc_addr;
	u_int sc_size;
};

int flash_match(struct device *, struct cfdata *, void *);
void flash_attach(struct device *, struct device *, void *);
dev_type_mmap(flash_mmap);
extern struct cfdriver nvram_cd;

CFDECL(nvram, sizeof(struct flash_softc), flash_match, flash_attach);

struct cdevsw nvram_cdevsw = {
	nullopen, nullclose, noread, nowrite, noioctl,
	nostop, notty, nopoll, flash_mmap, nokqfilter
};

int
flash_match(parent, cf, aux)
	struct device *parent;
	struct cfdata *cf;
	void *aux;
{
	struct confargs *ca = aux;

	if (strcmp(ca->ca_name, "flash") == 0)
		return 1;

	return 0;
}

#define IN8(addr) in8(sc->sc_addr + bus_width * (addr))
#define OUT8(addr, val) out8(sc->sc_addr + bus_width * (addr), val)

void
flash_attach(parent, self, aux)
	struct device *parent, *self;
	void *aux;
{
	struct flash_softc *sc = (void *)self;
	struct confargs *ca = aux;
	struct flashtab *t = rom;
	u_int bus_width, vendor, product;

	sc->sc_addr = ca->ca_addr;

	switch (board_type) {
	case BOARD_TS3000:
	case BOARD_ACS16:
	case BOARD_ACS32:
	case BOARD_ACS48:
	case BOARD_CS4008:
		bus_width = 1;
		break;
	default:
		bus_width = 2;
	}

	OUT8(0, 0xf0);

	OUT8(0x5555, 0xaa);
	OUT8(0x2aaa, 0x55);
	OUT8(0x5555, 0x90);

	vendor  = IN8(0);
	product = IN8(1);

	OUT8(0, 0xf0);

	while (t->size) {
		if (vendor == t->vendor && product == t->product)
			goto found;
		t++;
	}
	printf(": unknown flash chip\n");
	return;

found:
	printf(": %s (%dKB)\n", t->name, t->size);
	sc->sc_size = t->size * 1024;
}

paddr_t
flash_mmap(dev, off, prot)
	dev_t dev;
	off_t off;
	int prot;
{
	struct flash_softc *sc = device_lookup_private(&nvram_cd, minor(dev));

	if (sc == NULL)
		return -1;

	if ((u_int64_t)off >= sc->sc_size)
		return -1;

	return (sc->sc_addr + off) | PG_CI;
}
