/*	$Id: boot.c,v 1.17 2010/01/07 13:48:05 tsubai Exp $	*/

/*-
 * Copyright (c) 2009 Tsubai Masanari.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <lib/libkern/libkern.h>
#include <lib/libsa/stand.h>
#include <lib/libsa/loadfile.h>

#include <machine/cpu.h>
#include <machine/cpufunc.h>
#include <machine/mpc860dev.h>
#include <machine/spr.h>
#include <machine/sysconf.h>

#include "boot.h"

#define CST_CE		0x02000000 /* Cache enable */
#define CST_UNLOCK_ALL	0x0a000000 /* Unlock all */
#define CST_INV_ALL	0x0c000000 /* Invalidate all */

void main(void);
void parseline(char *);

extern char _start[], _end[], _edata[];
extern char bootprog_name[], bootprog_rev[], bootprog_maker[], bootprog_date[];
char *kernels[] = { "/netbsd", "/netbsd.gz", "/netbsd.ts3000", NULL };
char *devices[] = { "disk:", "net:", NULL };
char bootargs[32];
char bootpath[256];

static char stack[8192] __aligned(16);

void
startup(start)
	char *start;
{
	bcopy(start, _start , _edata - _start);
	bzero(_edata, _end - _edata);

	asm volatile ("mr 1,%0" :: "r"(stack + sizeof stack - 16));

	printf("start  = %p\n", start);
	printf("_start = %p\n", _start);

	mtspr(IC_CST, CST_UNLOCK_ALL);
	mtspr(IC_CST, CST_INV_ALL);
	mtspr(IC_CST, CST_CE);

	syncicache(_start, _end - _start);

	/* Branch to relocated main. */
	asm volatile ("mtctr %0; bctr" :: "r"(main));

	for (;;);
}

void
main()
{
	volatile struct mpc860dev *immr = get_immr();
	u_long marks[MARK_MAX];
	void (*entry)();
	char *ssym, *esym;
	int i, j;
	char path[256];

	/* Disable interrupts and address translations. */
	mtmsr(0);

	/* Start timebase. */
	immr->siu_tbscr = 0x0001;

	smcinit();

	printf("\n");
	printf(">> %s, Revision %s\n", bootprog_name, bootprog_rev);
	printf(">> (%s, %s)\n", bootprog_maker, bootprog_date);

	if (count_down()) {
		printf("> ");
		gets(bootpath);
		parseline(bootpath);
	}

	getenaddr();
	fecinit();

	for (i = 0; devices[i]; i++) {
		for (j = 0; kernels[j]; j++) {
			marks[MARK_START] = 0;
			sprintf(path, "%s%s", devices[i], kernels[j]);
			if (loadfile(path, marks, LOAD_KERNEL) != -1)
				goto ok;
		}
	}
	for (;;);

ok:
	entry = (void *)marks[MARK_ENTRY];
	ssym = (void *)marks[MARK_SYM];
	esym = (void *)marks[MARK_END];
	sprintf(bootpath, "%s %s ssym=%p", path, bootargs, ssym);

	printf(" start=%p\n", entry);
	syncicache(entry, ssym - (char *)entry);

	sswt();
	(*entry)(bootpath, esym);

	for (;;);
}

void
parseline(cmd)
	char *cmd;
{
	char *cp;

	while (*cmd == ' ')
		cmd++;

	if (cmd[0] == '-') {
		strcpy(bootargs, cmd);
		cmd[0] = 0;
	}
	cp = strchr(cmd, ' ');
	if (cp) {
		*cp++ = 0;
		strcpy(bootargs, cp);
	}

	if (strncmp(cmd, "disk:", 5) == 0) {
		devices[0] = "disk:";
		devices[1] = NULL;
	}
	if (strncmp(cmd, "net:", 4) == 0) {
		devices[0] = "net:";
		devices[1] = NULL;
	}

	cp = strchr(cmd, ':');
	if (cp)
		cp++;
	else
		cp = cmd;
	if (*cp) {
		kernels[0] = cp;
		kernels[1] = NULL;
	}
}

/* Service software watchdog timer. */
void
sswt()
{
	volatile struct mpc860dev *immr = get_immr();

	immr->siu_swsr = 0x556c;
	eieio();
	immr->siu_swsr = 0xaa39;
	eieio();
}
