/*	$Id: config.c,v 1.7 2015/06/23 19:28:01 tsubai Exp $	*/

/*-
 * Copyright (c) 2009 Tsubai Masanari.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <lib/libkern/libkern.h>
#include <lib/libsa/stand.h>

#include <ts3000/board.h>
#include <machine/mpc860dev.h>

#include "boot.h"

u_int
calc_crc(p, len)
	u_char *p;
	u_int len;
{
	u_int crc = 0;
	int i;

	while (len--) {
		crc ^= *p++ << 24;
		for (i = 0; i < 8; i++) {
			if (crc & 0x80000000)
				crc ^= 0x88108000;
			crc <<= 1;
		}
	}
	return crc >> 16;
}

struct nvh {
	u_short type;
	u_short size;
	u_short crc;
	u_char data[2];	/* variable length */
};

void *
search_record(addr, type, tries)
	void *addr;
	int type, tries;
{
	struct nvh *p = addr;

	while (tries-- > 0) {
		if (p->type == 9999)
			break;
		if ((p->type > 200 && p->type != 9000) || p->size > 10000)
			break;
		if (p->crc != calc_crc(p->data, p->size))
			break;
		if (p->type == type)
			return p->data;

		p = (void *)(p->data + p->size);
	}
	return NULL;
}

struct nvconfig {
	u_char boot_option;
	u_char filename[40];
	u_char watchdog;
	u_char check_crc;
	u_char memtest;
	u_char res[4];
	u_short maxint;
	u_char enaddr[6];
	u_char myaddr[4];
	u_char mynetmask[4];
	u_char console_speed;
	u_char lcd_reboot;
	u_char cpu_utilization;
	u_char mii_operation;
};

struct machinfo {
	u_int	mem_test_result;
	u_int	memory_detected;
	u_char	alt_boot;
	u_char	error_status;
	u_char	reserved;
	u_char	board_type;
	/* ... */
};

int busfreq;
int cons_speed;

void
getenaddr()
{
	volatile struct mpc860dev *immr = get_immr();
	struct machinfo *p = (void *)0x2000;
	struct nvconfig *nv;
	paddr_t base;
	u_int nvoffset;
	static int speed[] = { 4800, 9600, 19200, 38400, 57600, 115200 };

	busfreq = ((immr->siu_plprcr >> 20) + 1) * 4;	/* 4 MHz xtal */

	base = immr->siu_br0 & 0xffff8000;

	switch (p->board_type) {
	case BOARD_TS3000:
	case BOARD_ACS16:
	case BOARD_ACS32:
	case BOARD_ACS48:
	case BOARD_CS4008:
		nvoffset = 0x30000;
		break;
	default:
		nvoffset = 0x20000;
	}

	nv = search_record((char *)base + nvoffset, 68, 1000);
	if (nv == NULL) {
		printf("Warning: failed to find FEC ethernet address\n");
		return;
	}

	cons_speed = speed[nv->console_speed];
	bcopy(nv->enaddr, fec_enaddr, 6);
}
