/*	$Id: devopen.c,v 1.4 2010/01/07 13:59:15 tsubai Exp $	*/

/*-
 * Copyright (c) 2001,2002 Tsubai Masanari.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <lib/libkern/libkern.h>
#include <lib/libsa/stand.h>
#include <lib/libsa/nfs.h>
#include <lib/libsa/ufs.h>

#include <sys/disklabel.h>

#include "boot.h"

#ifdef DEBUG
# define DPRINTF printf
#else
# define DPRINTF while (0) printf
#endif

int net_open(void *);

int dkopen(struct open_file *, ...);
int dkclose(struct open_file *);
int dkstrategy(void *, int, daddr_t, size_t, void *, size_t *);

struct devsw devsw[] = {
	{ "dk", dkstrategy, dkopen, dkclose, noioctl }
};
int ndevs = sizeof(devsw) / sizeof(devsw[0]);

static struct fs_ops file_system_ufs = FS_OPS(ufs);
static struct fs_ops file_system_nfs = FS_OPS(nfs);

struct fs_ops file_system[3];
int nfsys;

static struct {
	int dummy;
} devdata;

char opened_name[256];

int
devopen(of, name, file)
	struct open_file *of;
	const char *name;
	char **file;
{
	char *cp;
	int error = 0;

	DPRINTF("devopen: name=%s\n", name);

	if (of->f_flags != F_READ)
		return EPERM;

	cp = strchr(name, ':');
	if (cp == NULL)
		return ENOENT;

	strcpy(opened_name, cp + 1);
	*file = opened_name;

	DPRINTF("file=%s\n", opened_name);

	of->f_dev = devsw;
	of->f_devdata = &devdata;

	if (strncmp(name, "net:", 4) == 0) {
		file_system[0] = file_system_nfs;
		nfsys = 1;
		error = net_open(&devdata);
		return error;
	}
	if (strncmp(name, "disk:", 5) == 0) {
		file_system[0] = file_system_ufs;
		nfsys = 1;
		return 0;
	}

	return ENOENT;
}

int
dkopen(struct open_file *f, ...)
{
	DPRINTF("dkopen\n");
	return 0;
}

int
dkclose(f)
	struct open_file *f;
{
	DPRINTF("dkclose\n");
	return 0;
}

int
dkstrategy(devdata, rw, block, size, buf, rsize)
	void *devdata;
	int rw;
	daddr_t block;
	size_t size;
	void *buf;
	size_t *rsize;
{
	int len = size;
	static char wdbuf[512];
	static struct disklabel label;
	static int initted;

	DPRINTF("dkstrategy buf=%p block=%lld, size=%d\n", buf, block, size);

	sswt();

	if (initted == 0) {
		initted = 1;
		wdcread(0, wdbuf);
		getdisklabel(wdbuf, &label);
	}

	block += label.d_partitions[0].p_offset;
	while (len > 0) {
		wdcread(block, wdbuf);
		bcopy(wdbuf, buf, min(sizeof wdbuf, len));
		len -= sizeof wdbuf;
		block++;
		buf = (char *)buf + sizeof wdbuf;
	}
	*rsize = size;
	return 0;
}
