/*	$Id: fec.c,v 1.5 2010/01/07 10:03:11 tsubai Exp $	*/

/*-
 * Copyright (c) 2001,2002 Tsubai Masanari.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <lib/libkern/libkern.h>

#include <lib/libsa/stand.h>
#include <lib/libsa/net.h>
#include <machine/mpc860dev.h>

#define roundup(x, y)	((((x)+((y)-1))/(y))*(y))

#include "boot.h"

/* FEC bd_flags definitions */
#define TxBD_R	 0x8000		/* Ready */
#define TxBD_TO1 0x4000		/* Transmit ownership (software) */
#define TxBD_W	 0x2000		/* Wrap */
#define TxBD_TO2 0x1000		/* Transmit ownership (software) */
#define TxBD_L	 0x0800		/* Last in frame */
#define TxBD_TC	 0x0400		/* Add CRC */
#define TxBD_DEF 0x0200		/* Defer indication */
#define TxBD_HB	 0x0100		/* Heartbeat error */
#define TxBD_LC	 0x0080		/* Late collision */
#define TxBD_RL	 0x0040		/* Retransmission limit */
#define TxBD_RC	 0x003c		/* Retry count */
#define TxBD_UN	 0x0002		/* Underrun */
#define TxBD_CSL 0x0001		/* Carrier sense lost */

#define TxBD_BITMASK "\20" \
    "\20R\17TO1\16W\15TO2\14L\13TC\12DEF\11HB\10LC\07RL\02UN\01CSL"

#define RxBD_E	 0x8000		/* Empty */
#define RxBD_RO1 0x4000		/* Receive ownership (software) */
#define RxBD_W	 0x2000		/* Wrap */
#define RxBD_RO2 0x1000		/* Receive ownership (software) */
#define RxBD_L	 0x0800		/* Last in frame */
#define RxBD_M	 0x0100		/* Miss */
#define RxBD_BC	 0x0080		/* Broadcast frame */
#define RxBD_MC	 0x0040		/* Multicast frame */
#define RxBD_LG	 0x0020		/* Frame length violation */
#define RxBD_NO	 0x0010		/* Nonoctet-aligned frame */
#define RxBD_SH	 0x0008		/* Short frame */
#define RxBD_CR	 0x0004		/* CRC error */
#define RxBD_OV	 0x0002		/* Overrun */
#define RxBD_TR	 0x0001		/* Truncate */

#define RxBD_BITMASK "\20" \
    "\20E\17RO1\16W\15RO2\14L\11M\10BC\07MC\06LG\05NO\04SH\03CR\02OV\01TR"

#define FEC_ECNTRL_PINMUX	0x00000004	/* FEC enable */
#define FEC_ECNTRL_ETHER_EN	0x00000002	/* Ethernet enable */
#define FEC_ECNTRL_RESET	0x00000001

#define FEC_RCNTRL_BC_REJ	0x00000010	/* Broadcast frame reject */
#define FEC_RCNTRL_PROM		0x00000008	/* Promiscuous mode */
#define FEC_RCNTRL_MII_MODE	0x00000004	/* MII mode */
#define FEC_RCNTRL_DRT		0x00000002	/* Disable recv on transmit */
#define FEC_RCNTRL_LOOP		0x00000001	/* Loopback */

#define FEC_XCNTRL_FDEN		0x00000004	/* Full duplex */
#define FEC_XCNTRL_HBC		0x00000002	/* Heartbeat control */
#define FEC_XCNTRL_GTS		0x00000001	/* Graceful trasmit stop */

u_char fec_enaddr[6];

void
fecinit()
{
	volatile struct mpc860dev *immr = get_immr();
	struct cpm_bdesc *TxBD, *RxBD;
	char *buf;
	u_char *ea = fec_enaddr;

	/* Disable ethernet. */
	immr->fec_ecntrl = FEC_ECNTRL_PINMUX;

	/* Reset ethernet. */
	immr->fec_ecntrl = FEC_ECNTRL_PINMUX | FEC_ECNTRL_RESET;
	delay(10);
	while (immr->fec_ecntrl & FEC_ECNTRL_RESET); /* Wait for reset done. */

	TxBD = immr->fec_x_des_start;
	RxBD = immr->fec_r_des_start;

	TxBD[0].bd_flags = 0;
	TxBD[1].bd_flags = TxBD_W;

	RxBD[0].bd_flags = RxBD_E;
	RxBD[1].bd_flags = RxBD_E|RxBD_W;

	immr->fec_hash_table_high = 0;
	immr->fec_hash_table_low = 0;
	immr->fec_r_buff_size = 0x5f0;

	immr->fec_r_cntrl = FEC_RCNTRL_MII_MODE;
	immr->fec_x_cntrl = FEC_XCNTRL_FDEN;

	immr->fec_fun_code = 0x7f000000;

	immr->fec_ecntrl = FEC_ECNTRL_PINMUX;
	eieio();

	/* Enable ethernet */
	immr->fec_ecntrl = FEC_ECNTRL_PINMUX | FEC_ECNTRL_ETHER_EN;

	/* Enable receiver. */
	immr->fec_r_des_active = 0x100;
	eieio();

return;


	buf = alloc(sizeof(struct cpm_bdesc) * 4 + 16);
	TxBD = (void *)roundup((u_int)buf, 16);

	buf = alloc(0x800 * 4 + 16);
	buf = (void *)roundup((u_int)buf, 16);
	TxBD[0].bd_flags = 0;
	TxBD[0].bd_addr = buf;
	buf += 0x800;
	TxBD[1].bd_flags = 0;
	TxBD[1].bd_addr = buf;

	buf += 0x800;
	RxBD = &TxBD[2];
	RxBD[0].bd_flags = RxBD_E;
	RxBD[0].bd_addr = buf;
	buf += 0x800;
	RxBD[1].bd_flags = RxBD_E|RxBD_W;
	RxBD[1].bd_addr = buf;

	immr->fec_imask = 0;			/* Mask all interrupts. */
	immr->fec_ievent = 0xffff;		/* Clear all events. */
	immr->fec_ivec = 0;
	immr->fec_addr_high = ea[0] << 24 | ea[1] << 16 | ea[2] << 8 | ea[3];
	immr->fec_addr_low  = ea[4] << 24 | ea[5] << 16;
	immr->fec_hash_table_high = 0;
	immr->fec_hash_table_low = 0;
	immr->fec_r_buff_size = 0x5f0;
	immr->fec_r_des_start = RxBD;
	immr->fec_x_des_start = TxBD;
	immr->fec_r_cntrl = 0x6;		/* XXX half-duplex */
	immr->fec_x_cntrl = 0x0;
	immr->fec_fun_code = 0x7f000000;

	immr->cpm_pdpar = 0x1fff;
	immr->cpm_pddir = 0x1fff;
	eieio();

	/* XXX Should we reset FEC here? */

	immr->fec_ecntrl = 0x04 | 0x02;		/* Enable ethernet */
	immr->fec_r_des_active = 0x100;		/* Enable receiver. */
	eieio();
}

int
fecsend(buf, len)
	char *buf;
	int len;
{
	volatile struct mpc860dev *immr = get_immr();
	volatile static struct cpm_bdesc *TxBD;

	if (len > 1520) {
		printf("fecsend: len > 1520 (%d)\n", len);
		len = 1520;
	}

	if (TxBD == NULL)
		TxBD = immr->fec_x_des_start;

	/* Wait for transmission done. */
	while (TxBD->bd_flags & TxBD_R)
		sswt();

	bcopy(buf, TxBD->bd_addr, len);
	if (TxBD == immr->fec_x_des_start) {
		TxBD->bd_flags = TxBD_R|TxBD_L|TxBD_TC;
		TxBD->bd_len = len;
		TxBD++;
		TxBD->bd_flags = TxBD_W;
	} else {
		TxBD->bd_flags = TxBD_R|TxBD_L|TxBD_TC|TxBD_W;
		TxBD->bd_len = len;
		TxBD = immr->fec_x_des_start;
		TxBD->bd_flags = 0;
	}

	eieio();

	/* Start transmission. */
	immr->fec_x_des_active = 0x0100;
	eieio();

	return len;
}

int
fecrecv(buf, timo)
	char *buf;
	int timo;
{
	volatile struct mpc860dev *immr = get_immr();
	volatile struct cpm_bdesc *RxBD;
	u_int flags;
	int len, t0;

	timo++;
	t0 = getsecs();

start:
	RxBD = immr->fec_r_des_start;
	for (;;) {
		flags = RxBD->bd_flags;
		if ((flags & RxBD_E) == 0)
			goto found;
		if (flags & RxBD_W)
			break;
		RxBD++;
	}
	if (getsecs() - t0 > timo)
		return -1;
	sswt();
	goto start;

found:
	flags &= RxBD_W;
	flags |= RxBD_E;

	len = RxBD->bd_len;
	if (len > 1520) {
		printf("fecrecv: len > 1520 (%d)\n", len);
		RxBD->bd_flags = flags;
		return -1;
	}
	bcopy(RxBD->bd_addr, buf, len);
	eieio();

	RxBD->bd_flags = flags;
	immr->fec_r_des_active = 0x100;		/* XXX needed? */

	return len;
}
