/*	$Id: netif.c,v 1.5 2012/06/01 12:57:31 tsubai Exp $	*/

/*-
 * Copyright (c) 2001 Tsubai Masanari.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <lib/libkern/libkern.h>
#include <lib/libsa/stand.h>
#include <lib/libsa/net.h>

#include <net/if_ether.h>

#include "boot.h"

#ifdef NETIF_DEBUG
# define DPRINTF printf
#else
# define DPRINTF while (0) printf
#endif

int fecsend(char *, int);
int fecrecv(char *, int);

struct iodesc sockets[SOPEN_MAX];

#ifdef NETIF_DEBUG
static void
print_packet(pkt, len)
	void *pkt;
	int len;
{
	struct ether_header *eh = pkt;

	printf("src: %s ", ether_sprintf(eh->ether_shost));
	printf("dst: %s ", ether_sprintf(eh->ether_dhost));
	printf("type: 0x%x ", eh->ether_type & 0xffff);
	printf("len: %d\n", len);
}
#endif

struct iodesc *
socktodesc(sock)
	int sock;
{
	if (sock != 0)
		return NULL;
	return sockets;
}

int
netif_open(machdep_hint)
	void *machdep_hint;
{
	struct iodesc *io;

	/* find a free socket */
	io = sockets;
	if (io->io_netif) {
		errno = ENFILE;
		return -1;
	}
	bzero(io, sizeof *io);

	io->io_netif = machdep_hint;

	/* Put our ethernet address in io->myea */
	bcopy(fec_enaddr, io->myea, 6);

	return 0;
}

int
netif_close(fd)
	int fd;
{
	struct iodesc *io;

	if (fd != 0) {
		errno = EBADF;
		return -1;
	}

	io = sockets;
	io->io_netif = NULL;

	return 0;
}

/*
 * Send a packet.  The ether header is already there.
 * Return the length sent (or -1 on error).
 */
ssize_t
netif_put(desc, pkt, len)
	struct iodesc *desc;
	void *pkt;
	size_t len;
{
	ssize_t rv;
	size_t sendlen;

#ifdef NETIF_DEBUG
	printf("netif_put: ");
	print_packet(pkt, len);
#endif
	sendlen = len;
	if (sendlen < 60)
		sendlen = 60;

	rv = fecsend(pkt, sendlen);

	return rv;
}

/*
 * Receive a packet, including the ether header.
 * Return the total length received (or -1 on error).
 */
ssize_t
netif_get(desc, pkt, maxlen, timo)
	struct iodesc *desc;
	void *pkt;
	size_t maxlen;
	saseconds_t timo;
{
	int len;

	DPRINTF("netif_get: maxlen = %d, timo = %d\n", maxlen, timo);

	len = fecrecv(pkt, timo);
	if (len == -1) {
		printf("timeout\n");
		/* XXX errno = ... */
	}

	if (len < 12)
		return -1;

#ifdef NETIF_DEBUG
	printf("netif_get: ");
	print_packet(pkt, len);
#endif

	return len;
}
