/*	$Id: smc.c,v 1.8 2010/01/07 10:03:11 tsubai Exp $	*/

/*-
 * Copyright (c) 2009 Tsubai Masanari.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <lib/libkern/libkern.h>
#include <lib/libsa/stand.h>

#include <machine/mpc860dev.h>

#include "boot.h"

#define TxBD_R 0x8000
#define TxBD_W 0x2000
#define RxBD_E 0x8000
#define RxBD_W 0x2000

/*
 * TS3000  DPRAM ϰʲΰȤƤ:
 *   2800 90 00 00 00 00 7e 09 90  b0 00 00 00 00 7e 09 98
 *   2810 90 00 00 00 00 7e 09 a0  90 00 00 00 00 7e 09 a8
 *   2820 90 00 00 00 00 7e 09 b0  90 00 00 00 00 7e 09 b8
 *   2830 90 00 00 00 00 7e 09 c0  b0 00 00 00 00 7e 09 c8
 *   2840 30 00 00 00 00 7e 09 60  00 00 00 00 00 00 00 00
 *
 *   2800 - 283f: RxBD[8]
 *   2840 - 2847: TxBD[1] (bd_addr ϽϤˤ¿Ѳ)
 */

void
smcinit()
{
	volatile struct mpc860dev *immr = get_immr();
	volatile struct smc_uart *uart = (void *)&immr->cpm_dpram[DPRAM_SMC1];
	volatile struct cpm_bdesc *bd;
	int i, flags;

	/*
	 * bd_addr  DPRAM ѹ롣
	 * ɬܤǤϤʤˤꥭå椬פˤʤ롣
	 */
	bd = (void *)&immr->cpm_dpram[uart->tbase];
	bd[0].bd_addr = (char *)0xff002850;

	bd = (void *)&immr->cpm_dpram[uart->rbase];
	for (i = 0; i < 32; i++) {
		flags = bd[i].bd_flags;
		bd[i].bd_addr = (char *)0xff002860 + i * 16;
		if (flags & RxBD_W)
			break;
	}
}

void
putchar(c)
	int c;
{
	volatile struct mpc860dev *immr = get_immr();
	volatile struct smc_uart *uart = (void *)&immr->cpm_dpram[DPRAM_SMC1];
	volatile struct cpm_bdesc *bd = (void *)&immr->cpm_dpram[uart->tbase];

	if (c == '\n')
		putchar('\r');

	while (bd->bd_flags & TxBD_R);
	eieio();

	*bd->bd_addr = c;
	bd->bd_len = 1;

	eieio();
	bd->bd_flags |= TxBD_R | TxBD_W;
}

int
getkey()
{
	volatile struct mpc860dev *immr = get_immr();
	volatile struct smc_uart *uart = (void *)&immr->cpm_dpram[DPRAM_SMC1];
	volatile struct cpm_bdesc *bd = (void *)&immr->cpm_dpram[uart->rbase];
	int i, flags;
	int c = 0;

	for (i = 0; i < 32; i++) {
		flags = bd->bd_flags;
		if ((flags & RxBD_E) == 0) {
			c = *bd->bd_addr;
			bd->bd_flags = flags | RxBD_E;
		}
		if (flags & RxBD_W)
			break;
		bd++;
	}
	return c;
}

int
getchar()
{
	volatile struct mpc860dev *immr = get_immr();
	volatile struct smc_uart *uart = (void *)&immr->cpm_dpram[DPRAM_SMC1];
	int c;

	uart->mrblr = 1;
	uart->max_idl = 0;

	sswt();
	for (;;) {
		c = getkey();
		if (c)
			return c;
	}
}
