/*	$Id: utils.c,v 1.3 2010/01/07 09:41:28 tsubai Exp $	*/

/*-
 * Copyright (c) 2001,2002 Tsubai Masanari.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <lib/libkern/libkern.h>
#include <lib/libsa/stand.h>
#include <lib/libsa/net.h>

#include <machine/cpu.h>
#include <machine/cpufunc.h>
#include <machine/sysconf.h>

#include "boot.h"

static int
getmsec()
{
	u_int64_t tb;

	tb = mftb();
	return tb / (TIMEBASE_FREQ / 4 / 1000);
}

satime_t
getsecs()
{
	return getmsec() / 1000;
}

int
count_down()
{
	int count, now;

	printf("(type SPACE to halt)... ");
	for (count = 5; count > 0; count--) {
		sswt();
		printf("\10%d", count);
		now = getmsec();
		while (getmsec() - now < 1000) {
			switch (getkey()) {
			case '\r':
				goto out;
			case ' ':
				printf("\n");
				return 1;
			}
		}
	}
out:
	printf("\10 \n");
	return 0;
}

void
delay(n)
	u_int n;
{
	const int ns_per_tick = 1000000000 / (TIMEBASE_FREQ / 4);
	u_quad_t tb;
	u_long tbh, tbl, scratch;

	tb = mftb();
	tb += (n * 1000 + ns_per_tick - 1) / ns_per_tick;
	tbh = tb >> 32;
	tbl = tb;
	asm volatile ("1: mftbu %0; cmplw %0,%1; blt 1b; bgt 2f;"
		      "mftb %0; cmplw %0,%2; blt 1b; 2:"
		      : "=r"(scratch) : "r"(tbh), "r"(tbl));
}

void
syncicache(from, len)
	void *from;
	int len;
{
	int l, off;
	char *p;

	off = (u_int)from & (CACHELINESIZE - 1);
	len += off;

	p = from;
	l = len;
	do {
		asm volatile ("dcbst 0,%0" :: "r"(p));
		p += CACHELINESIZE;
	} while ((l -= CACHELINESIZE) > 0);
	asm volatile ("sync");

	p = from;
	l = len;
	do {
		asm volatile ("icbi 0,%0" :: "r"(p));
		p += CACHELINESIZE;
	} while ((l -= CACHELINESIZE) > 0);
	asm volatile ("sync");	/* required on 745x */
	asm volatile ("isync");
}

__dead void
_rtt()
{
	for (;;);
}
