/*	$Id: wdc.c,v 1.2 2010/01/07 22:43:34 tsubai Exp $	*/

/*-
 * Copyright (c) 2001,2002 Tsubai Masanari.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <lib/libkern/libkern.h>

#include <machine/pio.h>

#include "boot.h"

/*
 * IDE Υ쥸 0x40040000 ȡ0x4004001d
 * DATA ʳϳƴɥ쥹¸ߤ롣
 * DATA 쥸Ͼ岼ХȤդˤĤʤäƤΤա
 */
#define cmdreg 0x40040000

#define DATA	0	/* data */
#define ERROR	3	/* features(w) / error(r) */
#define SECCNT	5	/* sector count */
#define SECNUM	7	/* sector number */
#define CYL_L	9	/* cylinder low */
#define CYL_H	11	/* cylinder high */
#define DEVHEAD	13	/* device / head */
#define COMMAND	15	/* command(w) / status(r) */
#define STATUS	15

#define CONTROL	29	/* device control(w) / alt status(r) */

#define BSY 0x80
#define DRQ 0x08

#define DEV 0x10
#define LBA 0x40

static inline u_int
wdread(off)
	u_int off;
{
	if (off == DATA)
		return in16rb(cmdreg + DATA);
	else
		return in8(cmdreg + off);
}

static inline void
wdwrite(off, val)
	u_int off, val;
{
	if (off == DATA)
		out16rb(cmdreg + DATA, val);
	else
		out8(cmdreg + off, val);
}

int
wdcread(lba, buf)
	u_int lba;
	void *buf;
{
	u_int16_t *p = buf;
	int i;

	/*
	 * Device selection
	 */
	while (wdread(STATUS) & (BSY|DRQ));

	wdwrite(DEVHEAD, LBA);
	delay(1);

	while (wdread(STATUS) & (BSY|DRQ));

	/*
	 * Read one sector.
	 */
	wdwrite(CONTROL, 0x02);
	wdwrite(SECCNT, 1);
	wdwrite(SECNUM, lba);
	wdwrite(CYL_L, lba >> 8);
	wdwrite(CYL_H, lba >> 16);
	wdwrite(DEVHEAD, 0x40 | 0x00 | (lba >> 24 & 0x0f));
	wdwrite(COMMAND, 0x20);
	delay(1);

	wdread(CONTROL);

	while ((wdread(STATUS) & (BSY|DRQ)) != DRQ);

	for (i = 0; i < 256; i++)
		*p++ = wdread(DATA);

	wdread(CONTROL);

	return wdread(STATUS);		/* status & (BSY|DF|ERR) */
}
