/*	$Id: bootxx.c,v 1.1 2010/02/08 13:27:30 tsubai Exp $	*/

/*-
 * Copyright (c) 2010 Tsubai Masanari.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <lib/libkern/libkern.h>
#include <lib/libsa/stand.h>
#include <machine/sysconf.h>

#include "boot.h"

#define MAXBLOCKNUM  16
#define LOAD_ADDRESS (void *)_end

void startup(char *);
void load(void);
static int mftb(void);
extern char _start[], _edata[], _end[];

u_int block_size = 1;
u_int block_table[MAXBLOCKNUM] = { 1 };

void
startup(start)
	char *start;
{
	bcopy(start, _start , _edata - _start);
	bzero(_edata, _end - _edata);

	/* Branch to relocated main. */
	asm volatile ("mtctr %0; bctr" :: "r"(load));
}

struct tsheader {
	u_char magic[16];
	u_int32_t size;
	u_int32_t size2;	/* unused? */
	u_int16_t crc;
	u_int16_t id;
	u_int32_t entry;
};

void
load()
{
	struct tsheader *tsh = LOAD_ADDRESS;
	char *addr = LOAD_ADDRESS;
	u_int bn, i, *p;

	for (p = block_table; *p; p++) {
		bn = *p;
		for (i = block_size / 512; i > 0; i--) {
			wdcread(bn, addr);
			bn++;
			addr += 512;
		}
	}

	bcopy(LOAD_ADDRESS, (char *)tsh->entry + 0x10000, tsh->size);
	asm volatile ("mtctr %0; bctr" :: "r"(tsh->entry + 0x10020));

	for (;;);	/* just in case */
}

static inline int
mftb()
{
	int tb;

	asm volatile ("mftb %0" : "=r"(tb));
	return tb;
}

void
delay(n)
	u_int n;
{
	int tb;

	tb = mftb();
	tb += (TIMEBASE_FREQ / 4 / 1000) * n / 1000;
	while (mftb() - tb < 0);
}

void
bcopy(src, dst, len)
	const void *src;
	void *dst;
	size_t len;
{
	const char *csrc = src;
	char *cdst = dst;

	while (len-- > 0)
		*cdst++ = *csrc++;
}

void
bzero(addr, len)
	void *addr;
	size_t len;
{
	char *p = addr;

	while (len-- > 0)
		*p++ = 0;
}
