/*	$Id: crc.c,v 1.4 2010/02/08 13:38:01 tsubai Exp $	*/

/*-
 * Copyright (c) 2009 Tsubai Masanari.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/types.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <err.h>
#include <fcntl.h>
#include <string.h>

#include "extern.h"

u_int calc_crc(u_char *, u_int);

u_int
calc_crc(p, len)
	u_char *p;
	u_int len;
{
	u_int crc = 0;
	int i;

	while (len--) {
		crc ^= *p++ << 24;
		for (i = 0; i < 8; i++) {
			if (crc & 0x80000000)
				crc ^= 0x88108000;
			crc <<= 1;
		}
	}
	return crc >> 16;
}

struct tsheader {
	u_char magic[16];
	u_int32_t size;
	u_int32_t size2;	/* unused? */
	u_int16_t crc;
	u_int16_t id;
	u_int32_t entry;
};

void
addcrc(name)
	char *name;
{
	int fd;
	u_int size;
	char *p;
	struct stat sb;

	fd = open(name, 2);
	if (fd < 0)
		err(1, "open %s", name);

	if (fstat(fd, &sb) < 0)
		err(1, "fstat");
	size = sb.st_size;

	p = mmap(0, size, PROT_READ|PROT_WRITE, MAP_SHARED, fd, 0);
	if (p == MAP_FAILED)
		err(1, "mmap");

	fillmagic(p, size);
}

void
fillmagic(p, size)
	char *p;
	u_int size;
{
	struct tsheader *h;

	h = (void *)p;
	h->magic[0] = 0;
	h->magic[1] = 0;
	h->magic[2] = 0x21;
	h->magic[3] = 0x20;
	memcpy(h->magic + 4, "Entry Point\0", 12);

	h->size = size;
	h->crc = calc_crc(p + 32, size - 32);
	h->id = 0x316b;
}
