/*	$Id: bus_space.c,v 1.6 2010/01/23 09:13:51 tsubai Exp $	*/

/*-
 * Copyright (c) 2002 Tsubai Masanari.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/param.h>
#include <sys/systm.h>

#include <machine/bus.h>
#include <machine/pio.h>

int
bus_space_map(t, addr, size, flags, hp)
	bus_space_tag_t t;
	bus_addr_t addr;
	bus_size_t size;
	int flags;
	bus_space_handle_t *hp;
{
	panic("bus_space_map");
}

void
bus_space_unmap(t, h, size)
	bus_space_tag_t t;
	bus_space_handle_t h;
	bus_size_t size;
{
}

u_int8_t
bus_space_read_1(t, h, off)
	bus_space_tag_t t;
	bus_space_handle_t h;
	bus_size_t off;
{
	return in8(h + off);
}

/* XXX IDE only */
u_int16_t
bus_space_read_2(t, h, off)
	bus_space_tag_t t;
	bus_space_handle_t h;
	bus_size_t off;
{
	return in16(h + off);
}

/* XXX IDE only */
void
bus_space_read_multi_2(t, h, off, addr, count)
	bus_space_tag_t t;
	bus_space_handle_t h;
	bus_size_t off;
	u_int16_t *addr;
	bus_size_t count;
{
	volatile u_int16_t *s = (void *)(h + off);

	while (count-- > 0)
		*addr++ = *s;
	asm volatile ("eieio");
}

/* XXX IDE only */
void
bus_space_read_multi_stream_2(t, h, off, addr, count)
	bus_space_tag_t t;
	bus_space_handle_t h;
	bus_size_t off;
	u_int16_t *addr;
	bus_size_t count;
{
	volatile u_int16_t *s = (void *)(h + off);

	while (count-- > 0)
		asm volatile ("lhbrx %0,0,%1" : "=r"(*addr++) : "r"(s));
	asm volatile ("eieio");
}

void
bus_space_read_multi_4(t, h, off, addr, count)
	bus_space_tag_t t;
	bus_space_handle_t h;
	bus_size_t off;
	u_int32_t *addr;
	bus_size_t count;
{
	panic("bus_space_read_multi_4");
}

void
bus_space_read_multi_stream_4(t, h, off, addr, count)
	bus_space_tag_t t;
	bus_space_handle_t h;
	bus_size_t off;
	u_int32_t *addr;
	bus_size_t count;
{
	panic("bus_space_read_multi_stream_4");
}

void
bus_space_write_1(t, h, off, val)
	bus_space_tag_t t;
	bus_space_handle_t h;
	bus_size_t off;
	u_int8_t val;
{
	out8(h + off, val);
}

/* XXX IDE only */
void
bus_space_write_2(t, h, off, val)
	bus_space_tag_t t;
	bus_space_handle_t h;
	bus_size_t off;
	u_int16_t val;
{
	out16(h + off, val);
}

void
bus_space_write_multi_1(t, h, off, addr, count)
	bus_space_tag_t t;
	bus_space_handle_t h;
	bus_size_t off;
	const u_int8_t *addr;
	bus_size_t count;

{
	volatile u_int8_t *d = (void *)(h + off);

	while (count--)
		*d = *addr++;
	asm volatile ("eieio");
}

/* XXX IDE only */
void
bus_space_write_multi_2(t, h, off, addr, count)
	bus_space_tag_t t;
	bus_space_handle_t h;
	bus_size_t off;
	const u_int16_t *addr;
	bus_size_t count;
{
	volatile u_int16_t *d = (void *)(h + off);

	while (count-- > 0)
		*d = *addr++;
	asm volatile ("eieio");
}

/* XXX IDE only */
void
bus_space_write_multi_stream_2(t, h, off, addr, count)
	bus_space_tag_t t;
	bus_space_handle_t h;
	bus_size_t off;
	const u_int16_t *addr;
	bus_size_t count;
{
	volatile u_int16_t *d = (void *)(h + off);

	while (count-- > 0)
		asm volatile ("sthbrx %0,0,%1" :: "r"(*addr++), "r"(d));
	asm volatile ("eieio");
}

void
bus_space_write_multi_4(t, h, off, addr, count)
	bus_space_tag_t t;
	bus_space_handle_t h;
	bus_size_t off;
	const u_int32_t *addr;
	bus_size_t count;
{
	panic("bus_space_write_multi_4");
}

void
bus_space_write_multi_stream_4(t, h, off, addr, count)
	bus_space_tag_t t;
	bus_space_handle_t h;
	bus_size_t off;
	const u_int32_t *addr;
	bus_size_t count;
{
	panic("bus_space_write_multi_stream_4");
}
