/*	$Id: mainbus.c,v 1.4 2015/06/23 19:31:58 tsubai Exp $	*/

/*-
 * Copyright (c) 2001,2002 Tsubai Masanari.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/param.h>
#include <sys/device.h>
#include <sys/systm.h>

#include <machine/autoconf.h>
#include <ts3000/board.h>

int mainbus_match(struct device *, struct cfdata *, void *);
void mainbus_attach(struct device *, struct device *, void *);
static char *board_name(void);

CFDECL(mainbus, sizeof(struct device), mainbus_match, mainbus_attach);

int
mainbus_match(parent, cf, aux)
	struct device *parent;
	struct cfdata *cf;
	void *aux;
{
	return 1;
}

/*
 * Attach the mainbus.
 */
void
mainbus_attach(parent, self, aux)
	struct device *parent, *self;
	void *aux;
{
	struct confargs ca;

	printf(": %s\n", board_name());

	ca.ca_name = "cpu";
	ca.ca_addr = 0;
	config_found(self, &ca, NULL);

	ca.ca_name = "cpm";
	ca.ca_addr = 0;
	config_found(self, &ca, NULL);

	ca.ca_name = "obio";
	ca.ca_addr = 0;
	config_found(self, &ca, NULL);
}

static struct {
	char type;
	char name[7];
} names[] = {
	{ BOARD_TS1000, "TS1000" },
	{ BOARD_TS2000, "TS2000" },
	{ BOARD_TS400,  "TS400" },
	{ BOARD_TS800,  "TS800" },
	{ BOARD_TS3000, "TS3000" },
	{ BOARD_ACS16,  "ACS16" },
	{ BOARD_ACS32,  "ACS32" },
	{ BOARD_ACS48,  "ACS48" },
	{ BOARD_CS4008, "CS4008" },
	{ 0 }
};

char *
board_name()
{
	int i = 0;
	static char name[16];

	while (names[i].type) {
		if (names[i].type == board_type)
			return names[i].name;
		i++;
	}
	snprintf(name, sizeof name, "ID %u", board_type);
	return name;
}
